/** 
 *******************************************************************************
 * @file    txz_uart_mdma.h
 * @brief   This file provides all the functions prototypes for UART driver.
 * @brief   If MDMA transfer is used, it is necessary.
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2020
 *******************************************************************************
 */
/*------------------------------------------------------------------------------*/
/*  Define to prevent recursive inclusion                                       */
/*------------------------------------------------------------------------------*/
#ifndef __UART_MDMA_H
#define __UART_MDMA_H

#ifdef __cplusplus
extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "txz_driver_def.h"
#include "txz_uart.h"
#include "txz_uart_include.h"
#include "txz_mdma.h"

/**
 *  @addtogroup Periph_Driver
 *  @{
 */

/** 
 *  @addtogroup UART
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup UART_Exported_define
 *  @{
 */
/** 
 *  @defgroup UART_MDMA_TransferType  Transfer Type
 *  @brief    MDMA transfer type.
 *  @{
 */
#define UART_MDMA_TYPE_UNUSE        ((uint32_t)0x00)    /*!< MDMA UNUSE    */
#define UART_MDMA_TYPE_AREA_TO_AREA ((uint32_t)0x01)    /*!< MDMA AREA to AREA */
#define UART_MDMA_TYPE_FIX_TO_AREA  ((uint32_t)0x02)    /*!< MDMA FIX to AREA */
#define UART_MDMA_TYPE_AREA_TO_FIX  ((uint32_t)0x03)    /*!< MDMA AREA to FIX */
#define UART_MDMA_TYPE_FIX_TO_FIX   ((uint32_t)0x04)    /*!< MDMA FIX to AREA */
/**
 *  @}
 */ /* End of group UART_MDMA_TransferType */
/** 
 *  @defgroup UART_MDMA_TransferType  Transfer Type
 *  @brief    MDMA transfer type.
 *  @{
 */
#define UART_MDMA_UART_CH0          ((uint32_t)0x00)    /*!< MDMA UART CH0    */
#define UART_MDMA_UART_CH1          ((uint32_t)0x01)    /*!< MDMA UART CH1    */
#define UART_MDMA_UART_CH2          ((uint32_t)0x02)    /*!< MDMA UART CH2    */
#define UART_MDMA_UART_CH3          ((uint32_t)0x03)    /*!< MDMA UART CH3    */
#define UART_MDMA_UART_CH4          ((uint32_t)0x04)    /*!< MDMA UART CH4    */
#define UART_MDMA_UART_CH5          ((uint32_t)0x05)    /*!< MDMA UART CH5    */
/**
 *  @}
 */ /* End of group UART_MDMA_TransferType */

/**
 *  @}
 */ /* End of group UART_Exported_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup UART_Exported_define
 *  @{
 */
/* UART connect to MDMA */
#if defined(TMPM4GR)
#define CH0 TSB_UART0
#define CH1 TSB_UART1
#define CH2 TSB_UART2
#define CH3 TSB_UART3
#define CH4 TSB_UART4
#define CH5 TSB_UART5


#elif defined(TMPM4GQ)
#define CH0 TSB_UART0
#define CH1 TSB_UART1
#define CH2 TSB_UART2
#define CH3 TSB_UART3
#define CH4 TSB_UART4
#define CH5 0
    
#elif defined(TMPM4GN)
#define CH0 TSB_UART0
#define CH1 TSB_UART1
#define CH2 TSB_UART2
#define CH3 0
#define CH4 0
#define CH5 0
    
#else
   #error "target device is non-select."
#endif

/**
 *  @}
 */ /* End of group UART_Exported_define */


/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup UART_Exported_typedef
 *  @{
 */

/*----------------------------------*/
/** 
 * @brief  MDMA setting structure definition.
*/
/*----------------------------------*/
typedef struct
{
    uint32_t type;      /*!< MDMA transfer type. 
                            : Use @ref UART_MDMA_TransferType          */
    uint32_t ch;        /*!< MDMA uart channel.  
                            : If MDMA isn't used, it isn't necessary.  */
} uart_mdma_setting_t;

/*----------------------------------*/
/** 
 * @brief  UART MDMA initial setting structure definition.
*/
/*----------------------------------*/
typedef struct
{
    mdma_t              *p_obj;  /*!< MDMA object address.  */
    uart_mdma_setting_t tx;      /*!< Transmit setting.    */
    uart_mdma_setting_t rx;      /*!< Receive setting.     */
} uart_mdma_initial_setting_t;




/*----------------------------------*/
/** 
 * @brief  UART MDMA handle structure definition.
*/
/*----------------------------------*/
typedef struct
{
    uart_t super;                       /*!< Uart super class. 
                                            : Use @ref uart_t                       */
    uart_mdma_initial_setting_t init;    /*!< Initial setting.
                                            : Use @ref uart_mdma_initial_setting_t   */
} uart_mdma_t;


/**
 *  @}
 */ /* End of group UART_Exported_typedef */


/*------------------------------------------------------------------------------*/
/*  Functions                                                                   */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup UART_Exported_functions
 *  @{
 */
TXZ_Result uart_mdma_init(uart_mdma_t *p_obj);
TXZ_Result uart_mdma_deinit(uart_mdma_t *p_obj);
TXZ_Result uart_mdma_discard_transmit(uart_mdma_t *p_obj);
TXZ_Result uart_mdma_discard_receive(uart_mdma_t *p_obj);
TXZ_Result uart_mdma_transmitIt(uart_mdma_t *p_obj, uart_transmit_t *p_info);
TXZ_Result uart_mdma_receiveIt(uart_mdma_t *p_obj, uart_receive_t *p_info);
TXZ_Result uart_mdma_get_uart_ch_to_mdma_ch(uint32_t uart_ch, uint32_t *tx_ch, uint32_t *rx_ch);
void uart_mdma_TXE_disable(uart_mdma_t *p_obj);
void uart_mdma_TXE_enable(uart_mdma_t *p_obj);
void uart_mdma_RXE_disable(uart_mdma_t *p_obj);
void uart_mdma_RXE_enable(uart_mdma_t *p_obj);
/**
 *  @}
 */ /* End of group UART_Exported_functions */

/**
 *  @}
 */ /* End of group UART */

/**
 *  @}
 */ /* End of group Periph_Driver */

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* __UART_DMA_H */


