/**
 *******************************************************************************
 * @file    txz_tspi_hdma.c
 * @brief   This file provides API functions for TSPI driver.
 * @brief   If DMA transfer is used, it is necessary.
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2020
 *******************************************************************************
 */

#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "txz_tspi_hdma_gx.h"

#if defined(__TSPI_HDMA_H)

/**
 *  @addtogroup Periph_Driver
 *  @{
 */

/** 
 *  @addtogroup TSPI
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup TSPI_Private_define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group TSPI_Private_define */

/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup TSPI_Private_define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group TSPI_Private_define */

/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup TSPI_Private_define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group TSPI_Private_define */

/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup TSPI_Private_typedef
 *  @{
 */

/* no define */

/**
 *  @}
 */ /* End of group TSPI_Private_typedef */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup TSPI_Private_fuctions
 *  @{
 */
#ifdef __DEBUG__
    __STATIC_INLINE int32_t check_param_tx_type(uint32_t param);
    __STATIC_INLINE int32_t check_param_rx_type(uint32_t param);
#endif /* #ifdef __DEBUG__ */
static void tx_transfer_done(uint32_t id, TXZ_Result result);
static void rx_transfer_done(uint32_t id, TXZ_Result result);

#ifdef __DEBUG__
/*--------------------------------------------------*/
/** 
  * @brief  Check the Tx DMA transfer type's parameter.
  * @param  param    :Tx DMA transfer type's parameter
  * @retval TSPI_PARAM_OK :Valid
  * @retval TSPI_PARAM_NG :Invalid
  * @note   Macro definition is @ref TSPI_HDMA_TransferType"TSPI_HDMA_TYPE_xxxx".
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_tx_type(uint32_t param)
{
    int32_t result = TSPI_PARAM_NG;

    switch (param) 
    {
    case TSPI_HDMA_TYPE_UNUSE:
    case TSPI_HDMA_TYPE_TX_USE:
        result = TSPI_PARAM_OK;
        break;
    default:
        /* no process */
        break;
    }

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Check the Rx DMA transfer type's parameter.
  * @param  param    :Rx DMA transfer type's parameter
  * @retval TSPI_PARAM_OK :Valid
  * @retval TSPI_PARAM_NG :Invalid
  * @note   Macro definition is @ref TSPI_HDMA_TransferType"TSPI_HDMA_TYPE_xxxx".
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_rx_type(uint32_t param)
{
    int32_t result = TSPI_PARAM_NG;

    switch (param) 
    {
    case TSPI_HDMA_TYPE_UNUSE:
    case TSPI_HDMA_TYPE_RX_USE:
        result = TSPI_PARAM_OK;
        break;
    default:
        /* no process */
        break;
    }

    return (result);
}
#endif /* #ifdef __DEBUG__ */

/*--------------------------------------------------*/
/**
  * @brief     DMA TX Transfer Done Handler.
  * @param     id     :User ID(TSPI DMA object).
  * @param     result :Result.
  * @retval    -
  * @note      This Function is called by DMA driver.
  */
/*--------------------------------------------------*/
static void tx_transfer_done(uint32_t id, TXZ_Result result)
{
    tspi_dma_t *p_obj = (tspi_dma_t *)id;

    /* TSPI Transmit Handler Call. */
    if ((p_obj != TSPI_NULL) && (p_obj->super.transmit.handler != TSPI_NULL))
    {
        p_obj->super.transmit.handler(p_obj->super.init.id, result);
    }
}

/*--------------------------------------------------*/
/**
  * @brief     DMA RX Transfer Done Handler.
  * @param     id     :User ID(TSPI DMA object).
  * @param     result :Result.
  * @retval    -
  * @note      This Function is called by DMA driver.
  */
/*--------------------------------------------------*/
static void rx_transfer_done(uint32_t id, TXZ_Result result)
{
    tspi_dma_t *p_obj = (tspi_dma_t *)id;

    /* TSPI Transmit Handler Call. */
    if ((p_obj != TSPI_NULL) && (p_obj->super.receive.handler != TSPI_NULL))
    {
        p_obj->super.receive.handler(p_obj->super.init.id, result, &p_obj->super.receive.info);
    }
}
/**
 *  @}
 */ /* End of group TSPI_Private_functions */


/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup TSPI_Exported_functions
 *  @{
 */
/*--------------------------------------------------*/
/**
  * @brief     Initialize the TSPI DMA object.
  * @param     p_obj   :TSPI DMA object.
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
TXZ_Result tspi_dma_init(tspi_dma_t *p_obj)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the TSPI_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->init.p_obj));
    /* Check the parameter. */
    assert_param(check_param_tx_type(p_obj->init.tx.type));
    assert_param(check_param_rx_type(p_obj->init.rx.type));
#endif /* #ifdef __DEBUG__ */
    /*------------------------------*/
    /*  Construct Superclass        */
    /*------------------------------*/
    if (tspi_init(&p_obj->super) != TXZ_SUCCESS)
    {
        result = TXZ_ERROR;
    }
    /*------------------------------*/
    /*  Construct Subclass          */
    /*------------------------------*/
    if (result == TXZ_SUCCESS)
    {

        /*--- DMA Driver "Transmit Channel" ---*/
        if (p_obj->init.tx.type == TSPI_HDMA_TYPE_TX_USE)
        {
            hdma_ch_setting_t *p_dma = &p_obj->init.p_obj->ch[p_obj->init.tx.ch];
            TSB_TSPI_TypeDef *tspi_reg = p_obj->super.p_instance;
            uint32_t dst_ch = 0;
            if(tspi_reg == 0)
            {
              result = TXZ_ERROR;
            }
            else if(tspi_reg == CH0)
            {
                dst_ch = 1;
            }
            else if(tspi_reg == CH1)
            {
                dst_ch = 1;
            }
            else if(tspi_reg == CH2)
            {
                dst_ch = 3;
            }
            else if(tspi_reg == CH3)
            {
                dst_ch = 3;
            }
            else if(tspi_reg == CH4)
            {
                dst_ch = 5;
            }
            else if(tspi_reg == CH5)
            {
                dst_ch = 5;
            }
            else
            {
              result = TXZ_ERROR;
            }
            
            if(result != TXZ_ERROR)
            {
              p_dma->handler       = tx_transfer_done;
              p_dma->id            = (uint32_t)p_obj;
              p_dma->cfg.flowCnt   = HDMA_CH_FLOWCNT_MEM_PERI;
              p_dma->srcReqCh      = HDMA_SRC_REQ_CH_MEM_USE;
              p_dma->dstReqCh      = dst_ch;
              if (hdma_channel_init(p_obj->init.p_obj, p_obj->init.tx.ch) != TXZ_SUCCESS)
              {
                  result = TXZ_ERROR;
              }
            }
        }
        /*--- DMA Driver "Receive Channel" ---*/
        if (p_obj->init.rx.type == TSPI_HDMA_TYPE_RX_USE)
        {
            hdma_ch_setting_t *p_dma = &p_obj->init.p_obj->ch[p_obj->init.rx.ch];
            TSB_TSPI_TypeDef *tspi_reg = p_obj->super.p_instance;
            uint32_t src_ch = 0;
            if(tspi_reg == 0)
            {
              result = TXZ_ERROR;
            }
            else if(tspi_reg == CH0)
            {
                src_ch = 0;
            }
            else if(tspi_reg == CH1)
            {
                src_ch = 0;
            }
             else if(tspi_reg == CH2)
            {
                src_ch = 2;
            }
            else if(tspi_reg == CH3)
            {
                src_ch = 2;
            }
            else if(tspi_reg == CH4)
            {
                src_ch = 4;
            }
            else if(tspi_reg == CH5)
            {
                src_ch = 4;
            }
            else
            {
              result = TXZ_ERROR;
            }
            
            if(result != TXZ_ERROR)
            {
              p_dma->handler       = rx_transfer_done;
              p_dma->id            = (uint32_t)p_obj;
              p_dma->cfg.flowCnt   = HDMA_CH_FLOWCNT_PERI_MEM;
              p_dma->srcReqCh      = src_ch;
              p_dma->dstReqCh      = HDMA_SRC_REQ_CH_MEM_USE;
              if (hdma_channel_init(p_obj->init.p_obj, p_obj->init.rx.ch) != TXZ_SUCCESS)
              {
                  result = TXZ_ERROR;
              }
             }
        }
    }

    return (result);
}

/*--------------------------------------------------*/
/**
  * @brief     Release the TSPI DMA object.
  * @param     p_obj   :TSPI DMA object.
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
TXZ_Result tspi_dma_deinit(tspi_dma_t *p_obj)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the TSPI_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
#endif /* #ifdef __DEBUG__ */
    /*------------------------------*/
    /*  Destruct Subclass           */
    /*------------------------------*/
    /*--- DMA Driver "Transmit Channel" ---*/
    if (p_obj->init.tx.type == TSPI_HDMA_TYPE_TX_USE)
    {
        __IO uint32_t cr2 = p_obj->super.p_instance->CR2;

        /*--- TSPIxCR2   ---*/
        cr2 &= ~TSPI_TX_DMA_INT_MASK;
        cr2 |=  TSPI_TX_DMA_INT_DISABLE;
        p_obj->super.p_instance->CR2 = cr2;
        /*--- DMA Driver ---*/

    }
    /*--- DMA Driver "Receive Channel" ---*/
    if (p_obj->init.rx.type == TSPI_HDMA_TYPE_RX_USE)
    {
        __IO uint32_t cr2 = p_obj->super.p_instance->CR2;

        /*--- TSPIxCR2   ---*/
        cr2 &= ~TSPI_RX_DMA_INT_MASK;
        cr2 |=  TSPI_RX_DMA_INT_DISABLE;
        p_obj->super.p_instance->CR2 = cr2;
        /*--- DMA Driver ---*/

    }
    /*------------------------------*/
    /*  Destruct Superclass         */
    /*------------------------------*/
    if (tspi_deinit(&p_obj->super) != TXZ_SUCCESS)
    {
        result = TXZ_ERROR;
    }

    return (result);
}

/*--------------------------------------------------*/
/**
  * @brief  Transmit data. Non-Blocking Communication.
  * @param  p_obj   :TSPI DMA object.
  * @param  p_info  :The information of transmit data.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   Asynchronous Processing.
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
TXZ_Result tspi_dma_transmitIt(tspi_dma_t *p_obj, tspi_transmit_t *p_info)
{
    TXZ_Result result  = TXZ_SUCCESS;
    uint32_t length = 0;
    /* Write to DMATE/DMARE (=0).               */
    p_obj->super.p_instance->CR2 &= ~TSPI_TX_DMA_INT_MASK; 
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the TSPI_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_info));
    /* Check the parameter of transmit. */
    if(p_obj->super.init.fmr0.fl  == TSPI_DATA_LENGTH_8)
    {
        /* 8 bit */
        assert_param(IS_POINTER_NOT_NULL(p_info->tx8.p_data));
    }
    else if ((p_obj->super.init.fmr0.fl > TSPI_DATA_LENGTH_8) && (p_obj->super.init.fmr0.fl  < TSPI_DATA_LENGTH_17) )
    {
        /* 9 - 16 bit */
        assert_param(IS_POINTER_NOT_NULL(p_info->tx16.p_data));
    }
    else
    {
        /* 17 - 32 bit */
        assert_param(IS_POINTER_NOT_NULL(p_info->tx32.p_data));
    }
#endif /* #ifdef __DEBUG__ */
    /*------------------------------*/
    /*  DMA Setting                 */
    /*------------------------------*/
    {
        hdma_initial_setting_t *p_dma_init = &p_obj->init.p_obj->init;
        hdma_ch_setting_t *p_dma = &p_obj->init.p_obj->ch[p_obj->init.tx.ch];
        p_dma_init->dstAdd = (uint32_t)&p_obj->super.p_instance->DR;
        length = (uint32_t)((p_obj->super.p_instance->FMTR0 & TSPI_DATA_LENGTH_MASK)  >> 24);
        if(length == DATA_LENGTH_8)
        {
            /* 8 bit */
            p_obj->super.transmit.info.tx8.p_data  = p_info->tx8.p_data;
            p_obj->super.transmit.info.tx8.num     = p_info->tx8.num;
                   
            p_dma_init->srcAdd      = (uint32_t)p_info->tx8.p_data;
            p_dma->cnt.dstInc       = HDMA_DST_ADDR_FIX;
            p_dma->cnt.dstSize      = HDMA_DST_BITWIDTH_1BYTE;
            p_dma->cnt.dstBurstSize = HDMA_DST_BURSTSIZE_1BEAT;
            p_dma->cnt.srcInc       = HDMA_SRC_ADDR_INC;
            p_dma->cnt.srcSize      = HDMA_SRC_BITWIDTH_1BYTE;
            p_dma->cnt.srcBurstSize = HDMA_SRC_BURSTSIZE_1BEAT;
            p_dma->cnt.transsize    = p_obj->super.transmit.info.tx8.num;
            p_dma->type             = HDMA_TRANS_TYPE_BURST;

        }
        else if ((length > DATA_LENGTH_8) && (length  < DATA_LENGTH_17) )
        {
            /* 9 - 16 bit */
            p_obj->super.transmit.info.tx16.p_data = p_info->tx16.p_data;
            p_obj->super.transmit.info.tx16.num    = p_info->tx16.num;

            p_dma_init->srcAdd      = (uint32_t)p_info->tx16.p_data;
            p_dma->cnt.dstInc       = HDMA_DST_ADDR_FIX;
            p_dma->cnt.dstSize      = HDMA_DST_BITWIDTH_2BYTE;
            p_dma->cnt.dstBurstSize = HDMA_DST_BURSTSIZE_1BEAT;
            p_dma->cnt.srcInc       = HDMA_SRC_ADDR_INC;
            p_dma->cnt.srcSize      = HDMA_SRC_BITWIDTH_2BYTE;
            p_dma->cnt.srcBurstSize = HDMA_SRC_BURSTSIZE_1BEAT;
            p_dma->cnt.transsize    = p_obj->super.transmit.info.tx16.num;
            p_dma->type             = HDMA_TRANS_TYPE_BURST;
        }
        else
        {
            /* 17 - 32 bit */
            p_obj->super.transmit.info.tx32.p_data = p_info->tx32.p_data;
            p_obj->super.transmit.info.tx32.num    = p_info->tx32.num;
            
            p_dma_init->srcAdd      = (uint32_t)p_info->tx32.p_data;
            p_dma->cnt.dstInc       = HDMA_DST_ADDR_FIX;
            p_dma->cnt.dstSize      = HDMA_DST_BITWIDTH_4BYTE;
            p_dma->cnt.dstBurstSize = HDMA_DST_BURSTSIZE_1BEAT;
            p_dma->cnt.srcInc       = HDMA_SRC_ADDR_INC;
            p_dma->cnt.srcSize      = HDMA_SRC_BITWIDTH_4BYTE;
            p_dma->cnt.srcBurstSize = HDMA_SRC_BURSTSIZE_1BEAT;
            p_dma->cnt.transsize    = p_obj->super.transmit.info.tx32.num;
            p_dma->type             = HDMA_TRANS_TYPE_BURST;
        }
        if (hdma_startPeriIt(p_obj->init.p_obj, p_obj->init.tx.ch ) != TXZ_SUCCESS)
        {
            result  = TXZ_ERROR;
        }
    }
    /*------------------------------*/
    /*  Enable Transmit             */
    /*------------------------------*/
    /* Write to DMATE(=1).               */
    p_obj->super.p_instance->CR2 |=  TSPI_TX_DMA_INT_ENABLE;
    /*--- TSPIxTRANS ---*/
    /* Write to TRXE(=1).               */
    /* Check if the TSPI is already enabled */
    if((p_obj->super.p_instance->CR0 & TSPI_ENABLE) != TSPI_ENABLE)
    {
        p_obj->super.p_instance->CR0 |=  TSPI_ENABLE;
    }
    
    /* Enable TSPI Transmission Control */
    p_obj->super.p_instance->CR1 |= TSPI_TRXE_ENABLE;

    return (result);
}

/*--------------------------------------------------*/
/**
  * @brief  Receive data. Non-Blocking Communication.
  * @param  p_obj   :TSPI DMA object.
  * @param  p_info  :The information of receive buffer.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   Asynchronous Processing.
  * @attention  "p_info->rx8(or rx16).num" must be over FIFO max(Refer @ref TSPI_FifoMax) num.
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
TXZ_Result tspi_dma_receiveIt(tspi_dma_t *p_obj, tspi_receive_t *p_info)
{
    TXZ_Result result = TXZ_SUCCESS;
    uint32_t length = 0;
    
    /* Write to DMATE/DMARE (=0).               */
    p_obj->super.p_instance->CR2 &= ~TSPI_RX_DMA_INT_MASK; 
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the TSPI_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_info));
    /* Check the parameter of transmit. */
    if(p_obj->super.init.fmr0.fl  == TSPI_DATA_LENGTH_8)
    {
        /* 8 bit */
        assert_param(IS_POINTER_NOT_NULL(p_info->rx8.p_data));
    }
    else if ((p_obj->super.init.fmr0.fl > TSPI_DATA_LENGTH_8) && (p_obj->super.init.fmr0.fl  < TSPI_DATA_LENGTH_17) )
    {
        /* 9 - 16 bit */
        assert_param(IS_POINTER_NOT_NULL(p_info->rx16.p_data));
    }
    else
    {
        /* 17 - 32 bit */
        assert_param(IS_POINTER_NOT_NULL(p_info->rx32.p_data));
    }
#endif /* #ifdef __DEBUG__ */
    /*------------------------------*/
    /*  DMA Setting                 */
    /*------------------------------*/
    {
        hdma_initial_setting_t *p_dma_init = &p_obj->init.p_obj->init;
        hdma_ch_setting_t *p_dma = &p_obj->init.p_obj->ch[p_obj->init.rx.ch];
        p_dma_init->srcAdd = (uint32_t)&p_obj->super.p_instance->DR;

        length = (uint32_t)((p_obj->super.p_instance->FMTR0 & TSPI_DATA_LENGTH_MASK)  >> 24);
        if(length == DATA_LENGTH_8)
        {
            /* 8 bit */
            p_obj->super.receive.info.rx8.p_data  = p_info->rx8.p_data;
            p_obj->super.receive.info.rx8.num     = p_info->rx8.num;
            
            p_dma_init->dstAdd      = (uint32_t)p_info->rx8.p_data;
            p_dma->cnt.dstInc       = HDMA_DST_ADDR_INC;
            p_dma->cnt.dstSize      = HDMA_DST_BITWIDTH_1BYTE;
            p_dma->cnt.dstBurstSize = HDMA_DST_BURSTSIZE_1BEAT;
            p_dma->cnt.srcInc       = HDMA_SRC_ADDR_FIX;
            p_dma->cnt.srcSize      = HDMA_SRC_BITWIDTH_1BYTE;
            p_dma->cnt.srcBurstSize = HDMA_SRC_BURSTSIZE_1BEAT;
            p_dma->cnt.transsize    = p_obj->super.receive.info.rx8.num;
            p_dma->type             = HDMA_TRANS_TYPE_BURST;
            
        }
        else if ((length > DATA_LENGTH_8) && (length  < DATA_LENGTH_17) )
        {
            /* 9 - 16 bit */
            p_obj->super.receive.info.rx16.p_data = p_info->rx16.p_data;
            p_obj->super.receive.info.rx16.num    = p_info->rx16.num;
            
            p_dma_init->dstAdd      = (uint32_t)p_info->rx16.p_data;
            p_dma->cnt.dstInc       = HDMA_DST_ADDR_INC;
            p_dma->cnt.dstSize      = HDMA_DST_BITWIDTH_2BYTE;
            p_dma->cnt.dstBurstSize = HDMA_DST_BURSTSIZE_1BEAT;
            p_dma->cnt.srcInc       = HDMA_SRC_ADDR_FIX;
            p_dma->cnt.srcSize      = HDMA_SRC_BITWIDTH_2BYTE;
            p_dma->cnt.srcBurstSize = HDMA_SRC_BURSTSIZE_1BEAT;
            p_dma->cnt.transsize    = p_obj->super.receive.info.rx16.num;
            p_dma->type             = HDMA_TRANS_TYPE_BURST;
        }
        else
        {
            /* 17 - 32 bit */
            p_obj->super.receive.info.rx32.p_data = p_info->rx32.p_data;
            p_obj->super.receive.info.rx32.num    = p_info->rx32.num;
            
            p_dma_init->dstAdd      = (uint32_t)p_info->rx32.p_data;
            p_dma->cnt.dstInc       = HDMA_DST_ADDR_INC;
            p_dma->cnt.dstSize      = HDMA_DST_BITWIDTH_4BYTE;
            p_dma->cnt.dstBurstSize = HDMA_DST_BURSTSIZE_1BEAT;
            p_dma->cnt.srcInc       = HDMA_SRC_ADDR_FIX;
            p_dma->cnt.srcSize      = HDMA_SRC_BITWIDTH_4BYTE;
            p_dma->cnt.srcBurstSize = HDMA_SRC_BURSTSIZE_1BEAT;
            p_dma->cnt.transsize    = p_obj->super.receive.info.rx32.num;
            p_dma->type             = HDMA_TRANS_TYPE_BURST;
        }
        
        if (hdma_startPeriIt(p_obj->init.p_obj, p_obj->init.rx.ch ) != TXZ_SUCCESS)
        {
            result  = TXZ_ERROR;
        }
    }
    /*------------------------------*/
    /*  Enable Receive              */
    /*------------------------------*/
    /* Write to DMARE(=1).               */
    p_obj->super.p_instance->CR2 |=  TSPI_RX_DMA_INT_ENABLE;
    /*--- TSPIxTRANS ---*/
    /* Write to TRXE(=1).               */
    /* Check if the TSPI is already enabled */
    if((p_obj->super.p_instance->CR0 & TSPI_ENABLE) != TSPI_ENABLE)
    {
        p_obj->super.p_instance->CR0 |=  TSPI_ENABLE;
    }
    
    /* Enable TSPI Transmission Control */
    p_obj->super.p_instance->CR1 |= TSPI_TRXE_ENABLE;

    return (result);
}

/*--------------------------------------------------*/
/**
  * @brief     Discard transmit.
  * @param     p_obj   :TSPI DMA object.
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      This function clears transmit's fifo, end flag and error info.
  * @attention This function is not available in interrupt.
  * @attention Use after interrupt is disabled.
  */
/*--------------------------------------------------*/
TXZ_Result tspi_dma_discard_transmit(tspi_dma_t *p_obj)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the TSPI_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
#endif /* #ifdef __DEBUG__ */
    /*------------------------------*/
    /*  Stop DMA                    */
    /*------------------------------*/
    if (p_obj->init.tx.type != TSPI_HDMA_TYPE_UNUSE)
    {
        if ( hdma_stopPeriIt(p_obj->init.p_obj, p_obj->init.tx.ch) != TXZ_SUCCESS)
        {
            result = TXZ_ERROR;
        }
    }
    /*------------------------------*/
    /*  Use Super Class             */
    /*------------------------------*/
    if (tspi_discard_transmit(&p_obj->super) != TXZ_SUCCESS)
    {
        result = TXZ_ERROR;
    }

    return (result);
}

/*--------------------------------------------------*/
/**
  * @brief     Discard receive.
  * @param     p_obj   :TSPI DMA object.
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      This function clears receive's fifo, end flag and error info.
  * @attention This function is not available in interrupt.
  * @attention Use after interrupt is disabled.
  */
/*--------------------------------------------------*/
TXZ_Result tspi_dma_discard_receive(tspi_dma_t *p_obj)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the TSPI_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
#endif /* #ifdef __DEBUG__ */
    /*------------------------------*/
    /*  Stop DMA                    */
    /*------------------------------*/
    if (p_obj->init.rx.type != TSPI_HDMA_TYPE_UNUSE)
    {
        if ( hdma_stopPeriIt(p_obj->init.p_obj, p_obj->init.rx.ch) != TXZ_SUCCESS)
        {
            result = TXZ_ERROR;
        }
    }
    /*------------------------------*/
    /*  Use Super Class             */
    /*------------------------------*/
    if (tspi_discard_receive(&p_obj->super) != TXZ_SUCCESS)
    {
        result = TXZ_ERROR;
    }

    return (result);
}

/**
 *  @}
 */ /* End of group TSPI_Exported_functions */

/**
 *  @} 
 */ /* End of group TSPI */

/**
 *  @} 
 */ /* End of group Periph_Driver */

#endif /* defined(__TSPI_HDMA_H)  */

#ifdef __cplusplus
}
#endif /* __cplusplus */

