/** 
 *******************************************************************************
 * @file    bsp_ei2c.c
 * @brief   This file provides API functions for BSP EI2C driver.
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2020
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "bsp_ei2c.h"
#include "txz_hal.h"

#if defined(__BSP_EI2C_H)

/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @addtogroup UTILITIES
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup UTILITIES_Private_macro
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group UTILITIES_Private_macro */

/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup UTILITIES_Private_define
 *  @{
 */

/** 
 *  @name  Parameter Result
 *  @brief Whether the parameter is specified or not.
 *  @{
 */
#define EI2C_PARAM_OK              ((int32_t)1)              /*!< Parameter is valid(specified).         */
#define EI2C_PARAM_NG              ((int32_t)0)              /*!< Parameter is invalid(not specified).   */
/**
 *  @}
 */ /* End of name Parameter Result */

/** 
 *  @name  timeout 
 *  @brief This timeouts are not based on accurate values, this just guarantee that 
           the application will not remain stuck if the EI2C communication is corrupted.
 *  @{
 */
#define EI2C_TIMEOUT     (100000)    /*>! fail safe. */

/**
 *  @}
 */ /* End of name timeout */

#define EI2CxSR_AL               ((uint32_t)0x00000008)     /*!< AL                      */
/** 
 *  @}
 */ /* End of group UTILITIES_Private_define */

/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup UTILITIES_Private_define
 *  @{
 */
#define EI2C_CH0                     (0)         /*!< EI2C Channel 0.            */
#define EI2C_CH1                     (1)         /*!< EI2C Channel 1.            */
#define EI2C_CH2                     (2)         /*!< EI2C Channel 2.            */
#define EI2C_CH3                     (3)         /*!< EI2C Channel 3.            */
#define EI2C_CH4                     (4)         /*!< EI2C Channel 3.            */
#define EI2C_CH_NUM                  (5)         /*!< Number of EI2C Channel.    */

/** 
 *  @}
 */ /* End of group UTILITIES_Private_define */

/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup UTILITIES_Private_define
 *  @{
 */
/*----------------------------------*/
/** 
 * @brief  Transfer State.
*/
/*----------------------------------*/
enum {
    EI2C_TRANSFER_STATE_IDLE = 0U,   /*!< Idle.                      */
    EI2C_TRANSFER_STATE_BUSY         /*!< Busy.                      */
} TransferState;

/** 
 *  @}
 */ /* End of group UTILITIES_Private_define */

/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup UTILITIES_Private_typedef
 *  @{
 */

/*----------------------------------*/
/** 
 * @brief  For IRQn_Type number definition.
*/
/*----------------------------------*/
typedef struct
{
    IRQn_Type ei2c;
    IRQn_Type tbe;
    IRQn_Type rbf;
} ei2c_irq_t;

/** 
 *  @}
 */ /* End of group UTILITIES_Private_typedef */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup UTILITIES_Private_variables
 *  @{
 */

/* no define */

/**
 *  @}
 */ /* End of group UTILITIES_Private_variables */

/*------------------------------------------------------------------------------*/
/*  Const Table                                                                 */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup UTILITIES_Private_const
 *  @{
 */
/*----------------------------------*/
/** 
 * @brief  Channel 0 IRQn_Type number table.
*/
/*----------------------------------*/
static const ei2c_irq_t EI2C_CH0_IRQN_TBL[1] = 
{
    { INTI2C0NST_IRQn, INTI2C0ATX_IRQn, INTI2C0BRX_IRQn }
};

/*----------------------------------*/
/** 
 * @brief  Channel 1 IRQn_Type number table.
*/
/*----------------------------------*/
static const ei2c_irq_t EI2C_CH1_IRQN_TBL[1] = 
{
    { INTI2C1NST_IRQn, INTI2C1ATX_IRQn, INTI2C1BRX_IRQn }
};

/*----------------------------------*/
/** 
 * @brief  Channel 2 IRQn_Type number table.
*/
/*----------------------------------*/
static const ei2c_irq_t EI2C_CH2_IRQN_TBL[1] = 
{
    { INTI2C2NST_IRQn, INTI2C2ATX_IRQn, INTI2C2BRX_IRQn }
};

#if (defined(TMPM4GR) || defined(TMPM4GQ))
/*----------------------------------*/
/** 
 * @brief  Channel 3 IRQn_Type number table.
*/
/*----------------------------------*/
static const ei2c_irq_t EI2C_CH3_IRQN_TBL[1] = 
{
    { INTI2C3NST_IRQn, INTI2C3ATX_IRQn, INTI2C3BRX_IRQn }
};

/*----------------------------------*/
/** 
 * @brief  Channel 4 IRQn_Type number table.
*/
/*----------------------------------*/
static const ei2c_irq_t EI2C_CH4_IRQN_TBL[1] = 
{
    { INTI2C4NST_IRQn, INTI2C4ATX_IRQn, INTI2C4BRX_IRQn }
};
#endif
/**
 *  @}
 */ /* End of group UTILITIES_Private_const */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup UTILITIES_Private_functions
 *  @{
 */
#ifdef __DEBUG__
__STATIC_INLINE int32_t check_param_irqn(uint32_t irqn);
__STATIC_INLINE int32_t check_param_address(int32_t address);
#endif
__STATIC_INLINE void enable_irq(uint32_t irqn);
__STATIC_INLINE void disable_irq(uint32_t irqn);
__STATIC_INLINE void clear_irq(uint32_t irqn);
__STATIC_INLINE void set_port_ch0(ei2c_port_t sda, ei2c_port_t scl);
__STATIC_INLINE void set_port_ch1(ei2c_port_t sda, ei2c_port_t scl);
__STATIC_INLINE void set_port_ch2(ei2c_port_t sda, ei2c_port_t scl);
#if (defined(TMPM4GR) || defined(TMPM4GQ))
__STATIC_INLINE void set_port_ch3(ei2c_port_t sda, ei2c_port_t scl);
__STATIC_INLINE void set_port_ch4(ei2c_port_t sda, ei2c_port_t scl);
#endif
__STATIC_INLINE uint32_t set_ei2c(uint8_t ch, uint32_t *p_irqn);
__STATIC_INLINE void reset_asynch(bsp_ei2c_t *p_obj);
static void ei2c_irq_handler(bsp_ei2c_t *p_obj);
static void ei2c_slave_irq_handler(bsp_ei2c_t *p_obj);

#ifdef __DEBUG__
/*--------------------------------------------------*/
/** 
  * @brief  Compare the IRQn's parameter.
  * @param  irqn         :EI2C IRQn List.
  * @retval EI2C_PARAM_OK :Available.
  * @retval EI2C_PARAM_NG :Not Available.
  * @note   -.
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_irqn(uint32_t irqn)
{
    int32_t result = EI2C_PARAM_NG;

    if (irqn == (uint32_t)&EI2C_CH0_IRQN_TBL)
    {
        result = EI2C_PARAM_OK;
    }
    if (irqn == (uint32_t)&EI2C_CH1_IRQN_TBL)
    {
        result = EI2C_PARAM_OK;
    }
    if (irqn == (uint32_t)&EI2C_CH2_IRQN_TBL)
    {
        result = EI2C_PARAM_OK;
    }
    if (irqn == (uint32_t)&EI2C_CH3_IRQN_TBL)
    {
        result = EI2C_PARAM_OK;
    }
    if (irqn == (uint32_t)&EI2C_CH4_IRQN_TBL)
    {
        result = EI2C_PARAM_OK;
    }
    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Compare the Slave address's parameter.
  * @param  address      :Address.
  * @retval EI2C_PARAM_OK :Available.
  * @retval EI2C_PARAM_NG :Not Available.
  * @note   Here, 10bit address has not supported.
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_address(int32_t address)
{
    int32_t result = EI2C_PARAM_NG;

    if ((address >= 0) && (address <= 255))
    {
        result = EI2C_PARAM_OK;
    }
    return (result);
}
#endif

/*--------------------------------------------------*/
/**
  * @brief     Enable EI2C IRQ
  * @param     irqn     :EI2C IRQn List.
  * @retval    -
  * @note      -
  */
/*--------------------------------------------------*/
__STATIC_INLINE void enable_irq(uint32_t irqn)
{
    ei2c_irq_t *p_irqn = (ei2c_irq_t *)irqn;
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    assert_param(check_param_irqn(irqn));
#endif /* #ifdef __DEBUG__ */
    NVIC_EnableIRQ(p_irqn->ei2c);
    NVIC_EnableIRQ(p_irqn->rbf);
    NVIC_EnableIRQ(p_irqn->tbe);
}

/*--------------------------------------------------*/
/**
  * @brief     Disable EI2C IRQ
  * @param     irqn     :EI2C IRQn List.
  * @retval    -
  * @note      -
  */
/*--------------------------------------------------*/
__STATIC_INLINE void disable_irq(uint32_t irqn)
{
    ei2c_irq_t *p_irqn = (ei2c_irq_t *)irqn;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    assert_param(check_param_irqn(irqn));
#endif /* #ifdef __DEBUG__ */
    NVIC_DisableIRQ(p_irqn->ei2c);
    NVIC_DisableIRQ(p_irqn->tbe);
    NVIC_DisableIRQ(p_irqn->rbf);
}

/*--------------------------------------------------*/
/**
  * @brief     ClearPending EI2C IRQ
  * @param     irqn     :EI2C IRQn List.
  * @retval    -
  * @note      -
  */
/*--------------------------------------------------*/
__STATIC_INLINE void clear_irq(uint32_t irqn)
{
    ei2c_irq_t *p_irqn = (ei2c_irq_t *)irqn;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    assert_param(check_param_irqn(irqn));
#endif /* #ifdef __DEBUG__ */
    NVIC_ClearPendingIRQ(p_irqn->ei2c);
    NVIC_ClearPendingIRQ(p_irqn->tbe);
    NVIC_ClearPendingIRQ(p_irqn->rbf);
}

/*--------------------------------------------------*/
/**
  * @brief     EI2C Port Setting (PG2, PG3)
  * @param     sda         :SDA port.
  * @param     scl         :SCL port.
  * @retval    -
  * @note      -
  */
/*--------------------------------------------------*/
__STATIC_INLINE void set_port_ch0(ei2c_port_t sda, ei2c_port_t scl)
{   
    if ((sda == EI2C_PORT_PG2) && (scl == EI2C_PORT_PG3))
    {
        /* Port G  */
        TSB_CG_FSYSMENB_IPMENB08 = 1;     /* PORT  G   */

        /* SCL */
        TSB_PG_IE_PG3IE   = 0;    /* Input       :Disable */
        TSB_PG_CR_PG3C    = 0;    /* Output      :Disable */
        TSB_PG_OD_PG3OD   = 1;    /* OD Control  :Open Drain */
        TSB_PG_PUP_PG3UP  = 0;    /* Pull-up     :Disable */
        TSB_PG_PDN_PG3DN  = 0;    /* Pull-down   :Disable */
        TSB_PG_DATA_PG3   = 0;    /* Data        :0       */
        TSB_PG_FR6_PG3F6  = 1;    /* Function    :EI2C0SCL */
        TSB_PG_IE_PG3IE   = 1;    /* Input       :Enable  */
        TSB_PG_CR_PG3C    = 1;    /* Output      :Enable  */

        /* SDA */
        TSB_PG_IE_PG2IE   = 0;    /* Input      :Disable  */
        TSB_PG_CR_PG2C    = 0;    /* Output     :Disable  */
        TSB_PG_OD_PG2OD   = 1;    /* OD Control :Open Drain */
        TSB_PG_PUP_PG2UP  = 0;    /* Pull-up    :Disable  */
        TSB_PG_PDN_PG2DN  = 0;    /* Pull-down  :Disable  */
        TSB_PG_DATA_PG2   = 0;    /* Data       :0        */
        TSB_PG_FR6_PG2F6  = 1;    /* Function   :EI2C0SDA  */
        TSB_PG_IE_PG2IE   = 1;    /* Input      :Enable   */
        TSB_PG_CR_PG2C    = 1;    /* Output     :Enable   */
    }
}

/*--------------------------------------------------*/
/**
  * @brief     EI2C Port Setting (PF2, PF3)
  * @param     sda         :SDA port.
  * @param     scl         :SCL port.
  * @retval    -
  * @note      -
  */
/*--------------------------------------------------*/
__STATIC_INLINE void set_port_ch1(ei2c_port_t sda, ei2c_port_t scl)
{
    if ((sda == EI2C_PORT_PF2) && (scl == EI2C_PORT_PF3))
    {
        /* Port F  */
        TSB_CG_FSYSMENB_IPMENB07 = 1;     /* PORT  F   */
        
        /* SCL */
        TSB_PF_IE_PF3IE   = 0;    /* Input       :Disable */
        TSB_PF_CR_PF3C    = 0;    /* Output      :Disable */
        TSB_PF_OD_PF3OD   = 1;    /* OD Control  :Open Drain */
        TSB_PF_PUP_PF3UP  = 0;    /* Pull-up     :Disable */
        TSB_PF_PDN_PF3DN  = 0;    /* Pull-down   :Disable */
        TSB_PF_DATA_PF3   = 0;    /* Data        :0       */
        TSB_PF_FR6_PF3F6  = 1;    /* Function    :EI2C1SCL */
        TSB_PF_IE_PF3IE   = 1;    /* Input       :Enable  */
        TSB_PF_CR_PF3C    = 1;    /* Output      :Enable  */

        /* SDA */
        TSB_PF_IE_PF2IE   = 0;    /* Input      :Disable  */
        TSB_PF_CR_PF2C    = 0;    /* Output     :Disable  */
        TSB_PF_OD_PF2OD   = 1;    /* OD Control :Open Drain */
        TSB_PF_PUP_PF2UP  = 0;    /* Pull-up    :Disable  */
        TSB_PF_PDN_PF2DN  = 0;    /* Pull-down  :Disable  */
        TSB_PF_DATA_PF2   = 0;    /* Data       :0        */
        TSB_PF_FR6_PF2F6  = 1;    /* Function   :EI2C1SDA  */
        TSB_PF_IE_PF2IE   = 1;    /* Input      :Enable   */
        TSB_PF_CR_PF2C    = 1;    /* Output     :Enable   */
    }
}

/*--------------------------------------------------*/
/**
  * @brief     EI2C Port Setting (PG4, PG5)(PV4, PV5)
  * @param     sda         :SDA port.
  * @param     scl         :SCL port.
  * @retval    -
  * @note      -
  */
/*--------------------------------------------------*/
__STATIC_INLINE void set_port_ch2(ei2c_port_t sda, ei2c_port_t scl)
{   
    if ((sda == EI2C_PORT_PG4) && (scl == EI2C_PORT_PG5))
    {
        /* Port G  */
        TSB_CG_FSYSMENB_IPMENB08 = 1;     /* PORT  G   */

        /* SCL */
        TSB_PG_IE_PG5IE   = 0;    /* Input       :Disable */
        TSB_PG_CR_PG5C    = 0;    /* Output      :Disable */
        TSB_PG_OD_PG5OD   = 1;    /* OD Control  :Open Drain */
        TSB_PG_PUP_PG5UP  = 0;    /* Pull-up     :Disable */
        TSB_PG_PDN_PG5DN  = 0;    /* Pull-down   :Disable */
        TSB_PG_DATA_PG5   = 0;    /* Data        :0       */
        TSB_PG_FR6_PG5F6  = 1;    /* Function    :EI2C2SCL */
        TSB_PG_IE_PG5IE   = 1;    /* Input       :Enable  */
        TSB_PG_CR_PG5C    = 1;    /* Output      :Enable  */

        /* SDA */
        TSB_PG_IE_PG4IE   = 0;    /* Input      :Disable  */
        TSB_PG_CR_PG4C    = 0;    /* Output     :Disable  */
        TSB_PG_OD_PG4OD   = 1;    /* OD Control :Open Drain */
        TSB_PG_PUP_PG4UP  = 0;    /* Pull-up    :Disable  */
        TSB_PG_PDN_PG4DN  = 0;    /* Pull-down  :Disable  */
        TSB_PG_DATA_PG4   = 0;    /* Data       :0        */
        TSB_PG_FR6_PG4F6  = 1;    /* Function   :EI2C2SDA  */
        TSB_PG_IE_PG4IE   = 1;    /* Input      :Enable   */
        TSB_PG_CR_PG4C    = 1;    /* Output     :Enable   */
    }
    else if ((sda == EI2C_PORT_PV5) && (scl == EI2C_PORT_PV4))
    {
#if (defined(TMPM4GR) || defined(TMPM4GQ))
        /* Port V  */
        TSB_CG_FSYSMENB_IPMENB19 = 1;     /* PORT  V   */

        /* SCL */
        TSB_PV_IE_PV4IE   = 0;    /* Input       :Disable */
        TSB_PV_CR_PV4C    = 0;    /* Output      :Disable */
        TSB_PV_OD_PV4OD   = 1;    /* OD Control  :Open Drain */
        TSB_PV_PUP_PV4UP  = 0;    /* Pull-up     :Disable */
        TSB_PV_PDN_PV4DN  = 0;    /* Pull-down   :Disable */
        TSB_PV_DATA_PV4   = 0;    /* Data        :0       */
        TSB_PV_FR1_PV4F1  = 1;    /* Function    :EI2C2SCL */
        TSB_PV_IE_PV4IE   = 1;    /* Input       :Enable  */
        TSB_PV_CR_PV4C    = 1;    /* Output      :Enable  */

        /* SDA */
        TSB_PV_IE_PV5IE   = 0;    /* Input      :Disable  */
        TSB_PV_CR_PV5C    = 0;    /* Output     :Disable  */
        TSB_PV_OD_PV5OD   = 1;    /* OD Control :Open Drain */
        TSB_PV_PDN_PV5DN  = 0;    /* Pull-down  :Disable  */
        TSB_PV_DATA_PV5   = 0;    /* Data       :0        */
        TSB_PV_FR1_PV5F1  = 1;    /* Function   :EI2C2SDA  */
        TSB_PV_IE_PV5IE   = 1;    /* Input      :Enable   */
        TSB_PV_CR_PV5C    = 1;    /* Output     :Enable   */
#endif
    }
}
#if (defined(TMPM4GR) || defined(TMPM4GQ))
/*--------------------------------------------------*/
/**
  * @brief     EI2C Port Setting (PJ6, PJ7)(PM0, PM1)
  * @param     sda         :SDA port.
  * @param     scl         :SCL port.
  * @retval    -
  * @note      -
  */
/*--------------------------------------------------*/
__STATIC_INLINE void set_port_ch3(ei2c_port_t sda, ei2c_port_t scl)
{   
    if ((sda == EI2C_PORT_PJ6) && (scl == EI2C_PORT_PJ7))
    {
#if (defined(TMPM4GR))
        /* Port J  */
        TSB_CG_FSYSMENB_IPMENB10 = 1;     /* PORT  J   */

        /* SCL */
        TSB_PJ_IE_PJ7IE   = 0;    /* Input      :Disable  */
        TSB_PJ_CR_PJ7C    = 0;    /* Output     :Disable  */
        TSB_PJ_OD_PJ7OD   = 1;    /* OD Control :Open Drain */
        TSB_PJ_PUP_PJ7UP  = 0;    /* Pull-up    :Disable  */
        TSB_PJ_PDN_PJ7DN  = 0;    /* Pull-down  :Disable  */
        TSB_PJ_DATA_PJ7   = 0;    /* Data       :0        */
        TSB_PJ_FR6_PJ7F6  = 1;    /* Function   :EI2C3SCL  */
        TSB_PJ_IE_PJ7IE   = 1;    /* Input      :Enable   */
        TSB_PJ_CR_PJ7C    = 1;    /* Output     :Enable   */

        /* SDA */
        TSB_PJ_IE_PJ6IE   = 0;    /* Input       :Disable */
        TSB_PJ_CR_PJ6C    = 0;    /* Output      :Disable */
        TSB_PJ_OD_PJ6OD   = 1;    /* OD Control  :Open Drain */
        TSB_PJ_PUP_PJ6UP  = 0;    /* Pull-up     :Disable */
        TSB_PJ_PDN_PJ6DN  = 0;    /* Pull-down   :Disable */
        TSB_PJ_DATA_PJ6   = 0;    /* Data        :0       */
        TSB_PJ_FR6_PJ6F6  = 1;    /* Function    :EI2C3SDA */
        TSB_PJ_IE_PJ6IE   = 1;    /* Input       :Enable  */
        TSB_PJ_CR_PJ6C    = 1;    /* Output      :Enable  */
#endif
    }
    else if ((sda == EI2C_PORT_PM0) && (scl == EI2C_PORT_PM1))
    {
        /* Port M  */
        TSB_CG_FSYSMENB_IPMENB13 = 1;     /* PORT  M   */

        /* SCL */
        TSB_PM_IE_PM1IE   = 0;    /* Input      :Disable  */
        TSB_PM_CR_PM1C    = 0;    /* Output     :Disable  */
        TSB_PM_OD_PM1OD   = 1;    /* OD Control :Open Drain */
        TSB_PM_PUP_PM1UP  = 0;    /* Pull-up    :Disable  */
        TSB_PM_PDN_PM1DN  = 0;    /* Pull-down  :Disable  */
        TSB_PM_DATA_PM1   = 0;    /* Data       :0        */
        TSB_PM_FR1_PM1F1  = 1;    /* Function   :EI2C3SCL  */
        TSB_PM_IE_PM1IE   = 1;    /* Input      :Enable   */
        TSB_PM_CR_PM1C    = 1;    /* Output     :Enable   */

        /* SDA */
        TSB_PM_IE_PM0IE   = 0;    /* Input       :Disable */
        TSB_PM_CR_PM0C    = 0;    /* Output      :Disable */
        TSB_PM_OD_PM0OD   = 1;    /* OD Control  :Open Drain */
        TSB_PM_PUP_PM0UP  = 0;    /* Pull-up     :Disable */
        TSB_PM_PDN_PM0DN  = 0;    /* Pull-down   :Disable */
        TSB_PM_DATA_PM0   = 0;    /* Data        :0       */
        TSB_PM_FR1_PM0F1  = 1;    /* Function    :EI2C3SDA */
        TSB_PM_IE_PM0IE   = 1;    /* Input       :Enable  */
        TSB_PM_CR_PM0C    = 1;    /* Output      :Enable  */
    }
}
#endif

#if (defined(TMPM4GR) || defined(TMPM4GQ))
/*--------------------------------------------------*/
/**
  * @brief     I2C Port Setting (PJ2, PJ3)(PM6, PM7)
  * @param     sda         :SDA port.
  * @param     scl         :SCL port.
  * @retval    -
  * @note      -
  */
/*--------------------------------------------------*/
__STATIC_INLINE void set_port_ch4(ei2c_port_t sda, ei2c_port_t scl)
{   
    if ((sda == EI2C_PORT_PJ3) && (scl == EI2C_PORT_PJ2))
    {
#if (defined(TMPM4GR))
        /* Port J  */
        TSB_CG_FSYSMENB_IPMENB10 = 1;     /* PORT  J   */

        /* SCL */
        TSB_PJ_IE_PJ2IE   = 0;    /* Input       :Disable */
        TSB_PJ_CR_PJ2C    = 0;    /* Output      :Disable */
        TSB_PJ_OD_PJ2OD   = 1;    /* OD Control  :Open Drain */
        TSB_PJ_PUP_PJ2UP  = 0;    /* Pull-up     :Disable */
        TSB_PJ_PDN_PJ2DN  = 0;    /* Pull-down   :Disable */
        TSB_PJ_DATA_PJ2   = 0;    /* Data        :0       */
        TSB_PJ_FR6_PJ2F6  = 1;    /* Function    :EI2C4SCL */
        TSB_PJ_IE_PJ2IE   = 1;    /* Input       :Enable  */
        TSB_PJ_CR_PJ2C    = 1;    /* Output      :Enable  */

        /* SDA */
        TSB_PJ_IE_PJ3IE   = 0;    /* Input      :Disable  */
        TSB_PJ_CR_PJ3C    = 0;    /* Output     :Disable  */
        TSB_PJ_OD_PJ3OD   = 1;    /* OD Control :Open Drain */
        TSB_PJ_PUP_PJ3UP  = 0;    /* Pull-up    :Disable  */
        TSB_PJ_PDN_PJ3DN  = 0;    /* Pull-down  :Disable  */
        TSB_PJ_DATA_PJ3   = 0;    /* Data       :0        */
        TSB_PJ_FR6_PJ3F6  = 1;    /* Function   :EI2C4SDA  */
        TSB_PJ_IE_PJ3IE   = 1;    /* Input      :Enable   */
        TSB_PJ_CR_PJ3C    = 1;    /* Output     :Enable   */
#endif
    }
    else if ((sda == EI2C_PORT_PM6) && (scl == EI2C_PORT_PM7))
    {
        /* Port M  */
        TSB_CG_FSYSMENB_IPMENB13 = 1;     /* PORT  M   */

        /* SCL */
        TSB_PM_IE_PM7IE   = 0;    /* Input       :Disable */
        TSB_PM_CR_PM7C    = 0;    /* Output      :Disable */
        TSB_PM_OD_PM7OD   = 1;    /* OD Control  :Open Drain */
        TSB_PM_PUP_PM7UP  = 0;    /* Pull-up     :Disable */
        TSB_PM_PDN_PM7DN  = 0;    /* Pull-down   :Disable */
        TSB_PM_DATA_PM7   = 0;    /* Data        :0       */
        TSB_PM_FR1_PM7F1  = 1;    /* Function    :EI2C4SCL */
        TSB_PM_IE_PM7IE   = 1;    /* Input       :Enable  */
        TSB_PM_CR_PM7C    = 1;    /* Output      :Enable  */

        /* SDA */
        TSB_PM_IE_PM6IE   = 0;    /* Input      :Disable  */
        TSB_PM_CR_PM6C    = 0;    /* Output     :Disable  */
        TSB_PM_OD_PM6OD   = 1;    /* OD Control :Open Drain */
        TSB_PM_PUP_PM6UP  = 0;    /* Pull-up    :Disable  */
        TSB_PM_PDN_PM6DN  = 0;    /* Pull-down  :Disable  */
        TSB_PM_DATA_PM6   = 0;    /* Data       :0        */
        TSB_PM_FR1_PM6F1  = 1;    /* Function   :EI2C4SDA  */
        TSB_PM_IE_PM6IE   = 1;    /* Input      :Enable   */
        TSB_PM_CR_PM6C    = 1;    /* Output     :Enable   */
    }
}
#endif
/*--------------------------------------------------*/
/**
  * @brief     EI2C Setting
  * @param     ch       :EI2C Channel.
  * @param     p_irqn   :Destination Address of a EI2C IRQn List.
  * @retval    non-zero :Instance Address.
  * @retval    zero     :Channel not supported.
  * @note      -
  */
/*--------------------------------------------------*/
__STATIC_INLINE uint32_t set_ei2c(uint8_t ch, uint32_t *p_irqn)
{   
    uint32_t instance = 0;

    switch (ch)
    {
        case EI2C_CH0:
            TSB_CG_FSYSMENC_IPMENC05 = 1;     /* EI2C   0   */
            instance = (uint32_t)TSB_EI2C0;
            *p_irqn = (uint32_t)&EI2C_CH0_IRQN_TBL;
            break;

        case EI2C_CH1:
            TSB_CG_FSYSMENC_IPMENC06 = 1;     /* EI2C   1   */
            instance = (uint32_t)TSB_EI2C1;
            *p_irqn = (uint32_t)&EI2C_CH1_IRQN_TBL;
            break;

        case EI2C_CH2:
            TSB_CG_FSYSMENC_IPMENC07 = 1;     /* EI2C   2   */
            instance = (uint32_t)TSB_EI2C2;
            *p_irqn = (uint32_t)&EI2C_CH2_IRQN_TBL;
            break;

#if (defined(TMPM4GR) || defined(TMPM4GQ))
        case EI2C_CH3:
            TSB_CG_FSYSMENC_IPMENC08 = 1;     /* EI2C   3   */
            instance = (uint32_t)TSB_EI2C3;
            *p_irqn = (uint32_t)&EI2C_CH3_IRQN_TBL;
            break;

        case EI2C_CH4:
            TSB_CG_FSYSMENC_IPMENC09 = 1;     /* EI2C   4   */
            instance = (uint32_t)TSB_EI2C4;
            *p_irqn = (uint32_t)&EI2C_CH4_IRQN_TBL;
            break;
#endif         
        
        default:
            break;
    }
    return (instance);
}

/*--------------------------------------------------*/
/**
  * @brief     Reset Asynch Transfer
  * @param     p_obj    :ei2c object
  * @retval    -
  * @note      -
  */
/*--------------------------------------------------*/
__STATIC_INLINE void reset_asynch(bsp_ei2c_t *p_obj)
{
    disable_irq(p_obj->info.irqn);
    EI2C_disable_interrupt(&p_obj->ei2c);
}

/*--------------------------------------------------*/
/**
  * @brief     EI2C Transfer handler
  * @param     p_obj     :ei2c object.
  * @retval    -
  * @note      Called by ei2c_irq_handler_asynch.
  */
/*--------------------------------------------------*/
static void ei2c_irq_handler(bsp_ei2c_t *p_obj)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    assert_param(IS_POINTER_NOT_NULL(p_obj));
#endif /* #ifdef __DEBUG__ */

    if(p_obj->info.asynch.state != EI2C_TRANSFER_STATE_BUSY)
    {
        p_obj->info.asynch.event = EI2C_EVENT_ERROR;
        p_obj->info.asynch.state = EI2C_TRANSFER_STATE_IDLE;
    }
    else
    { 
        /* check ACK */
       if (!EI2C_get_ack(&p_obj->ei2c))
       {
          if (EI2C_transmitter(&p_obj->ei2c))
          {
              /* transmit data */
              if (p_obj->tx_buff.pos < p_obj->tx_buff.length)
              {
                 if(EI2C_status_tbe(&p_obj->ei2c))
                 {
                    EI2C_write_data(&p_obj->ei2c, (uint32_t)p_obj->tx_buff.p_buffer[p_obj->tx_buff.pos++]);
                 }
              }
           }
           else
           {
               /*  receive data */
               if (p_obj->rx_buff.pos < p_obj->rx_buff.length)
               {
                  /* read data */
                  p_obj->rx_buff.p_buffer[p_obj->rx_buff.pos++] = (uint8_t)EI2C_read_data(&p_obj->ei2c);
               
                  /* when next transmit over set ACKWAIT */
                  if (p_obj->rx_buff.pos == (p_obj->rx_buff.length-1))
                  {
                      EI2C_set_ack(&p_obj->ei2c,1);
                  }
                  /* when transmit over set ACKSEL_NACK */
                  else if (p_obj->rx_buff.pos == p_obj->rx_buff.length)
                  {
                     p_obj->ei2c.p_instance->ACR1 |= EI2CxCR1_ACKSEL_NACK;
                     EI2C_stop_condition(&p_obj->ei2c);                      
                  }
               }
            }
         }
         else
         {
             /* when status NACK,stop condition */
              EI2C_stop_condition(&p_obj->ei2c);
              p_obj->ei2c.p_instance->ASR1 |= EI2CxSR1_NACK;
         }
     }
   
    if (p_obj->info.asynch.state == EI2C_TRANSFER_STATE_IDLE)
    {
       reset_asynch(p_obj);
    }
}

/*--------------------------------------------------*/
/**
  * @brief     EI2C Slave handler
  * @param     p_obj     :ei2c object.
  * @retval    -
  * @note      Called by ei2c_slave_irq_handler_asynch.
  */
/*--------------------------------------------------*/
static void ei2c_slave_irq_handler(bsp_ei2c_t *p_obj)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    assert_param(IS_POINTER_NOT_NULL(p_obj));
#endif /* #ifdef __DEBUG__ */

    if ((EI2C_master(&p_obj->ei2c)) || (p_obj->info.asynch.state != EI2C_TRANSFER_STATE_BUSY))
    {
        p_obj->info.asynch.event = EI2C_EVENT_ERROR;
        p_obj->info.asynch.state = EI2C_TRANSFER_STATE_IDLE;
    }
    else
    {
        if (EI2C_transmitter(&p_obj->ei2c))
        {
            if (!EI2C_get_ack(&p_obj->ei2c))
            {
                /* slave trnasmit(write data)  */
                if (p_obj->tx_buff.pos < p_obj->tx_buff.length)
                {
                    if(EI2C_status_tbe(&p_obj->ei2c))
                    {
                       EI2C_write_data(&p_obj->ei2c, (uint32_t)p_obj->tx_buff.p_buffer[p_obj->tx_buff.pos++]);
                    }
                }
            }
            else
            {
                /* error event not be set */
                p_obj->info.asynch.event = EI2C_EVENT_TRANSFER_COMPLETE;
                p_obj->info.asynch.state = EI2C_TRANSFER_STATE_IDLE;
            }
        }
        else
        {
            /*  receive data */
            if (p_obj->rx_buff.pos < p_obj->rx_buff.length)
            {
                /* read data */
                p_obj->rx_buff.p_buffer[p_obj->rx_buff.pos++] = (uint8_t)EI2C_read_data(&p_obj->ei2c);
               
                /* when next transmit over set ACKWAIT */
                if (p_obj->rx_buff.pos == (p_obj->rx_buff.length-1))
                {
                    p_obj->ei2c.p_instance->ACR1 |= EI2CxCR1_ACKWAIT;
                }
                /* when transmit over set ACKSEL_NACK */
                else if (p_obj->rx_buff.pos == p_obj->rx_buff.length)
                {
                    p_obj->ei2c.p_instance->ACR1 |= EI2CxCR1_ACKSEL_NACK;                   
                }
            }
        }
    }
 
    if (p_obj->info.asynch.state == EI2C_TRANSFER_STATE_IDLE)
    {
        reset_asynch(p_obj);
        EI2C_slave_init(&p_obj->ei2c);
    }
}

/*--------------------------------------------------*/
/**
  * @brief     Enable EI2C IRQ
  * @param     p_obj     :ei2c object.
  * @retval    -
  * @note      -
  */
/*--------------------------------------------------*/
void ei2c_enable_irq(bsp_ei2c_t *p_obj)
{
    enable_irq(p_obj->info.irqn);
}

/*--------------------------------------------------*/
/**
  * @brief     Disable EI2C IRQ
  * @param     p_obj     :ei2c object.
  * @retval    -
  * @note      -
  */
/*--------------------------------------------------*/
void ei2c_disable_irq(bsp_ei2c_t *p_obj)
{
    disable_irq(p_obj->info.irqn);
}

/*--------------------------------------------------*/
/**
  * @brief     Return the EI2c clock setting
  * @param     p_obj       :EI2C object.
  * @param     frequency   :Maximum frequency.
  * @param     fsys        :SystemCoreClock.
  * @param     p_setting   :Clock data pointer.
  * @retval    Non-zero    :Scl frequency.
  * @retval    0           :Error.
  * @note      -
  */
/*--------------------------------------------------*/
uint8_t ei2c_get_dnf_num(uint32_t dnf)
{
    uint8_t dnf_num = 0;

    switch(dnf)
    {
        case EI2CxCR0_DNF_1PRSCK:
        case EI2CxCR0_DNF_NA7:
        case EI2CxCR0_DNF_NA8:
             dnf_num =0;
             break;

        case EI2CxCR0_DNF_2PRSCK:
             dnf_num =1;
             break;

        case EI2CxCR0_DNF_3PRSCK:
             dnf_num =2;
             break;

        case EI2CxCR0_DNF_4PRSCK:
             dnf_num =3;
             break;

        case EI2CxCR0_DNF_5PRSCK:
             dnf_num =4;
             break;

        case EI2CxCR0_DNF_6PRSCK:
             dnf_num =5;
             break;

        default:
            dnf_num =0;
            break;  
    }
    return dnf_num;
}
/**
 *  @}
 */ /* End of group UTILITIES_Private_functions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup UTILITIES_Exported_functions
 *  @{
 */

/*--------------------------------------------------*/
/**
  * @brief     Initialize the EI2C Driver
  * @param     p_obj       :ei2c object.
  * @param     sda         :SDA port.
  * @param     scl         :SCL port.
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
TXZ_Result ei2c_init(bsp_ei2c_t *p_obj, ei2c_port_t sda, ei2c_port_t scl)
{
    TXZ_Result result = TXZ_ERROR;
    uint32_t instance = 0;
    uint32_t irqn = 0;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    assert_param(IS_POINTER_NOT_NULL(p_obj));
#endif /* #ifdef __DEBUG__ */

    /* ch0 */
    if ((sda == EI2C_PORT_PG2) && (scl == EI2C_PORT_PG3))
    {
        set_port_ch0(sda, scl);
        instance = set_ei2c(EI2C_CH0, &irqn);
    }
    /* ch1 */
    if ((sda == EI2C_PORT_PF2) && (scl == EI2C_PORT_PF3))
    {
        set_port_ch1(sda, scl);
        instance = set_ei2c(EI2C_CH1, &irqn);
    }
    /* ch2 */
    if(((sda == EI2C_PORT_PG4) && (scl == EI2C_PORT_PG5))
    || ((sda == EI2C_PORT_PV5) && (scl == EI2C_PORT_PV4)))
    {
        set_port_ch2(sda, scl);
        instance = set_ei2c(EI2C_CH2, &irqn);
    }
    
#if (defined(TMPM4GR) || defined(TMPM4GQ))    
    /* ch3 */
    if(((sda == EI2C_PORT_PJ6) && (scl == EI2C_PORT_PJ7))
    || ((sda == EI2C_PORT_PM0) && (scl == EI2C_PORT_PM1)))
    {
        set_port_ch3(sda, scl);
        instance = set_ei2c(EI2C_CH3, &irqn);
    }
    /* ch4 */
    if(((sda == EI2C_PORT_PJ3) && (scl == EI2C_PORT_PJ2))
    || ((sda == EI2C_PORT_PM6) && (scl == EI2C_PORT_PM7)))
    {
        set_port_ch4(sda, scl);
        instance = set_ei2c(EI2C_CH4, &irqn);
    }
#endif
    
    if ((instance != 0) && (irqn != 0))
    {
        disable_irq(irqn);
        clear_irq(irqn);

        /* Set irqn table */
        p_obj->info.irqn = irqn;

        /* Set instance */
        p_obj->ei2c.p_instance = (TSB_EI2C_TypeDef *)instance;

        /* EI2C Reset */
        ei2c_reset(p_obj);

        /* set prsclk scl information */
        result = ei2c_set_prs_scl(&p_obj->ei2c);

        if( result != TXZ_ERROR )
        {
          /* control0 Register setting */
          p_obj->ei2c.init.cr0.toper = EI2CxCR0_TOPER_4PRSCK;
          p_obj->ei2c.init.cr0.toe   = EI2CxCR0_TOE_ENABLE;
          p_obj->ei2c.init.cr0.espe  = EI2CxCR0_ESPE_ENABLE;
          p_obj->ei2c.init.cr0.este  = EI2CxCR0_ESTE_ENABLE;
          p_obj->ei2c.init.cr0.nacke = EI2CxCR0_NACKE_ENABLE;
          p_obj->ei2c.init.cr0.gce   = EI2CxCR0_GCE_ENABLE;
          p_obj->ei2c.init.cr0.ale   = EI2CxCR0_ALE_ENABLE;

          /* EI2C driver Register init */
          EI2C_init(&p_obj->ei2c);
        }
    }
    return (result);
}

/*--------------------------------------------------*/
/**
  * @brief     Return the EI2c clock setting
  * @param     p_obj       :EI2C object.
  * @param     frequency   :Maximum frequency.
  * @param     fsys        :SystemCoreClock.
  * @param     p_setting   :Clock data pointer.
  * @retval    Non-zero    :Scl frequency.
  * @retval    0           :Error.
  * @note      -
  */
/*--------------------------------------------------*/
TXZ_Result ei2c_set_prs_scl_dig(ei2c_t *p_obj)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    assert_param(IS_POINTER_NOT_NULL(p_obj));
#endif /* #ifdef __DEBUG__ */

    uint32_t frequency,prs,prs_tmp;
    uint32_t prsck;
    uint32_t scl_tf,scl_tr;
    uint32_t sclh,sclh_tmp,scll;
    int32_t scll_tmp;
    uint32_t t_total,t_high,t_low;
    uint32_t low_width_min;
    uint32_t high_width_min;
    cg_t paramCG;
    uint32_t fsysm;
    uint8_t dnf;

    frequency = p_obj->init.clock.sck;
    paramCG.p_instance = TSB_CG;
    fsysm = cg_get_fsysm(&paramCG);
    dnf=ei2c_get_dnf_num(p_obj->init.cr0.dnf);

    /* unuse value init */
    prs  = EI2C_UNUSE_VALUE;
    sclh = EI2C_UNUSE_VALUE;
    scll = EI2C_UNUSE_VALUE;

    if(frequency <= EI2CSCL_FRQ_FMP)
    {
        /* STD */
        if(frequency <= EI2CSCL_FRQ_STD)
        {
            high_width_min = EI2CSCL_SCLH_STDMIN;
            low_width_min  = EI2CSCL_SCLL_STDMIN;
            scl_tf = EI2CSCL_TF_STD;
            scl_tr = EI2CSCL_TR_STD;
        }
        /* FM */
        else if(frequency <= EI2CSCL_FRQ_FM)
        {
            high_width_min = EI2CSCL_SCLH_FMMIN;
            low_width_min  = EI2CSCL_SCLL_FMMIN;
            scl_tf = EI2CSCL_TF_FM;
            scl_tr = EI2CSCL_TR_FM;
        }
        /* FMP */
        else
        {
            high_width_min = EI2CSCL_SCLH_FMPMIN;
            low_width_min  = EI2CSCL_SCLL_FMPMIN;
            scl_tf = EI2CSCL_TF_FMP;
            scl_tr = EI2CSCL_TR_FMP;
        }

        /*---------------------------------------------------------*/
        /* freq(hz)=(1/(tHIGH(ns)+tLOW(ns)+tr(ns)+tf(ns)))*(10^9)  */
        /* tHIGH(ns)+tLOW(ns)=(10^9/freq(hz))-tr(ns)-tf(ns)        */
        /*---------------------------------------------------------*/
        t_total = (EI2C_UNIT_1US*1000/frequency) - scl_tf - scl_tr;

        /* if small than min return */
        if(t_total < (high_width_min + low_width_min))
        {
           result = TXZ_ERROR;
           return result;
        }

       for (prs_tmp = 0; prs_tmp < EI2CxPRS_MAX; prs_tmp++)
       {
           /*-----------------------------------------------*/
           /* clock width (s) = 1 / PCLK                    */
           /* clock width (s) = 1 / (fsysm / PRS)           */
           /* clock width(ns) = (10^9) / (fsysm / PRS)      */
           /*-----------------------------------------------*/
           /* NFSEL=0: (50ns/(<DNF>+1))<=clock width(ns)    */
           /*            <=(tHIGH(Min)/(<DNF>+2)            */
           /* ((10^9)*(<DNF>+2))/(tHIGH(Min) <= (fsysm/PRS) */
           /*            <=((10^9)*(<DNF>+1))/(50ns)        */
           /*-----------------------------------------------*/
           /* PRS: 1~64   DNF:1~6                           */
           /* (tHIGH(Min): Use @ref EI2CSCL_MAX             */
           /*-----------------------------------------------*/
           prsck = ((uint32_t)fsysm /((prs_tmp+1)*1000));

           if((prsck < ((EI2C_UNIT_1US/high_width_min)*(dnf+2)))
           || (prsck > ((EI2C_UNIT_1US/50)*(dnf+1))))
           {
              continue;
           }

           
           if(t_total == (high_width_min + low_width_min))
           {
                t_high = high_width_min;
                t_low = low_width_min;
                sclh_tmp = ((t_high*prsck)/EI2C_UNIT_1US) - 3 - dnf;
                scll_tmp = ((t_low*prsck)/EI2C_UNIT_1US) - 3 - dnf;
               
                /* paramater check */
                if(( sclh_tmp <= EI2CxSCL_MAX )
                && ( scll_tmp <= EI2CxSCL_MAX ))
                {
                     prs  = prs_tmp;
                     sclh = sclh_tmp;
                     scll = scll_tmp;
                     break;
                }
           }
           else
           {
               /*-----------------------------------------------*/
               /* t_high(ns)=((SCLH+3+DNF)/prsck(Hz))*(10^9)    */
               /* t_low(ns) =((SCLL+3+DNF)/prsck(Hz))*(10^9)    */
               /* t_high(ns) + t_low(ns) = t_total              */
               /* t_high(ns)>= high_width_min                   */
               /* t_low(ns) >= low_width_min                    */
               /*-----------------------------------------------*/
               for(sclh_tmp = 0; sclh_tmp <= EI2CxSCL_MAX; sclh_tmp++)
               {
                   t_high = ((sclh_tmp + dnf + 3)*EI2C_UNIT_1US)/prsck;
                   t_low  = t_total - t_high;

                   /*  paramater check */
                   if((t_high < high_width_min)
                   || (t_low  < low_width_min))
                   {
                      continue;
                   }
                   
                   scll_tmp = ((t_low*prsck)/EI2C_UNIT_1US) - 3 - dnf;
                   
                   /* paramater check */
                   if(( scll_tmp >= 0 )
                   && ( scll_tmp <= EI2CxSCL_MAX ))
                   {
                       prs  = prs_tmp;
                       sclh = sclh_tmp;
                       scll = scll_tmp;
                       break;
                   }
               }
            }

           /*  when set over break the loop  */
           if((prs != EI2C_UNUSE_VALUE)
           &&(sclh != EI2C_UNUSE_VALUE)
           &&(scll != EI2C_UNUSE_VALUE))
           {
              break;
           }
       }

       /* set the paramater value */
       p_obj->init.clock.prs = prs;
       p_obj->init.clock.sclh = sclh;
       p_obj->init.clock.scll = scll;
   }
   else
   {
      result = TXZ_ERROR;
   }

return (result);
}
/*--------------------------------------------------*/
/**
  * @brief     Return the EI2c clock setting
  * @param     p_obj       :EI2C object.
  * @param     frequency   :Maximum frequency.
  * @param     fsys        :SystemCoreClock.
  * @param     p_setting   :Clock data pointer.
  * @retval    Non-zero    :Scl frequency.
  * @retval    0           :Error.
  * @note      -
  */
/*--------------------------------------------------*/
TXZ_Result ei2c_set_prs_scl_ana(ei2c_t *p_obj)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    assert_param(IS_POINTER_NOT_NULL(p_obj));
#endif /* #ifdef __DEBUG__ */

    uint32_t frequency,prs,prs_tmp;
    uint32_t prsck;
    uint32_t scl_tf,scl_tr;
    uint32_t sclh,sclh_tmp,scll;
    int32_t scll_tmp;
    uint32_t t_total,t_high,t_low;
    uint32_t low_width_min;
    uint32_t high_width_min;
    cg_t paramCG;
    uint32_t fsysm;

    frequency = p_obj->init.clock.sck;
    paramCG.p_instance = TSB_CG;
    fsysm = cg_get_fsysm(&paramCG);

    /* unuse value init */
    prs  = EI2C_UNUSE_VALUE;
    sclh = EI2C_UNUSE_VALUE;
    scll = EI2C_UNUSE_VALUE;

    if(frequency <= EI2CSCL_FRQ_FMP)
    {
        /* STD */
         if(frequency <= EI2CSCL_FRQ_STD)
         {
            high_width_min = EI2CSCL_SCLH_STDMIN;
            low_width_min  = EI2CSCL_SCLL_STDMIN;
            scl_tf = EI2CSCL_TF_STD;
            scl_tr = EI2CSCL_TR_STD;
         }
         /* FM */
         else if(frequency <= EI2CSCL_FRQ_FM)
         {
             high_width_min = EI2CSCL_SCLH_FMMIN;
             low_width_min  = EI2CSCL_SCLL_FMMIN;
             scl_tf = EI2CSCL_TF_FM;
             scl_tr = EI2CSCL_TR_FM;
         }
         /* FMP */
         else
         {
             high_width_min = EI2CSCL_SCLH_FMPMIN;
             low_width_min  = EI2CSCL_SCLL_FMPMIN;
             scl_tf = EI2CSCL_TF_FMP;
             scl_tr = EI2CSCL_TR_FMP;
         }

        /*---------------------------------------------------------*/
        /* freq(hz)=(1/(tHIGH(ns)+tLOW(ns)+tr(ns)+tf(ns)))*(10^9)  */
        /* tHIGH(ns)+tLOW(ns)=(10^9/freq(hz))-tr(ns)-tf(ns)        */
        /*---------------------------------------------------------*/
        t_total = (EI2C_UNIT_1US*1000/frequency) - scl_tf - scl_tr;
        
        /* if small than min return */
        if(t_total < (high_width_min + low_width_min))
        {
            result = TXZ_ERROR;
            return result;
        }
        
        for (prs_tmp = 0; prs_tmp < EI2CxPRS_MAX; prs_tmp++)
        {
            /*-----------------------------------------------*/
            /* clock width (s) = 1 / PRSCK                   */
            /* clock width (s) = 1 / (fsysm /(PRS+1))        */
            /* clock width(ns) = (10^9) / (fsysm/(PRS+1))    */
            /*-----------------------------------------------*/
            /* NFSEL=1: clock width(ns)<= (tHIGH(Min)/2)     */
            /* ((10^9)*2)/(tHIGH(Min)) <= (fsysm/(PRS+1))    */
            /*-----------------------------------------------*/
            /* PRS: 0~63                                     */
            /* (tHIGH(Min): Use @ref EI2CSCL_MAX             */
            /*-----------------------------------------------*/
            prsck = ((uint32_t)fsysm /((prs_tmp+1)*1000));

            if(prsck < ((EI2C_UNIT_1US*2)/high_width_min))
            {
               continue;
            }

            /*-----------------------------------------------*/
            /* t_high(ns)=((SCLH+ 2)/prsck(Hz))*(10^9)       */
            /* t_low(ns) =((SCLL+ 2)/prsck(Hz))*(10^9)       */
            /* t_high(ns) + t_low(ns) = t_total              */
            /* t_high(ns)>= high_width_min                   */
            /* t_low(ns) >= low_width_min                    */
            /*-----------------------------------------------*/
            for(sclh_tmp = 0; sclh_tmp <= EI2CxSCL_MAX; sclh_tmp++)
            {
               t_high = ((sclh_tmp + 2)*EI2C_UNIT_1US)/prsck;
               t_low  = t_total - t_high;
               
               /*  paramater chec */
               if((t_high < high_width_min)
               || (t_low  < low_width_min))
               {
                  continue;
               }
               
               scll_tmp = (t_low*prsck)/EI2C_UNIT_1US - 2;
               
               /*  paramater chec */
               if(( scll_tmp >= 0 )
               && ( scll_tmp <= EI2CxSCL_MAX ))
               {
                  prs  = prs_tmp;
                  sclh = sclh_tmp;
                  scll = scll_tmp;
                  break;
               }
            }

            /* when set over break the loop */
            if((prs != EI2C_UNUSE_VALUE)
            &&(sclh != EI2C_UNUSE_VALUE)
            &&(scll != EI2C_UNUSE_VALUE))
            {
              break;
            }
       }

       /* set the paramater value */
       p_obj->init.clock.prs = prs;
       p_obj->init.clock.sclh = sclh;
       p_obj->init.clock.scll = scll;
    }
    else
    {
        result = TXZ_ERROR;
    }

    return (result);
}
/*--------------------------------------------------*/
/**
  * @brief     Return the EI2c clock setting
  * @param     p_obj       :EI2C object.
  * @param     frequency   :Maximum frequency.
  * @param     fsys        :SystemCoreClock.
  * @param     p_setting   :Clock data pointer.
  * @retval    Non-zero    :Scl frequency.
  * @retval    0           :Error.
  * @note      -
  */
/*--------------------------------------------------*/
TXZ_Result ei2c_set_prs_scl(ei2c_t *p_obj)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    assert_param(IS_POINTER_NOT_NULL(p_obj));
#endif /* #ifdef __DEBUG__ */

    /* Digtal noise filter */
    if(p_obj->init.cr0.nfsel == EI2CxCR0_NFSEL_DIG)
    {
       result = ei2c_set_prs_scl_dig(p_obj);
    }
    /* Analog noise filter */
    else if(p_obj->init.cr0.nfsel == EI2CxCR0_NFSEL_ANA)
    {
       result = ei2c_set_prs_scl_ana(p_obj);
    }
    /* other */
    else
    {
       result = TXZ_ERROR;
    }

    return (result);
}
/*--------------------------------------------------*/
/**
  * @brief     Reset EI2C peripheral
  * @param     p_obj       :ei2c object.
  * @retval    -
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
void ei2c_reset(bsp_ei2c_t *p_obj)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    assert_param(IS_POINTER_NOT_NULL(p_obj));
#endif /* #ifdef __DEBUG__ */

    /* Software reset */
    EI2C_reset(&p_obj->ei2c);
}

/*--------------------------------------------------*/
/**
  * @brief     Reset EI2C peripheral
  * @param     p_obj       :ei2c object.
  * @retval    -
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
void ei2c_acr1_clear(bsp_ei2c_t *p_obj)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    assert_param(IS_POINTER_NOT_NULL(p_obj));
#endif /* #ifdef __DEBUG__ */

    /* Software reset */
    EI2C_acr1_clear(&p_obj->ei2c);
}

/*--------------------------------------------------*/
/**
  * @brief     Attempts to determine if the EI2C bus is already in use
  * @param     p_obj     :ei2c object.
  * @retval    0         :Non-active.
  * @retval    1         :Active.
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
uint8_t ei2c_active(bsp_ei2c_t *p_obj)
{
    uint8_t result;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    assert_param(IS_POINTER_NOT_NULL(p_obj));
#endif /* #ifdef __DEBUG__ */

    if (EI2C_status_busy(&p_obj->ei2c))
    {
        result = 1;     
    }
    else
    {
        result = 0;
    }
    return (result);
}

/*--------------------------------------------------*/
/**
  * @brief     Start EI2C asynchronous transfer
  * @param     p_obj     :ei2c object.
  * @param     p_tx      :Buffer of write data.
  * @param     tx_length :Length of write data.
  * @param     p_rx      :Buffer of read data.
  * @param     rx_length :Length of read data.
  * @param     address   :Slave address(7-bit) and last bit is 0.
  * @param     stop      :Stop to be generated after the transfer is done.
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      Master and non-blocking function. 
  * @note      Events of this function will be notified on ei2c_irq_handler_asynch.
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
TXZ_Result ei2c_transfer_asynch(bsp_ei2c_t *p_obj, uint8_t *p_tx, int32_t tx_length, uint8_t *p_rx, int32_t rx_length, int32_t address, int32_t stop)
{
    TXZ_Result result = TXZ_ERROR;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(check_param_address(address));
#endif /* #ifdef __DEBUG__ */

    if (p_obj->info.asynch.state == EI2C_TRANSFER_STATE_IDLE)
    {
        if (((p_tx != EI2C_NULL) && (tx_length > 0)) || ((p_rx != EI2C_NULL) && (rx_length > 0)))
        {
            reset_asynch(p_obj);
            EI2C_clear_int_status(&p_obj->ei2c);
            clear_irq(p_obj->info.irqn);
            p_obj->info.asynch.address = (uint32_t)address;
            p_obj->info.asynch.event = 0;
            p_obj->info.asynch.stop = (uint32_t)stop;
            p_obj->tx_buff.p_buffer = p_tx;
            p_obj->tx_buff.length = (uint32_t)tx_length;
            p_obj->tx_buff.pos = 0;
            p_obj->rx_buff.p_buffer = p_rx;
            p_obj->rx_buff.length = (uint32_t)rx_length;
            p_obj->rx_buff.pos = 0;
            p_obj->info.asynch.state = EI2C_TRANSFER_STATE_BUSY;
            EI2C_enable_interrupt(&p_obj->ei2c);
            enable_irq(p_obj->info.irqn);
            EI2C_startcondition(&p_obj->ei2c);
                 
            p_obj->info.bus_free = 0;
            p_obj->info.start = 0;
            result = TXZ_SUCCESS;
        }
    }
    return (result);
}

/*--------------------------------------------------*/
/**
  * @brief     ei2c_master_st_irq_handler
  * @param     p_obj     :ei2c object.
  * @retval    -
  * @note      ei2c status interrupt.
  */
/*--------------------------------------------------*/
uint32_t ei2c_master_st_irq_handler(bsp_ei2c_t *p_obj)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    assert_param(IS_POINTER_NOT_NULL(p_obj));
#endif /* #ifdef __DEBUG__ */

    if(EI2C_status_stcf(&p_obj->ei2c))
    {
        /* clear start condition&tbe status */
        EI2C_clear_int_stcf(&p_obj->ei2c);
        /* transmit over change to receive */
        if((p_obj->tx_buff.length == 0)
        && (p_obj->rx_buff.length != 0))
        {          
          if (p_obj->rx_buff.length == 1) 
          {
             EI2C_set_ack(&p_obj->ei2c,1);          
          }
          /* clear restart condition&tbe status */
          EI2C_write_data(&p_obj->ei2c, (uint32_t)(p_obj->info.asynch.address|1U));   
        }
    	else if(p_obj->tx_buff.length != 0)
        {
          EI2C_write_data(&p_obj->ei2c, (uint32_t)p_obj->info.asynch.address);    
    	}
        EI2C_clear_int_tbe(&p_obj->ei2c);  
    }
    /* restart condition */
    else if(EI2C_status_rscf(&p_obj->ei2c))
    {
        /* clear restart condition&tbe status */
         EI2C_clear_int_rscf(&p_obj->ei2c);
        /* transmit over change to receive */
        if((p_obj->tx_buff.pos == p_obj->tx_buff.length)
        && (p_obj->rx_buff.length != 0))
        {
          if (p_obj->rx_buff.length == 1) 
          {
             EI2C_set_ack(&p_obj->ei2c,1);          
          }
          EI2C_write_data(&p_obj->ei2c, (uint32_t)(p_obj->info.asynch.address|1U));   
        }
        EI2C_clear_int_tbe(&p_obj->ei2c);  
    }
    else if(EI2C_status_spcf(&p_obj->ei2c))
    {
         EI2C_clear_int_status(&p_obj->ei2c);
         p_obj->info.asynch.event = EI2C_EVENT_TRANSFER_COMPLETE;
         p_obj->info.asynch.state = EI2C_TRANSFER_STATE_IDLE;
    }
    else if(EI2C_status_tend(&p_obj->ei2c))
    {
         if(p_obj->tx_buff.pos < p_obj->tx_buff.length)
         {
              EI2C_clear_int_tend(&p_obj->ei2c);
         }
         /* check the receive buff */
         else if(p_obj->rx_buff.length != 0)
         {
             /* restart condition*/
             EI2C_clear_int_tend(&p_obj->ei2c);
             EI2C_restartcondition(&p_obj->ei2c);
         }
         else
         {
             /* transmit over clear the tend and stop condition */
             EI2C_stop_condition(&p_obj->ei2c);
             EI2C_clear_int_tend(&p_obj->ei2c); 
         }
    }
 
    if (p_obj->info.asynch.state == EI2C_TRANSFER_STATE_IDLE)
    {
       reset_asynch(p_obj);
    }
    
    return (p_obj->info.asynch.event & EI2C_EVENT_ALL);
}

/*--------------------------------------------------*/
/**
  * @brief     ei2c_slave_st_irq_handler
  * @param     p_obj     :ei2c object.
  * @retval    -
  * @note      ei2c status interrupt.
  */
/*--------------------------------------------------*/
uint32_t ei2c_slave_st_irq_handler(bsp_ei2c_t *p_obj)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    assert_param(IS_POINTER_NOT_NULL(p_obj));
#endif /* #ifdef __DEBUG__ */

    if((p_obj->ei2c.p_instance->ASR1 & EI2CxSR1_AAS1) == EI2CxSR1_AAS1)
    {
        /* clear AAS1 status */        
         p_obj->ei2c.p_instance->ASR1 |= EI2CxSR1_AAS1; 
    }
    else if((p_obj->ei2c.p_instance->ASR1 & EI2CxSR1_AAS2) == EI2CxSR1_AAS2)
    {
        /* clear AAS1 status */        
         p_obj->ei2c.p_instance->ASR1 |= EI2CxSR1_AAS2; 
    }
    else if(EI2C_status_spcf(&p_obj->ei2c))
    {
         EI2C_clear_int_status(&p_obj->ei2c);
         p_obj->info.asynch.event = EI2C_EVENT_TRANSFER_COMPLETE;
         p_obj->info.asynch.state = EI2C_TRANSFER_STATE_IDLE;
    }
    else if(EI2C_status_tend(&p_obj->ei2c))
    {
         EI2C_clear_int_tend(&p_obj->ei2c);
    }

    if (p_obj->info.asynch.state == EI2C_TRANSFER_STATE_IDLE)
    {
        reset_asynch(p_obj);
        EI2C_slave_init(&p_obj->ei2c);
    }
    
    return (p_obj->info.asynch.event & EI2C_EVENT_ALL);
}
/*--------------------------------------------------*/
/**
  * @brief     The asynchronous IRQ handler
  * @param     p_obj     :ei2c object.
  * @retval    zero      :Transfer in progress.
  * @retval    non-zero  :Event information.
  * @note      Macro definition of return values is @ref EI2C_Events.
  * @attention This function should be implement as INTI2Cx_IRQHandler.
  */
/*--------------------------------------------------*/
uint32_t ei2c_irq_handler_asynch(bsp_ei2c_t *p_obj)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    assert_param(IS_POINTER_NOT_NULL(p_obj));
#endif /* #ifdef __DEBUG__ */

    ei2c_irq_handler(p_obj);

    return (p_obj->info.asynch.event & EI2C_EVENT_ALL);
}

/*--------------------------------------------------*/
/**
  * @brief     Abort asynchronous transfer
  * @param     p_obj     :ei2c object.
  * @retval    -
  * @note      After error event occurred on ei2c_irq_handler_asynch, 
  * @note      call this function and clear error status. 
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
void ei2c_abort_asynch(bsp_ei2c_t *p_obj)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    assert_param(IS_POINTER_NOT_NULL(p_obj));
#endif /* #ifdef __DEBUG__ */

    reset_asynch(p_obj);
    p_obj->info.asynch.state = EI2C_TRANSFER_STATE_IDLE;
    ei2c_reset(p_obj);
    EI2C_init(&p_obj->ei2c);
    clear_irq(p_obj->info.irqn);
}

/*--------------------------------------------------*/
/**
  * @brief     Configure EI2C as slave or master.
  * @param     p_obj        :ei2c object.
  * @param     enable_slave :Enable slave mode.
  * @retval    -
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
void ei2c_slave_mode(bsp_ei2c_t *p_obj, int32_t enable_slave)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    assert_param(IS_POINTER_NOT_NULL(p_obj));
#endif /* #ifdef __DEBUG__ */

    disable_irq(p_obj->info.irqn);

    if (enable_slave)
    {
        EI2C_slave_init(&p_obj->ei2c);
    }
    else
    {
        /* Slave Disable Settings. */
        ei2c_reset(p_obj);
        EI2C_init(&p_obj->ei2c);
    }
    p_obj->info.bus_free = 0;
    p_obj->info.start = 0;
    EI2C_clear_int_status(&p_obj->ei2c);
}

/*--------------------------------------------------*/
/**
  * @brief     Configure EI2C slave address.
  * @param     p_obj     :ei2c object.
  * @param     address   :Address to be set.
  * @param     bit7      :7bit Address or 10bit address.
  * @retval    -
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
void ei2c_slave_address1(bsp_ei2c_t *p_obj, uint32_t address,int32_t bit7)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(check_param_address((int32_t)address));
#endif /* #ifdef __DEBUG__ */

    EI2C_set_address1(&p_obj->ei2c, address,bit7);
}


/*--------------------------------------------------*/
/**
  * @brief     Start EI2C asynchronous transfer
  * @param     p_obj     :ei2c object.
  * @param     p_tx      :Buffer of write data.
  * @param     tx_length :Length of write data.
  * @param     p_rx      :Buffer of read data.
  * @param     rx_length :Length of read data.
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      Slave and non-blocking function.
  * @note      Events of this function will be notified on ei2c_slave_irq_handler_asynch.
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
TXZ_Result ei2c_slave_transfer_asynch(bsp_ei2c_t *p_obj, uint8_t *p_tx, int32_t tx_length, uint8_t *p_rx, int32_t rx_length)
{
    TXZ_Result result = TXZ_ERROR;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    assert_param(IS_POINTER_NOT_NULL(p_obj));
#endif /* #ifdef __DEBUG__ */

    if (p_obj->info.asynch.state == EI2C_TRANSFER_STATE_IDLE)
    {
        if (((p_tx != EI2C_NULL) && (tx_length > 0)) || ((p_rx != EI2C_NULL) && (rx_length > 0)))
        {
            reset_asynch(p_obj);
            EI2C_clear_int_status(&p_obj->ei2c);
            clear_irq(p_obj->info.irqn);
            p_obj->info.asynch.address = 0;
            p_obj->info.asynch.event = 0;
            p_obj->info.asynch.stop = 0;
            p_obj->tx_buff.p_buffer = p_tx;
            p_obj->tx_buff.length = (uint32_t)tx_length;
            p_obj->tx_buff.pos = 0;
            p_obj->rx_buff.p_buffer = p_rx;
            p_obj->rx_buff.length = (uint32_t)rx_length;
            p_obj->rx_buff.pos = 0;
            p_obj->info.asynch.state = EI2C_TRANSFER_STATE_BUSY;
            EI2C_enable_interrupt(&p_obj->ei2c);
            enable_irq(p_obj->info.irqn);
            result = TXZ_SUCCESS;
        }
    }
    return (result);
}

/*--------------------------------------------------*/
/**
  * @brief     The asynchronous IRQ handler
  * @param     p_obj     :ei2c object.
  * @retval    zero      :Transfer in progress.
  * @retval    non-zero  :Event information.
  * @note      Macro definition of return values is @ref EI2C_Events.
  * @attention This function should be implement as INTI2Cx_IRQHandler.
  */
/*--------------------------------------------------*/
uint32_t ei2c_slave_irq_handler_asynch(bsp_ei2c_t *p_obj)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    assert_param(IS_POINTER_NOT_NULL(p_obj));
#endif /* #ifdef __DEBUG__ */

    ei2c_slave_irq_handler(p_obj);

    return (p_obj->info.asynch.event & EI2C_EVENT_ALL);
}

/*--------------------------------------------------*/
/**
  * @brief     Abort asynchronous transfer
  * @param     p_obj     :ei2c object.
  * @retval    -
  * @note      For a non-blocking function.
  * @note      After error event occurred on ei2c_slave_irq_handler_asynch, 
  * @note      call this function and clear error status. 
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
void ei2c_slave_abort_asynch(bsp_ei2c_t *p_obj)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    assert_param(IS_POINTER_NOT_NULL(p_obj));
#endif /* #ifdef __DEBUG__ */

    reset_asynch(p_obj);
    p_obj->info.asynch.state = EI2C_TRANSFER_STATE_IDLE;
    EI2C_slave_init(&p_obj->ei2c);
    EI2C_clear_int_status(&p_obj->ei2c);
    clear_irq(p_obj->info.irqn);
}

/**
 *  @}
 */ /* End of group UTILITIES_Exported_functions */

/**
 *  @} 
 */ /* End of group UTILITIES */

/**
 *  @} 
 */ /* End of group Example */

#endif /* defined(__BSP_EI2C_H)  */

#ifdef __cplusplus
}
#endif /* __cplusplus */
