/** 
 *******************************************************************************
 * @file    bsp_fuart_io.c
 * @brief   FUART input/output
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2020
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif
/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include <stdio.h>
#include <stdarg.h>
#include "bsp_fuart_io.h"

#if defined(__BSP_FUART_IO_H)
/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @addtogroup BSP_UTILITIES
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup BSP_UTILITIES_Private_macro
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group BSP_UTILITIES_Private_macro */


/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup BSP_UTILITIES_Private_define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group BSP_UTILITIES_Private_define */


/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup BSP_UTILITIES_Private_define
 *  @{
 */

#define FUART_IO_NULL           ((void *)0)  /*!< Null Pointer */

/*----------------------*/
/*  for Debug           */
/*----------------------*/
#ifdef __DEBUG__
    #if 1
        #define FUART_IO_DEBUG
    #endif
#endif
/** 
 *  @}
 */ /* End of group BSP_UTILITIES_Private_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup BSP_UTILITIES_Private_define
 *  @{
 */

/** 
 *  @}
 */ /* End of group BSP_UTILITIES_Private_define */


/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup BSP_UTILITIES_Private_typedef
 *  @{
 */

/**
 *  @}
 */ /* End of group BSP_UTILITIES_Private_typedef */


/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup BSP_UTILITIES_Private_variables
 *  @{
 */
static fuart_io_t instance;  /*!< USB FUART Instance.  */
/**
 *  @}
 */ /* End of group BSP_UTILITIES_Private_variables */


/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup BSP_UTILITIES_Private_fuctions
 *  @{
 */
static void fuart_io_error(void);
static int put_char(int c);
static int get_char(void);
static void transmit_handler(uint32_t id, TXZ_Result result);
static void receive_handler(uint32_t id, TXZ_Result result, fuart_receive_t *p_info);
/*--------------------------------------------------*/
/** 
  * @brief  Error Hook
  * @param  -
  * @return -
  * @retval -
  * @note   for debug
  */
/*--------------------------------------------------*/
static void fuart_io_error(void)
{
#ifdef FUART_IO_DEBUG
    while(1)
    {
        __NOP();
    }
#endif
}

/*--------------------------------------------------*/
/** 
  * @brief      The character is shown to an output stream.
  * @param[in]  s       :Source address.
  * @retval     Suceess :Num of output data(bytes).
  * @retval     Failure :-1
  * @pre        Have finished termination of a designated character string by NULL character.
  * @note       -
  */
/*--------------------------------------------------*/
int put_s(const char *s)
{
    int ret = -1;
    fuart_io_private_tx_t *p_tx = &instance.info.tx;

    if (p_tx->state == FUART_IO_SEND_IDLE)
    {
        volatile TXZ_WorkState loopBreak = TXZ_BUSY;

        /*------------------------------*/
        /*  Check Data                  */
        /*------------------------------*/
        p_tx->wp      = 0;
        p_tx->rp      = 0;
        p_tx->state   = FUART_IO_SEND_RUN;
        while (loopBreak == TXZ_BUSY)
        {
            if(p_tx->wp < (FUART_IO_SEND_MAX-1))
            {
                if (*s != '\0')
                {
                    p_tx->buff[p_tx->wp] = (uint8_t)*s;
                    p_tx->wp++;
                    s++;
                }
                else
                {
                    loopBreak = TXZ_DONE;
                }
            }
            else
            {
                p_tx->wp = FUART_IO_SEND_MAX-1;
                p_tx->buff[p_tx->wp] = '\0';
                p_tx->wp++;
                loopBreak = TXZ_DONE;
            }
        }
    #if 0
        while(p_tx->wp < 5)
        {
               p_tx->buff[p_tx->wp] = '\0';
               p_tx->wp++;
        }
    #endif
        if (p_tx->wp != 0)
        {
            fuart_transmit_t param;
            uint32_t status;

            /*------------------------------*/
            /*  Transmit                    */
            /*------------------------------*/
            param.p_data    = (uint8_t *)&p_tx->buff[0];
            param.num       = p_tx->wp;
            if (fuart_transmitIt(&instance.info.fuart, &param) != TXZ_SUCCESS)
            {
                fuart_io_error();
            }
            /*------------------------------*/
            /*  Wait to transmit            */
            /*------------------------------*/
            loopBreak = TXZ_BUSY;
            while(loopBreak == TXZ_BUSY)
            {
                switch (p_tx->state)
                {
                case FUART_IO_SEND_RUN:
                    /* no process */
                    break;
                case FUART_IO_SEND_SUCCESS:
                    if (fuart_get_status(&instance.info.fuart, &status) == TXZ_SUCCESS)
                    {
                        if ((status & FUARTxFR_BUSY_MASK) != FUARTxFR_BUSY_FLAG_SET)
                        {
                            loopBreak = TXZ_DONE;
                        }
                    }
                    break;
                default:
                    loopBreak = TXZ_DONE;
                    break;
                }
            }
            /*------------------------------*/
            /*  Check Error                 */
            /*------------------------------*/
            if (p_tx->state == FUART_IO_SEND_FAILURE)
            {
                fuart_io_error();
            }
            /* Clear status. */
            p_tx->state = FUART_IO_SEND_IDLE;
            ret = (int)p_tx->wp;
        }
    }

    return (ret);
}
/*--------------------------------------------------*/
/** 
  * @brief  The string is acquired from an input stream.
  * @param[in]  s       :Destination address.
  * @param[in]  num
  * @retval     Suceess :Num of input data(bytes).
  * @retval     Failure :-1
  * @note   -
  */
/*--------------------------------------------------*/
#if 0
int get_s(char *s)
{
    volatile TXZ_WorkState loopBreak = TXZ_BUSY;
    fuart_io_private_rx_t *p_rx = &instance.info.rx;
    int count = 0;

    while (loopBreak == TXZ_BUSY)
    {
        if (p_rx->state == FUART_IO_RECV_RUN)
        {
            /*>>> Critical information. */
            volatile uint32_t length = p_rx->length;
            /*<<< Critical information. */

            /*------------------------------*/
            /*  Check Receive Length        */
            /*------------------------------*/
            if (length > 0)
            {
                *s = p_rx->buff[p_rx->rp];
                p_rx->rp++;
                if (p_rx->rp >= FUART_IO_RECEIVE_MAX)
                {
                    p_rx->rp = 0;
                }
                /*>>> Critical information. */
                NVIC_DisableIRQ(instance.init.irq_n);
                p_rx->length--;
                NVIC_EnableIRQ(instance.init.irq_n);
                /*<<< Critical information. */
                if (*s == '\n')
                {
                    *s = '\0';
                    loopBreak = TXZ_DONE;
                }
                else{
                    ++s;
                    ++count;
                }
            }
        }
        else
        {
            loopBreak = TXZ_DONE;
        }
    }
    /*------------------------------*/
    /*  Check Error                 */
    /*------------------------------*/
    if (p_rx->state == FUART_IO_RECV_FAILURE)
    {
        p_rx->state = FUART_IO_RECV_IDLE;
        /* Set result. */
    }
    return (count);
}
#else
int get_s(char *s, uint8_t num)
{
    int ret = 0;
    uint8_t i = 0;
    fuart_io_private_rx_t *p_rx = &instance.info.rx;
    volatile TXZ_WorkState loopBreak = TXZ_BUSY;
    
    if( num > (FUART_IO_RECEIVE_MAX-2)){
        num = FUART_IO_RECEIVE_MAX-2;
    }
    /* recive start */
    if (p_rx->state == FUART_IO_RECV_IDLE)
    {
        {
            fuart_receive_t param;

            param.p_data    = &instance.info.work[0];
            param.num       = num;
            p_rx->state   = FUART_IO_RECV_RUN;
            if (fuart_receiveIt(&instance.info.fuart, &param) != TXZ_SUCCESS)
            {
                fuart_io_error();
            }
        }
        /* Message transfer */
        {
            char s_tmp[4] = {0};
            s_tmp[0] = (char)(0x30 + (num / 100) % 10);
            s_tmp[1] = (char)(0x30 + (num / 10) % 10);
            s_tmp[2] = (char)(0x30 + num % 10);
            s_tmp[3] = '\0';
            if (put_s("Please Input ") == -1)
            {
                fuart_io_error();
            }
            if (put_s((const char *)&s_tmp[0]) == -1)
            {
                fuart_io_error();
            }
            if (put_s(" characters = ") == -1)
            {
                fuart_io_error();
            }
        }
        /* wait receive */
        while (loopBreak == TXZ_BUSY)
        {
            if (p_rx->state == FUART_IO_RECV_RUN)
            {
                /*>>> Critical information. */
                volatile uint32_t length = p_rx->length;
                /*<<< Critical information. */
                
                /*------------------------------*/
                /*  Check Receive Length        */
                /*------------------------------*/
                if (length >= num)
                {
                /*>>> Critical information. */
                NVIC_DisableIRQ(instance.init.irq_n);
                p_rx->length = 0x00;
                p_rx->rp = 0;
                NVIC_EnableIRQ(instance.init.irq_n);
                /*<<< Critical information. */
                    loopBreak = TXZ_DONE;
                }
            }
            else
            {
                loopBreak = TXZ_DONE;
            }
        }
        /* "Echo" tresfer */
        if (put_s("\n Echo = ") == -1)
        {
            fuart_io_error();
        }
        /* dsta trasnfer */
        instance.info.work[num] = '\0';
        for(i=0; i<=num; ++i)
        {
            *s++ = (char)instance.info.work[i];
        }
        if (put_s((const char *)&instance.info.work[0]) == -1)
        {
            fuart_io_error();
        }
        if (put_s(" \n") == -1)
        {
            fuart_io_error();
        }
    }
    ret = num;
    return(ret);
}
#endif

/*--------------------------------------------------*/
/** 
  * @brief  The character is shown to an output stream.
  * @param  c :Character data.
  * @return Output Character data.
  * @retval (value < 0) :Failure
  * @note   -
  */
/*--------------------------------------------------*/
static int put_char(int c)
{
    int ret = -1;
    fuart_io_private_tx_t *p_tx = &instance.info.tx;

    if (p_tx->state == FUART_IO_SEND_IDLE)
    {
        fuart_transmit_t param;
        volatile TXZ_WorkState loopBreak = TXZ_BUSY;
        uint32_t status;

        /*------------------------------*/
        /*  Transmit                    */
        /*------------------------------*/
        p_tx->buff[0] = (uint8_t)c;
        p_tx->wp      = 1;
        p_tx->rp      = 0;
        p_tx->state   = FUART_IO_SEND_RUN;
        param.p_data    = &p_tx->buff[0];
        param.num       = p_tx->wp;
        if (fuart_transmitIt(&instance.info.fuart, &param) != TXZ_SUCCESS)
        {
            fuart_io_error();
        }
        /*------------------------------*/
        /*  Wait to transmit            */
        /*------------------------------*/
        loopBreak = TXZ_BUSY;
        while(loopBreak == TXZ_BUSY)
        {
            switch (p_tx->state)
            {
            case FUART_IO_SEND_RUN:
                /* no process */
                break;
            case FUART_IO_SEND_SUCCESS:
                if (fuart_get_status(&instance.info.fuart, &status) == TXZ_SUCCESS)
                {
                    if ((status & FUARTxFR_BUSY_MASK) != FUARTxFR_BUSY_FLAG_SET)
                    {
                        loopBreak = TXZ_DONE;
                    }
                }
                break;
            default:
                loopBreak = TXZ_DONE;
                break;
            }
        }
        /*------------------------------*/
        /*  Check Error                 */
        /*------------------------------*/
        if (p_tx->state == FUART_IO_SEND_FAILURE)
        {
            fuart_io_error();
        }
        /* Clear status. */
        p_tx->state = FUART_IO_SEND_IDLE;
        /* Set result. */
        ret = c;
    }

    return (ret);
}

/*--------------------------------------------------*/
/** 
  * @brief  The character is acquired from an input stream.
  * @param  -
  * @return Input Character data.
  * @retval (value < 0) :Failure
  * @note   -
  */
/*--------------------------------------------------*/
static int get_char(void)
{
    volatile TXZ_WorkState loopBreak = TXZ_BUSY;
    volatile int c = -1;
    fuart_io_private_rx_t *p_rx = &instance.info.rx;

    while (loopBreak == TXZ_BUSY)
    {
        if (p_rx->state == FUART_IO_RECV_RUN)
        {
            /*>>> Critical information. */
            volatile uint32_t length = p_rx->length;
            /*<<< Critical information. */

            /*------------------------------*/
            /*  Check Receive Length        */
            /*------------------------------*/
            if (length > 0)
            {
                c = (int)p_rx->buff[p_rx->rp];
                p_rx->rp++;
                if (p_rx->rp >= FUART_IO_RECEIVE_MAX)
                {
                    p_rx->rp = 0;
                }
                /*>>> Critical information. */
                NVIC_DisableIRQ(instance.init.irq_n);
                p_rx->length--;
                NVIC_EnableIRQ(instance.init.irq_n);
                /*<<< Critical information. */
                loopBreak = TXZ_DONE;
            }
        }
        else
        {
            loopBreak = TXZ_DONE;
        }
    }
    /*------------------------------*/
    /*  Check Error                 */
    /*------------------------------*/
    if (p_rx->state == FUART_IO_RECV_FAILURE)
    {
        p_rx->state = FUART_IO_RECV_IDLE;
        /* Set result. */
    }
    return (c);
}

/*--------------------------------------------------*/
/** 
  * @brief  Transmit handler.
  * @param  id      :User ID.
  * @param  result  :Result.
  * @return -
  * @retval -
  * @note   Called by fuart driver.
  */
/*--------------------------------------------------*/
static void transmit_handler(uint32_t id, TXZ_Result result)
{
    fuart_io_t *p_instance = (fuart_io_t *)id;

    if (p_instance != FUART_IO_NULL)
    {
        fuart_io_private_tx_t *p_tx = &p_instance->info.tx;

        if (p_tx->state != FUART_IO_SEND_IDLE)
        {
            /*------------------------------*/
            /*  Result Check                */
            /*------------------------------*/
            if (result == TXZ_SUCCESS)
            {
                p_tx->state = FUART_IO_SEND_SUCCESS;
            }
            else
            {
                p_tx->state = FUART_IO_SEND_FAILURE;
            }
        }
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  Receive handler.
  * @param  id      :User ID.
  * @param  result  :Result.
  * @param  p_info  :Received information Source Address.
  * @return -
  * @retval -
  * @note   Called by fuart driver.
  */
/*--------------------------------------------------*/
static void receive_handler(uint32_t id, TXZ_Result result, fuart_receive_t *p_info)
{
    fuart_io_t *p_instance = (fuart_io_t *)id;

    if (p_instance != FUART_IO_NULL)
    {
        fuart_io_private_rx_t *p_rx = &p_instance->info.rx;

        if (p_rx->state == FUART_IO_RECV_RUN)
        {
            if (result == TXZ_SUCCESS)
            {
               if (p_info != FUART_IO_NULL)
               {
                    uint32_t i;
                    uint32_t num    = p_info->num;
                    /*>>> Critical information. */
                    volatile uint32_t length = p_rx->length;
                    /*<<< Critical information. */

                    if (length < FUART_IO_RECEIVE_MAX) 
                    {
                        if ((length + num) > FUART_IO_RECEIVE_MAX)
                        {
                            num = (uint32_t)(FUART_IO_RECEIVE_MAX - length);
                        }
                        for (i=0; i<num; i++)
                        {
                            p_rx->buff[p_rx->wp] = *(p_info->p_data + i);
                            p_rx->wp++;
                            if (p_rx->wp >= FUART_IO_RECEIVE_MAX)
                            {
                                p_rx->wp = 0;
                            }
                        }
                        /*>>> Critical information. */
                        p_rx->length += num;
                        /*<<< Critical information. */
                    }
                    else
                    {
                        p_rx->state = FUART_IO_RECV_FAILURE;
                    }
                }
                else
                {
                    p_rx->state = FUART_IO_RECV_FAILURE;
                }
            }
            else
            {
                p_rx->state = FUART_IO_RECV_FAILURE;
            }
        }
    }
}
/**
 *  @}
 */ /* End of group BSP_UTILITIES_Private_fuctions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup BSP_UTILITIES_Exported_functions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  Initialize
  * @param  p_param :Initial Setting Information Source Address.
  * @return Result.
  * @retval TXZ_SUCCESS :Success
  * @retval TXZ_ERROR   :Failure
  * @note   Parameter check isn't performed.
  * @attention   Singleton.
  */
/*--------------------------------------------------*/
TXZ_Result fuart_io_initialize(fuart_io_initial_setting_t *p_param)
{
    /*------------------------------*/
    /*  Initialize Variable         */
    /*------------------------------*/
    instance.init.id            = p_param->id;
    instance.init.p_reg         = p_param->p_reg;
    instance.init.boudrate      = p_param->boudrate;
    instance.init.irq_n         = p_param->irq_n;
    instance.info.tx.state      = FUART_IO_SEND_IDLE;
    instance.info.tx.wp         = 0;
    instance.info.tx.rp         = 0;
    instance.info.rx.state      = FUART_IO_RECV_RUN;     /* Auto run. */
    instance.info.rx.wp         = 0;
    instance.info.rx.rp         = 0;
    instance.info.rx.length     = 0;
    /*------------------------------*/
    /*  Uart Setting                */
    /*------------------------------*/
    /*--- Construct (Register Allocation) ---*/
    instance.info.fuart.p_instance = instance.init.p_reg;
    /*--- Driver Initialize ---*/
    instance.info.fuart.init.id            = (uint32_t)&instance;  /* User ID: Any Value (Example...UART driver instance address). */
    {
        cg_t paramCG;
        paramCG.p_instance = TSB_CG;

        if (fuart_get_boudrate_setting(
                cg_get_mphyt0(&paramCG), 
                instance.init.boudrate, 
                &instance.info.fuart.init.boudrate
                ) != TXZ_SUCCESS)
        {
            fuart_io_error();
            return(TXZ_ERROR);
        }
    }
    /* Uart     */
    instance.info.fuart.init.tx_int.inttx  = FUART_TX_INT_ENABLE;
    instance.info.fuart.init.tx_int.level  = FUART_FIFO_LEVEL_4;
    instance.info.fuart.init.rx_int.intrx  = FUART_RX_INT_ENABLE;
    instance.info.fuart.init.rx_int.level  = FUART_FIFO_LEVEL_4;
    instance.info.fuart.init.interr        = FUART_OV_ERR_INT_ENABLE | FUART_BK_ERR_INT_ENABLE | FUART_PA_ERR_INT_ENABLE | FUART_FR_ERR_INT_ENABLE;
    instance.info.fuart.init.fifo          = FUART_FIFO_DISABLE;
    instance.info.fuart.init.ctse          = FUART_CTS_DISABLE;
    instance.info.fuart.init.rtse          = FUART_RTS_DISABLE;
    instance.info.fuart.init.sblen         = FUART_STOP_BIT_1;
    instance.info.fuart.init.even          = FUART_PARITY_BIT_ODD;
    instance.info.fuart.init.pe            = FUART_PARITY_DISABLE;
    instance.info.fuart.init.sm            = FUART_DATA_LENGTH_8;
    instance.info.fuart.init.stpa          = FUART_STATIC_PARITY_DISABLE;
    instance.info.fuart.transmit.handler   = transmit_handler;
    instance.info.fuart.receive.handler    = receive_handler;
    if (fuart_init(&instance.info.fuart) != TXZ_SUCCESS)
    {
        fuart_io_error();
        return(TXZ_ERROR);
    }
    /*--- Driver Start :Receive ---*/
    {
        fuart_receive_t rx;

        rx.p_data = &instance.info.work[0];
        rx.num    = 1;
        if (fuart_receiveIt(&instance.info.fuart, &rx) != TXZ_SUCCESS)
        {
            fuart_io_error();
            return(TXZ_ERROR);
        }
    }
    /*------------------------------*/
    /*  Enable Interrupt            */
    /*------------------------------*/
    NVIC_EnableIRQ(instance.init.irq_n);

    return(TXZ_SUCCESS);
}

/*--------------------------------------------------*/
/** 
  * @brief  Finalize
  * @param  -
  * @return -
  * @retval -
  * @note   Parameter check isn't performed.
  * @attention   Singleton.
  */
/*--------------------------------------------------*/
void fuart_io_finalize(void)
{
    if (instance.info.fuart.p_instance != FUART_IO_NULL)
    {    
        /*------------------------------*/
        /*  Disable Enable              */
        /*------------------------------*/
        NVIC_DisableIRQ(instance.init.irq_n);
        /*------------------------------*/
        /*  Uart Finalize               */
        /*------------------------------*/
        /*--- Driver Finalize ---*/
        if (fuart_deinit(&instance.info.fuart) != TXZ_SUCCESS)
        {
            fuart_io_error();
        }
        /*--- Destruct (Register release) ---*/
        instance.info.fuart.p_instance = FUART_IO_NULL;
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  FUART IRQ Handler.
  * @param  -
  * @return -
  * @retval -
  * @note   Parameter check isn't performed.
  * @attention   Singleton.
  */
/*--------------------------------------------------*/
void fuart_io_irq_handler(void)
{
    fuart_irq_handler(&instance.info.fuart);
}

/*--------------------------------------------------*/
/** 
  * @brief  Receive Stop
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success
  * @retval TXZ_ERROR   :Failure
  * @note   -
  * @attention   Singleton.
  */
/*--------------------------------------------------*/
TXZ_Result fuart_io_receive_stop(void)
{
    fuart_io_private_rx_t *p_rx = &instance.info.rx;

    if(fuart_discard_receive(&instance.info.fuart) != TXZ_SUCCESS)
    {
        /* no process */
    }
    p_rx->wp         = 0;
    p_rx->rp         = 0;
    p_rx->length     = 0;
    p_rx->state = FUART_IO_RECV_IDLE;
    return(TXZ_SUCCESS);
}
/*--------------------------------------------------*/
/** 
  * @brief  Receive Restar
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success
  * @retval TXZ_ERROR   :Failure
  * @note   -
  * @attention   Singleton.
  */
/*--------------------------------------------------*/
TXZ_Result fuart_io_receive_restart(void)
{
    fuart_io_private_rx_t *p_rx = &instance.info.rx;

    if (p_rx->state != FUART_IO_RECV_IDLE){
        return(TXZ_ERROR);
    }
    p_rx->wp         = 0;
    p_rx->rp         = 0;
    p_rx->length     = 0;
    p_rx->state = FUART_IO_RECV_RUN;
    /*--- Driver Start :Receive ---*/
    {
        fuart_receive_t rx;

        rx.p_data = &instance.info.work[0];
        rx.num    = 1;
        if (fuart_receiveIt(&instance.info.fuart, &rx) != TXZ_SUCCESS)
        {
            fuart_io_error();
            return(TXZ_ERROR);
        }
    }
    return(TXZ_SUCCESS);
}



/**
 *  @}
 */ /* End of group BSP_UTILITIES_Exported_functions */


/*------------------------------------------------------------------------------*/
/*  Semihosting Retarget                                                        */
/*------------------------------------------------------------------------------*/
#if defined ( __CC_ARM   )      /* RealView Compiler */
struct __FILE { int handle; /* Add whatever you need here */ };
FILE __stdout;
FILE __stdin;
int fputc(int ch, FILE *f) {
#elif defined ( __ICCARM__ )    /*IAR Compiler */
int putchar(int ch) {
#endif
    return (put_char(ch));
}

#if defined ( __CC_ARM   )      /* RealView Compiler */
int getc(FILE * f)
#elif defined ( __ICCARM__ )    /*IAR Compiler */
int getchar(void)
#endif
{
    return (get_char());
}

#if defined ( __CC_ARM   )      /* RealView Compiler */
int ferror(FILE *f)
#elif defined ( __ICCARM__ )    /*IAR Compiler */
int ferror(void)
#endif
{
    return (0);
}

/**
 *  @}
 */ /* End of group BSP_UTILITIES */

/**
 *  @} 
 */ /* End of group Example */

#endif /* defined(__BSP_UART_IO_H)  */
#ifdef __cplusplus
}
#endif /* __cplusplus */
