/** 
 *******************************************************************************
 * @file    bsp_fuart_io.h
 * @brief   External reference declaration for USB FUART.
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2020
 *******************************************************************************
 */
/*------------------------------------------------------------------------------*/
/*  Define to prevent recursive inclusion                                       */
/*------------------------------------------------------------------------------*/
#ifndef __BSP_FUART_IO_H
#define __BSP_FUART_IO_H

#ifdef __cplusplus
extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include <stdarg.h>
#include "txz_sample_def.h"

/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @addtogroup BSP_UTILITIES
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup BSP_UTILITIES_Exported_macro
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group BSP_UTILITIES_Exported_macro */


/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup BSP_UTILITIES_Exported_define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group BSP_UTILITIES_Exported_define */


/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup BSP_UTILITIES_Exported_define
 *  @{
 */

#define FUART_IO_SEND_MAX       ((uint32_t)32)   /*!< Size of send max. (byte)    */
#define FUART_IO_RECEIVE_MAX    ((uint32_t)32)   /*!< Size of receive max. (byte) */
/** 
 *  @}
 */ /* End of group BSP_UTILITIES_Exported_define */

/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup BSP_UTILITIES_Exported_define
 *  @{
 */

/*----------------------------------*/
/** 
 * @brief  Send state.
*/
/*----------------------------------*/
typedef enum {
    FUART_IO_SEND_IDLE = 0U,     /*!< Idle                   */
    FUART_IO_SEND_RUN,           /*!< Transmiting.           */
    FUART_IO_SEND_SUCCESS,       /*!< Transmission complete. */
    FUART_IO_SEND_FAILURE        /*!< Transmission error.    */
} FUartIOTxState;

/*----------------------------------*/
/** 
 * @brief  Receive state.
*/
/*----------------------------------*/
typedef enum {
    FUART_IO_RECV_IDLE = 0U,     /*!< Idle                   */
    FUART_IO_RECV_RUN,           /*!< Receiving              */
    FUART_IO_RECV_FAILURE        /*!< Received error.        */
} FUartIORxState;

/**
 *  @}
 */ /* End of group BSP_UTILITIES_Exported_define */


/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup BSP_UTILITIES_Exported_typedef
 *  @{
 */
/*----------------------------------*/
/** 
 * @brief  Initial setting. \n
 *         User must be set.
*/
/*----------------------------------*/
typedef struct
{
    uint32_t            id;             /*!< User ID.                */
    TSB_FURT_TypeDef    *p_reg;         /*!< Register Bass Address.  */
    uint32_t            boudrate;       /*!< Boudrate(bps).          */
    IRQn_Type           irq_n;          /* Interrupt Number          */
} fuart_io_initial_setting_t;

/*----------------------------------*/
/** 
 * @brief  Send information.
 * @brief  Private Member.
*/
/*----------------------------------*/
typedef struct
{
    volatile FUartIOTxState  state;              /*!< Status.                     */
    uint32_t                wp;                 /*!< Write pointer.              */
    uint32_t                rp;                 /*!< Read pointer(Now, no use).  */
    uint8_t                 buff[FUART_IO_SEND_MAX];     /*!< Send data.                  */
} fuart_io_private_tx_t;

/*----------------------------------*/
/** 
 * @brief  Receive information.
 * @brief  Private Member.
*/
/*----------------------------------*/
typedef struct
{
    volatile FUartIORxState  state;              /*!< Status.                     */
    volatile uint32_t       wp;                 /*!< Write pointer.              */
    volatile uint32_t       rp;                 /*!< Read pointer(Now, no use).  */
    volatile uint32_t       length;             /*!< Num of receive data.        */
    volatile uint8_t        buff[FUART_IO_RECEIVE_MAX];  /*!< Receive data.               */
} fuart_io_private_rx_t;

/*----------------------------------*/
/** 
 * @brief  Private Member.
*/
/*----------------------------------*/
typedef struct
{
    fuart_t                  fuart;               /*!< Uart driver instance.       */
    fuart_io_private_tx_t    tx;                 /*!< Send information.           */
    fuart_io_private_rx_t    rx;                 /*!< Receive information.        */
    uint8_t                 work[FUART_RX_FIFO_MAX];     /*!< Receive buffer for fuart driver. */
} fuart_io_private_info_t;

/*----------------------------------*/
/** 
 * @brief  USB FUART class.
*/
/*----------------------------------*/
typedef struct
{
    fuart_io_initial_setting_t init; /*!< Initial setting.     */
    fuart_io_private_info_t    info; /*!< Private information. */
} fuart_io_t;

/**
 *  @}
 */ /* End of group BSP_UTILITIES_Exported_typedef */

/*------------------------------------------------------------------------------*/
/*  Functions                                                                   */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup BSP_UTILITIES_Exported_functions
 *  @{
 */
int put_s(const char *s);
int get_s(char *s, uint8_t num);
TXZ_Result fuart_io_initialize(fuart_io_initial_setting_t *p_param);
void fuart_io_finalize(void);
void fuart_io_irq_handler(void);
TXZ_Result fuart_io_receive_stop(void);
TXZ_Result fuart_io_receive_restart(void);
/** 
 *  @}
 */ /* End of group BSP_UTILITIES_Exported_functions */

/**
 *  @}
 */ /* End of group BSP_UTILITIES */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* __BSP_UART_IO_H */

