/** 
 *******************************************************************************
 * @file    bsp_fuart_io_mdma.h
 * @brief   External reference declaration for FUART input/output
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2020
 *******************************************************************************
 */
/*------------------------------------------------------------------------------*/
/*  Define to prevent recursive inclusion                                       */
/*------------------------------------------------------------------------------*/
#ifndef __BSP_FUART_IO_MDMA_H
#define __BSP_FUART_IO_MDMA_H

#ifdef __cplusplus
extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include <stdarg.h>
#include "txz_sample_def.h"

/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @addtogroup BSP_UTILITIES
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup BSP_UTILITIES_Exported_macro
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group BSP_UTILITIES_Exported_macro */


/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup BSP_UTILITIES_Exported_define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group BSP_UTILITIES_Exported_define */


/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup BSP_UTILITIES_Exported_define
 *  @{
 */

#define FUART_IO_MDMA_SEND_MAX       ((uint32_t)32)   /*!< Size of send max. (byte)    */
#define FUART_IO_MDMA_RECEIVE_MAX    ((uint32_t)32)   /*!< Size of receive max. (byte) */


/** 
 *  @}
 */ /* End of group BSP_UTILITIES_Exported_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup BSP_UTILITIES_Exported_define
 *  @{
 */

/*----------------------------------*/
/** 
 * @brief  Send state.
*/
/*----------------------------------*/
typedef enum {
    FUART_IO_MDMA_SEND_IDLE = 0U,     /*!< Idle                   */
    FUART_IO_MDMA_SEND_RUN,           /*!< Transmiting.           */
    FUART_IO_MDMA_SEND_SUCCESS,       /*!< Transmission complete. */
    FUART_IO_MDMA_SEND_FAILURE        /*!< Transmission error.    */
} FUartIOMDmaTxState;

/*----------------------------------*/
/** 
 * @brief  Receive state.
*/
/*----------------------------------*/
typedef enum {
    FUART_IO_MDMA_RECV_IDLE = 0U,     /*!< Idle                   */
    FUART_IO_MDMA_RECV_RUN,           /*!< Receiving              */
    FUART_IO_MDMA_RECV_SUCCESS,       /*!< Received complete.     */
    FUART_IO_MDMA_RECV_FAILURE        /*!< Received error.        */
} FUartIOMDmaRxState;

/**
 *  @}
 */ /* End of group BSP_UTILITIES_Exported_define */


/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup BSP_UTILITIES_Exported_typedef
 *  @{
 */
/*----------------------------------*/
/** 
 * @brief  USB FUART Initial Setting.
*/
/*----------------------------------*/
typedef struct
{
    uint32_t            id;         /*!< User ID.         */
    TSB_FURT_TypeDef    *p_reg;     /*!< Register Bass Address.  */
    uint32_t            boudrate;   /*!< Boudrate(bps).   */
    struct
    {
        mdma_t           *p_mdma;   /*!< MDMA driver instance address.  */
    } mdma;                         /*!< MDMA dirver instance. */ 
} fuart_io_mdma_initial_setting_t;

/*----------------------------------*/
/** 
 * @brief  USB FUART Send information.
 * @brief  Private Member.
*/
/*----------------------------------*/
typedef struct
{
    volatile FUartIOMDmaTxState   state;                          /*!< Status.                     */
    uint8_t                      buff[FUART_IO_MDMA_SEND_MAX];    /*!< Send data.                  */
} fuart_io_mdma_private_tx_t;

/*----------------------------------*/
/** 
 * @brief  USB FUART Receive information.
 * @brief  Private Member.
*/
/*----------------------------------*/
typedef struct
{
    volatile FUartIOMDmaRxState   state;  /*!< Status.                     */
    uint8_t                     buff[FUART_IO_MDMA_RECEIVE_MAX];    /*!< Receive data.               */
} fuart_io_mdma_private_rx_t;

/*----------------------------------*/
/** 
 * @brief  USB FUART Private Member.
*/
/*----------------------------------*/
typedef struct
{
    fuart_mdma_t                  fuart;  /*!< FUart driver instance.       */
    fuart_io_mdma_private_tx_t    tx;    /*!< Send information.           */
    fuart_io_mdma_private_rx_t    rx;    /*!< Receive information.        */
} fuart_io_mdma_private_info_t;

/*----------------------------------*/
/** 
 * @brief  USB FUART class.
*/
/*----------------------------------*/
typedef struct
{
    fuart_io_mdma_initial_setting_t init; /*!< Initial setting.     */
    fuart_io_mdma_private_info_t    info; /*!< Private information. */
} fuart_io_mdma_t;

/**
 *  @}
 */ /* End of group BSP_UTILITIES_Exported_typedef */


/*------------------------------------------------------------------------------*/
/*  Functions                                                                   */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup BSP_UTILITIES_Exported_functions
 *  @{
 */
fuart_io_mdma_t *fuart_io_mdma_initialize(fuart_io_mdma_initial_setting_t *p_param);
void fuart_io_mdma_finalize(fuart_io_mdma_t *p_instance);
int put_s(const char *s);
int get_s(char *s, uint8_t num);
/** 
 *  @}
 */ /* End of group BSP_UTILITIES_Exported_functions */

/**
 *  @}
 */ /* End of group BSP_UTILITIES */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* __BSP_FUART_IO_MDMA_H */

