/** 
 *******************************************************************************
 * @file    main.c
 * @brief   CEC CH to CH Sample Application.
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2020
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif
/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "txz_sample_def.h"
#include "bsp.h"
#include "bsp_uart_io.h"

/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup CEC_CH_TO_CH CEC_CH_TO_CH Sample Appli
 *  @{
 */

/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup CEC_CH_TO_CH_Private_macro CEC_CH_TO_CH Private Macro
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group CEC_CH_TO_CH_Private_macro */


/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup CEC_CH_TO_CH_Private_define CEC_CH_TO_CH Private Define
 *  @{
 */
/* no define */

/** 
 *  @}
 */ /* End of group CEC_CH_TO_CH_Private_define */

/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup CEC_CH_TO_CH_Private_define CEC_CH_TO_CH Private Define
 *  @{
 */

#define MAIN_NULL       ((void *)0)            /*!< Null Pointer.                 */
#define USER_ID_UART    ((uint32_t)1)          /*!< User ID :USB UART (Any value) */
#define CEC_IO_SEND_MAX       ((uint32_t)32)   /*!< Size of send max. (byte)      */
#define CEC_IO_RECEIVE_MAX    ((uint32_t)32)   /*!< Size of receive max. (byte)   */

/** 
 *  @}
 */ /* End of group CEC_CH_TO_CH_Private_define */

/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup CEC_CH_TO_CH_Private_define CEC_CH_TO_CH Private Define
 *  @{
 */

/*----------------------------------*/
/** 
 * @enum   Processing
 * @brief  Processing mode.
*/
/*----------------------------------*/
typedef enum
{
    PROC_IDLE,              /*!< Idle.                        */
    PROC_START,             /*!< Start.                       */
    PROC_STOP,               /*!< Stop .                       */
    PROC_OUTPUT_MSG,        /*!< Output first message.        */
    PROC_INPUT,             /*!< Wait to input.               */
    PROC_OUTPUT_ECHO,       /*!< Output Echo back data.       */
    PROC_OUTPUT_ERR,        /*!< Output input error message.  */
    PROC_OUTPUT_RECV_ERR,   /*!< Output Receive error message.  */
    PROC_CEC_TRANSFER,      /*!< CEC trasfer.                 */
    PROC_ERROR              /*!< Error.                       */
} Processing;

/*----------------------------------*/
/** 
 * @brief  Send state.
*/
/*----------------------------------*/
typedef enum {
    CEC_IO_SEND_IDLE = 0U,     /*!< Idle                   */
    CEC_IO_SEND_RUN,           /*!< Transmiting.           */
    CEC_IO_SEND_SUCCESS,       /*!< Transmission complete. */
    CEC_IO_SEND_FAILURE,       /*!< Transmission error.    */
    CEC_IO_SEND_NO_ACK         /*!< Transmission No ACK.   */
} CecIOTxState;

/*----------------------------------*/
/** 
 * @brief  Receive state.
*/
/*----------------------------------*/
typedef enum {
    CEC_IO_RECV_IDLE = 0U,     /*!< Idle                   */
    CEC_IO_RECV_RUN,           /*!< Receiving              */
    CEC_IO_RECV_SUCCESS,       /*!< Receive complete. */
    CEC_IO_RECV_FAILURE,        /*!< Received error.        */
    CEC_IO_RECV_OVER_FLOW      /*!> Received over flow.    */
} CecIORxState;


/** 
 *  @}
 */ /* End of group CEC_CH_TO_CH_Private_define */

/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup CEC_CH_TO_CH_Private_typedef CEC_CH_TO_CH Private Typedef
 *  @{
 */
/*----------------------------------*/
/** 
 * @struct input_t
 * @brief  Input data information.
*/
/*----------------------------------*/
typedef struct
{
    char    data[(UART_IO_RECEIVE_MAX+1)];  /*!< Input data(+ for NULL 1byte). */
    uint8_t wp;                             /*!< Write pointer.                */
} input_t;

/*----------------------------------*/
/** 
 * @brief  Send information.
 * @brief  Private Member.
*/
/*----------------------------------*/
typedef struct
{
    volatile CecIOTxState  state;               /*!< Status.                     */
    uint32_t                wp;                 /*!< Write pointer.              */
    uint32_t                rp;                 /*!< Read pointer(Now, no use).  */
    uint8_t                 buff[CEC_IO_SEND_MAX];     /*!< Send data.                  */
} cec_io_private_tx_t;

/*----------------------------------*/
/** 
 * @brief  Receive information.
 * @brief  Private Member.
*/
/*----------------------------------*/
typedef struct
{
    volatile CecIORxState  state;               /*!< Status.                     */
    volatile uint32_t       wp;                 /*!< Write pointer.              */
    volatile uint32_t       rp;                 /*!< Read pointer.               */
    volatile uint32_t       length;             /*!< Num of receive data.        */
             uint8_t        buff[CEC_IO_RECEIVE_MAX];  /*!< Receive data.               */
} cec_io_private_rx_t;

/*----------------------------------*/
/** 
 * @brief  Private Member.
*/
/*----------------------------------*/
typedef struct
{
    cec_t                  cec;               /*!< Uart driver instance.       */
    cec_io_private_tx_t    tx;                 /*!< Send information.           */
    cec_io_private_rx_t    rx;                 /*!< Receive information.        */
} cec_io_private_info_t;

/*----------------------------------*/
/** 
 * @brief  USB cec class.
*/
/*----------------------------------*/
typedef struct
{
    cec_io_private_info_t    info; /*!< Private information. */
} cec_io_t;

/** 
 *  @}
 */ /* End of group CEC_CH_TO_CH_Private_typedef */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup CEC_CH_TO_CH_Private_variables CEC_CH_TO_CH Private Variables
 *  @{
 */
static cec_io_t instance;       /*!< Instance object */
static Processing proc;         /*!< Processing mode.        */
static input_t input;           /*!< Input data information. */
static char read_data[CEC_IO_RECEIVE_MAX];   /*!< Read data information. */
static int count;               /*!< cec data once count */
static uint32_t total_count;   /*!< cec data total count */
static cec_t   cec_obj;        /*!< CEC object. */
/**
 *  @}
 */ /* End of group CEC_CH_TO_CH_Private_variables */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup CEC_CH_TO_CH_Private_fuctions CEC_CH_TO_CH Private Fuctions
 *  @{
 */
static void clear_data(void);
static void clear_input_data(void);
static void variable_initialize(void);
static TXZ_Result application_initialize(void);
static TXZ_Result application_finalize(void);
static void transmit_handler(uint32_t id, TXZ_Result result, uint32_t tx_status);
static void receive_handler(uint32_t id, TXZ_Result result, cec_receive_t *p_info, uint32_t rx_status);
static TXZ_Result disp_cec_address(void);
static void bsp_irq_cecrx_enable(void);
static void bsp_irq_cecrx_disable(void);
static void bsp_irq_cectx_enable(void);
static void bsp_irq_cectx_disable(void);
static void cec_io_error(void);

/*--------------------------------------------------*/
/** 
  * @brief  Clear input data.
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void clear_data(void)
{
    uint8_t i;

    for (i = 0; i < (UART_IO_RECEIVE_MAX+1); i++)
    {
        input.data[i] = 0;
    }
    input.wp = 0;
    
    for (i = 0; i < CEC_IO_RECEIVE_MAX; i++)
    {
        read_data[i] = 0;
    }
    count = 0;
    total_count = 0;
}
/*--------------------------------------------------*/
/** 
  * @brief  Clear input data.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void clear_input_data(void)
{
    uint8_t i;

    for (i=0; i<(UART_IO_RECEIVE_MAX+1); i++)
    {
        input.data[i] = 0;
    }
    input.wp = 0;
}
/*--------------------------------------------------*/
/** 
  * @brief  Variable Initialize
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void variable_initialize(void)
{
    /* Instance. */
    cec_obj.p_instance = MAIN_NULL;
    instance.info.rx.state = CEC_IO_RECV_IDLE;
    instance.info.tx.state = CEC_IO_SEND_IDLE;
    /* Processing mode. */
    proc = PROC_START;
    /* Information. */
    clear_data();
}
/*--------------------------------------------------*/
/** 
  * @brief  Application Initialize.
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result application_initialize(void)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*----------------------*/
    /* CEC                  */
    /*----------------------*/
    cec_obj.p_instance = TSB_CEC0;
    cec_obj.init.id = (uint32_t)&instance;
    cec_obj.init.logical_address = CECxADD_CECADD_5 | CECxADD_CECADD_7;
    cec_obj.init.ack_disable = CECxRCR1_CECACKDIS_ACK_ON;
    cec_obj.init.high_noise_cancel = CECxRCR1_CECHNC_NONE;
    cec_obj.init.low_noise_cancel = CECxRCR1_CECLNC_NONE;
    cec_obj.init.interval_min = CECxRCR1_CECMIN_NONE;
    cec_obj.init.interval_max = CECxRCR1_CECMAX_NONE;
    cec_obj.init.data_point = CECxRCR1_CECDAT_NONE;
    cec_obj.init.time_out = CECxRCR1_CECTOUT_1BIT;
    cec_obj.init.receive_error_hold = CECxRCR1_CECRIHLD_OFF;
    cec_obj.init.mismach_receive = CECxRCR1_CECOTH_OFF;
    cec_obj.init.start_bit_fall_max = CECxRCR2_CECSWAV3_NONE;
    cec_obj.init.start_bit_fall_min = CECxRCR2_CECSWAV2_NONE;
    cec_obj.init.start_bit_rise_max = CECxRCR2_CECSWAV1_NONE;
    cec_obj.init.start_bit_rise_min = CECxRCR2_CECSWAV0_NONE;
    cec_obj.init.data_0_rise_max = CECxRCR3_CECWAV3_NONE;
    cec_obj.init.data_0_rise_min = CECxRCR3_CECWAV2_NONE;
    cec_obj.init.data_1_rise_max = CECxRCR3_CECWAV1_NONE;
    cec_obj.init.data_1_rise_min = CECxRCR3_CECWAV0_NONE;
    cec_obj.init.start_bit_enable = CECxRCR3_CECRSTAEN_ENABLE;
    cec_obj.init.wav_error_enable = CECxRCR3_CECWAVEN_ENABLE;
    cec_obj.init.start_bit_rise_timing = CECxTCR_CECSTRS_NONE;
    cec_obj.init.start_bit_interval_timing = CECxTCR_CECSPRD_NONE;
    cec_obj.init.data_bit_rise_timing = CECxTCR_CECDTRS_NONE;
    cec_obj.init.data_bit_interval_timing = CECxTCR_CECDPRD_NONE;
    cec_obj.init.bus_free_wait = CECxTCR_CECFREE_NONE;
    cec_obj.init.clk = CECxFSSEL_CECCLK_FS;
    cec_obj.transmit.handler   = transmit_handler;
    cec_obj.receive.handler    = receive_handler;
    result  = cec_init(&cec_obj);
    if(result == TXZ_SUCCESS)
    {
        cec_io_private_rx_t *p_rx = &instance.info.rx;
        
        if(cec_discard_receive(&cec_obj) != TXZ_SUCCESS)
        {
            cec_io_error();
        }
        p_rx->wp         = 0;
        p_rx->rp         = 0;
        p_rx->length     = 0;
        {
            cec_receive_t param;
            
            param.p_data    = &p_rx->buff[0];
            param.num       = CEC_IO_RECEIVE_MAX;
            p_rx->state   = CEC_IO_RECV_RUN;
            if (cec_receiveIt(&cec_obj, &param) != TXZ_SUCCESS)
            {
                cec_io_error();
            }
        }
        NVIC_EnableIRQ(INTCEC0RX_IRQn);
        NVIC_EnableIRQ(INTCEC0TX_IRQn);
        bsp_irq_cecrx_enable();
        bsp_irq_cectx_enable();
     }
     else
     {
        cec_io_error();
     }
    
    /*----------------------*/
    /* Usb Uart             */
    /*----------------------*/
    /* Tx & Rx use same UART channel. */
    {
        uart_io_initial_setting_t param;

        param.id         = USER_ID_UART;
        switch (bsp_get_usb_uart_tx_ch(BSP_USB_UART_0))
        {
        case 0:
            param.p_reg  = TSB_UART0;
            break;
        case 1:
            param.p_reg  = TSB_UART1;
            break;
        case 2:
            param.p_reg  = TSB_UART2;
            break;
        default:
            param.p_reg  = MAIN_NULL;
            break;
        }
        param.boudrate   = bsp_get_usb_uart_tx_baudrate(BSP_USB_UART_0);
        param.irq.tx     = bsp_get_usb_uart_tx_nvic(BSP_USB_UART_0);
        param.irq.rx     = bsp_get_usb_uart_rx_nvic(BSP_USB_UART_0);
        param.irq.err    = bsp_get_usb_uart_err_nvic(BSP_USB_UART_0);
        if (param.p_reg != MAIN_NULL)
        {
            result  = uart_io_initialize(&param);
        }
        else
        {
            result = TXZ_ERROR;
        }
    }
    return (result);
}
/*--------------------------------------------------*/
/** 
  * @brief  Application Finalize.
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result application_finalize(void)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*----------------------*/
    /* CEC                  */
    /*----------------------*/
    if (cec_obj.p_instance != MAIN_NULL)
    {
        if (cec_deinit(&cec_obj))
        {
            result = TXZ_ERROR;
        }
    }
    NVIC_DisableIRQ(INTCEC0RX_IRQn);
    NVIC_DisableIRQ(INTCEC0TX_IRQn);
    bsp_irq_cecrx_disable();
    bsp_irq_cectx_disable();
    /*----------------------*/
    /* Usb Uart             */
    /*----------------------*/
    uart_io_finalize();
    return (result);
}
/*--------------------------------------------------*/
/** 
  * @brief  Transmit handler.
  * @param  id       :User ID.
  * @param  result   :Result.
  * @param  tx_status:TX Status.
  * @return -
  * @retval -
  * @note   Called by uart driver.
  */
/*--------------------------------------------------*/
static void transmit_handler(uint32_t id, TXZ_Result result, uint32_t tx_status)
{
    cec_io_t *p_instance = (cec_io_t *)id;

    if (p_instance != MAIN_NULL)
    {
        cec_io_private_tx_t *p_tx = &p_instance->info.tx;

        if (p_tx->state != CEC_IO_SEND_IDLE)
        {
            /*------------------------------*/
            /*  Result Check                */
            /*------------------------------*/
            if (result == TXZ_SUCCESS)
            {
                p_tx->state = CEC_IO_SEND_SUCCESS;
            }
            else
            {
                if((tx_status & CECxTSTAT_CECTIACK_MASK) == CECxTSTAT_CECTIACK_SET)
                {
                    p_tx->state = CEC_IO_SEND_NO_ACK;
                }
                else{
                    p_tx->state = CEC_IO_SEND_FAILURE;
                }
            }
        }
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  Receive handler.
  * @param  id       :User ID.
  * @param  result   :Result.
  * @param  p_info   :Received information Source Address.
  * @param  rx_status:RX Status.
  * @return -
  * @retval -
  * @note   Called by uart driver.
  */
/*--------------------------------------------------*/
static void receive_handler(uint32_t id, TXZ_Result result, cec_receive_t *p_info, uint32_t rx_status)
{
    cec_io_t *p_instance = (cec_io_t *)id;

    if (p_instance != MAIN_NULL)
    {
        cec_io_private_rx_t *p_rx = &p_instance->info.rx;

        if (p_rx->state == CEC_IO_RECV_RUN)
        {
            if (result == TXZ_SUCCESS)
            {
                if (p_info != MAIN_NULL)
                {
                    if ( p_info->num < CEC_IO_RECEIVE_MAX)
                    {
                        p_rx->length = p_info->num;
                        p_rx->state = CEC_IO_RECV_SUCCESS;
                    }
                    else
                    {
                        p_rx->state = CEC_IO_RECV_FAILURE;
                    }
                }
                else
                {
                    p_rx->state = CEC_IO_RECV_FAILURE;
                }
            }
            else
            {
                if( (p_info != MAIN_NULL) && ((rx_status & CECxRSTAT_CECRIBFOV_MASK) == CECxRSTAT_CECRIBFOV_SET) )
                {
                    if ( p_info->num < CEC_IO_RECEIVE_MAX)
                    {
                        p_rx->length = p_info->num;
                    }
                    p_rx->state = CEC_IO_RECV_OVER_FLOW;
                }
                else
                {
                    p_rx->state = CEC_IO_RECV_FAILURE;
                }
            }
        }
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  Display CEC Address
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result disp_cec_address(void)
{
    TXZ_Result result = TXZ_SUCCESS;
    
    if(cec_obj.init.logical_address & CECxADD_CECADD_0)
    {
        if(printf("CEC Address = 0\n") == -1 )
        {
           result = TXZ_ERROR;
        }
    }
    if(cec_obj.init.logical_address & CECxADD_CECADD_1)
    {
        if(printf("CEC Address = 1\n") == -1 )
        {
           result = TXZ_ERROR;
        }
    }
    if(cec_obj.init.logical_address & CECxADD_CECADD_2)
    {
        if(printf("CEC Address = 2\n") == -1 )
        {
           result = TXZ_ERROR;
        }
    }
    if(cec_obj.init.logical_address & CECxADD_CECADD_3)
    {
        if(printf("CEC Address = 3\n") == -1 )
        {
           result = TXZ_ERROR;
        }
    }
    if(cec_obj.init.logical_address & CECxADD_CECADD_4)
    {
        if(printf("CEC Address = 4\n") == -1 )
        {
           result = TXZ_ERROR;
        }
    }
    if(cec_obj.init.logical_address & CECxADD_CECADD_5)
    {
        if(printf("CEC Address = 5\n") == -1 )
        {
           result = TXZ_ERROR;
        }
    }
    if(cec_obj.init.logical_address & CECxADD_CECADD_6)
    {
        if(printf("CEC Address = 6\n") == -1 )
        {
           result = TXZ_ERROR;
        }
    }
    if(cec_obj.init.logical_address & CECxADD_CECADD_7)
    {
        if(printf("CEC Address = 7\n") == -1 )
        {
           result = TXZ_ERROR;
        }
    }
    if(cec_obj.init.logical_address & CECxADD_CECADD_8)
    {
        if(printf("CEC Address = 8\n") == -1 )
        {
           result = TXZ_ERROR;
        }
    }
    if(cec_obj.init.logical_address & CECxADD_CECADD_9)
    {
        if(printf("CEC Address = 9\n") == -1 )
        {
           result = TXZ_ERROR;
        }
    }
    if(cec_obj.init.logical_address & CECxADD_CECADD_A)
    {
        if(printf("CEC Address = A\n") == -1 )
        {
           result = TXZ_ERROR;
        }
    }
    if(cec_obj.init.logical_address & CECxADD_CECADD_B)
    {
        if(printf("CEC Address = B\n") == -1 )
        {
           result = TXZ_ERROR;
        }
    }
    if(cec_obj.init.logical_address & CECxADD_CECADD_C)
    {
        if(printf("CEC Address = C\n") == -1 )
        {
           result = TXZ_ERROR;
        }
    }
    if(cec_obj.init.logical_address & CECxADD_CECADD_D)
    {
        if(printf("CEC Address = D\n") == -1 )
        {
           result = TXZ_ERROR;
        }
    }
    if(cec_obj.init.logical_address & CECxADD_CECADD_E)
    {
        if(printf("CEC Address = E\n") == -1 )
        {
           result = TXZ_ERROR;
        }
    }
    if(cec_obj.init.logical_address & CECxADD_CECADD_F)
    {
        if(printf("CEC Address = F\n") == -1 )
        {
           result = TXZ_ERROR;
        }
    }
    return(result);
}

/*--------------------------------------------------*/
/** 
  * @brief  IRQ CECRX Enable
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void bsp_irq_cecrx_enable(void)
{
    /*-----------------------------*/
    /* Bit | Bit Symbol   | Value  */
    /*-----------------------------*/
    /* 7   | INTNCLR      | -      */
    /* 6   | INTPCLR      | 0      */
    /* 5   | INTNFLG      | -      */
    /* 4   | INTPFLG      | 0      */
    /* 3-1 | INTMODE[2:0] | - (11) */
    /* 0   | INTEN        | 1      */
    /*-----------------------------*/
    TSB_IA->IMC50 = (uint8_t)0x01;
}

/*--------------------------------------------------*/
/** 
  * @brief  IRQ CECRX Disable
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void bsp_irq_cecrx_disable(void)
{
    /*--- IAIMC50 Disable ---*/
    /*-----------------------------*/
    /* Bit | Bit Symbol   | Value  */
    /*-----------------------------*/
    /* 7   | INTNCLR      | -      */
    /* 6   | INTPCLR      | 0      */
    /* 5   | INTNFLG      | -      */
    /* 4   | INTPFLG      | 0      */
    /* 3-1 | INTMODE[2:0] | - (11) */
    /* 0   | INTEN        | 0      */
    /*-----------------------------*/
    TSB_IA->IMC50 = (uint8_t)0x00;
    /*--- IAIMC50 Clear   ---*/
    /*-----------------------------*/
    /* Bit | Bit Symbol   | Value  */
    /*-----------------------------*/
    /* 7   | INTNCLR      | -      */
    /* 6   | INTPCLR      | 1      */
    /* 5   | INTNFLG      | -      */
    /* 4   | INTPFLG      | 0      */
    /* 3-1 | INTMODE[2:0] | - (11) */
    /* 0   | INTEN        | 0      */
    /*-----------------------------*/
    TSB_IA->IMC50 = (uint8_t)0x40;
}


/*--------------------------------------------------*/
/** 
  * @brief  IRQ CECTX Enable
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void bsp_irq_cectx_enable(void)
{
    /*-----------------------------*/
    /* Bit | Bit Symbol   | Value  */
    /*-----------------------------*/
    /* 7   | INTNCLR      | -      */
    /* 6   | INTPCLR      | 0      */
    /* 5   | INTNFLG      | -      */
    /* 4   | INTPFLG      | 0      */
    /* 3-1 | INTMODE[2:0] | - (11) */
    /* 0   | INTEN        | 1      */
    /*-----------------------------*/
    TSB_IA->IMC51 = (uint8_t)0x01;
}

/*--------------------------------------------------*/
/** 
  * @brief  IRQ CECTX Disable
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void bsp_irq_cectx_disable(void)
{
    /*--- IAIMC51 Disable ---*/
    /*-----------------------------*/
    /* Bit | Bit Symbol   | Value  */
    /*-----------------------------*/
    /* 7   | INTNCLR      | -      */
    /* 6   | INTPCLR      | 0      */
    /* 5   | INTNFLG      | -      */
    /* 4   | INTPFLG      | 0      */
    /* 3-1 | INTMODE[2:0] | - (11) */
    /* 0   | INTEN        | 0      */
    /*-----------------------------*/
    TSB_IA->IMC51 = (uint8_t)0x00;
    /*--- IAIMC51 Clear   ---*/
    /*-----------------------------*/
    /* Bit | Bit Symbol   | Value  */
    /*-----------------------------*/
    /* 7   | INTNCLR      | -      */
    /* 6   | INTPCLR      | 1      */
    /* 5   | INTNFLG      | -      */
    /* 4   | INTPFLG      | 0      */
    /* 3-1 | INTMODE[2:0] | - (11) */
    /* 0   | INTEN        | 0      */
    /*-----------------------------*/
    TSB_IA->IMC51 = (uint8_t)0x40;
}

/*--------------------------------------------------*/
/** 
  * @brief  Error Hook
  * @param  -
  * @return -
  * @retval -
  * @note   for debug
  */
/*--------------------------------------------------*/
static void cec_io_error(void)
{
#ifdef __DEBUG__
    while(1)
    {
        __NOP();
    }
#endif
}

/**
 *  @}
 */ /* End of group CEC_CH_TO_CH_Private_fuctions */


/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup CEC_CH_TO_CH_Exported_functions CEC_CH_TO_CH Exported Functions
 *  @{
 */
int put_s_cec(const char *s);
int get_s_cec(char *s, uint32_t *wp);
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
int main(void)
{
    /*----------------------*/
    /* BSP initialize       */
    /*----------------------*/
    bsp_initialize();
    /*----------------------*/
    /* Initialize           */
    /*----------------------*/
    /*--- variable    ---*/
    variable_initialize();
    /*--- application ---*/
    if (application_initialize() != TXZ_SUCCESS)
    {
        proc = PROC_ERROR;
    }
    /*----------------------*/
    /* Main Processing      */
    /*----------------------*/
    while(proc != PROC_ERROR)
    {
        count = get_s_cec(read_data, &total_count);
        if(count < 0)
        {
            proc =  PROC_OUTPUT_RECV_ERR;
        }
        switch(proc)
        {
        /*--- Start ---*/
        case PROC_START:
            {
                if(disp_cec_address() != TXZ_SUCCESS)
                {
                    proc = PROC_STOP;
                }
                else
                {
                    proc = PROC_OUTPUT_MSG;
                }
            }
            break;
        /*--- Stop ---*/
        case PROC_STOP:
            {
                proc = PROC_ERROR;
            }
            break;
        /*--- Output "First Message" ---*/
        case PROC_OUTPUT_MSG:
            {
                /* Clear input data.     */
                clear_input_data();
                /* Output first message. */
                if (printf("Input = ") != -1 )
                {
                    proc = PROC_INPUT;
                }
                else
                {
                    proc = PROC_STOP;
                }
            }
            break;
        /*--- Wait to Input ---*/
        case PROC_INPUT:
            {
                int data;

                data = get_char_once();
                if (data == -1)
                {
                    break;
                }
                if(input.wp < UART_IO_RECEIVE_MAX)
                {
                    input.data[input.wp] = (char)data;
                    input.wp++;
                    switch ((char)data)
                    {
                    /*--- CR: carriage return ---*/
                    case '\r':
                        /* no process */
                        break;
                    /*--- LF: line feed ---*/
                    case '\n':
                        proc = PROC_OUTPUT_ECHO;
                        break;
                    /*--- default ---*/
                    default:
                        /* no process */
                        break;
                    }
                }
                else
                {
                    proc = PROC_OUTPUT_ERR;
                }
            }
            break;
        /*--- Output "Echo Back Data" ---*/
        case PROC_OUTPUT_ECHO:
            {
                if (printf(" CEC Snd = %s", (const char *)&input.data[0]) != -1 )
                {
                   proc = PROC_CEC_TRANSFER;
                }
                else
                {
                    proc = PROC_STOP;
                }
            }
            break;
        /*--- Output "Input Error Message" ---*/
        case PROC_OUTPUT_ERR:
            {
                if (printf("\n Input Error !! \n") != -1 )
                {
                        proc = PROC_OUTPUT_MSG;
                }
                else
                {
                    proc = PROC_STOP;
                }
            }
            break;
        /*--- Output "Receive Error Message" ---*/
        case PROC_OUTPUT_RECV_ERR:
            {
                if (printf("\n Receive Error !! \n") != -1 )
                {
                        proc = PROC_OUTPUT_MSG;
                }
                else
                {
                    proc = PROC_STOP;
                }
            }
            break;
        /*--- CEC Trasfer ---*/
        case PROC_CEC_TRANSFER:
            if(put_s_cec((const char *)&input.data[0]) < 0)
            {
                /* no process */
            }
            proc = PROC_OUTPUT_MSG;
            break;
        /*--- Error ---*/
        case PROC_ERROR:
            /* no processing */
            break;
        /*--- Default ---*/
        default:
            proc = PROC_ERROR;
            break;
        }
    }
    /*----------------------*/
    /* Finalize             */
    /*----------------------*/
    /*--- application ---*/
    if (application_finalize() != TXZ_SUCCESS)
    {
        /* no processing */
    }
    /*----------------------*/
    /* BSP finalize         */
    /*----------------------*/
    bsp_finalize();
#ifdef __DEBUG__
    assert_failed(MAIN_NULL, 0);
#endif

    return(0);
}

/*--------------------------------------------------*/
/** 
  * @brief      The character is shown to an output stream.
  * @param[in]  s       :Source address.
  * @retval     Suceess :Num of output data(bytes).
  * @retval     Failure :-1
  * @pre        Have finished termination of a designated character string by NULL character.
  * @note       -
  */
/*--------------------------------------------------*/
int put_s_cec(const char *s)
{
    int ret = -1;
    cec_io_private_tx_t *p_tx = &instance.info.tx;
    uint32_t last_tx_status = 0;

    if (p_tx->state == CEC_IO_SEND_IDLE)
    {
        volatile TXZ_WorkState loopBreak = TXZ_BUSY;

        /*------------------------------*/
        /*  Check Data                  */
        /*------------------------------*/
        p_tx->wp      = 0;
        p_tx->rp      = 0;
        p_tx->state   = CEC_IO_SEND_RUN;
        while (loopBreak == TXZ_BUSY)
        {
            if(p_tx->wp < (CEC_IO_SEND_MAX-1))
            {
                if ((*s != '\0') && (*s != '\r') && (*s != '\n'))
                {
                    p_tx->buff[p_tx->wp] = (uint8_t)*s;
                    p_tx->wp++;
                    s++;
                }
                else
                {
                    loopBreak = TXZ_DONE;
                }
            }
            else
            {
                p_tx->wp = CEC_IO_SEND_MAX-1;
                p_tx->buff[p_tx->wp] = '\0';
                p_tx->wp++;
                loopBreak = TXZ_DONE;
            }
        }
        if (p_tx->wp != 0)
        {
            cec_transmit_t param;

            /*------------------------------*/
            /*  Transmit                    */
            /*------------------------------*/
            param.p_data    = &p_tx->buff[0];
            param.num       = p_tx->wp;
            if((p_tx->buff[0] & 0x0F) == 0x0F)
            {
                param.broadcast = CECxTCR_CECBRD_ON;
            }
            else
            {
                param.broadcast = CECxTCR_CECBRD_OFF;
            }
            if (cec_transmitIt(&cec_obj, &param) != TXZ_SUCCESS)
            {
                cec_io_error();
            }
            /*------------------------------*/
            /*  Wait to transmit            */
            /*------------------------------*/
            loopBreak = TXZ_BUSY;
            while(loopBreak == TXZ_BUSY)
            {
                switch (p_tx->state)
                {
                case CEC_IO_SEND_RUN:
                    /* no process */
                    break;
                case CEC_IO_SEND_SUCCESS:
                    loopBreak = TXZ_DONE;
                    break;
                default:
                    loopBreak = TXZ_DONE;
                    break;
                }
            }
            /*------------------------------*/
            /*  Check Error                 */
            /*------------------------------*/
            if(cec_get_tx_status(&cec_obj, &last_tx_status) != TXZ_SUCCESS)
            {
                /* no process */
            }
            
            if (p_tx->state == CEC_IO_SEND_FAILURE)
            {
                if(printf("\n***CEC Snd Error***\n") < 0)
                {
                    /* no process */
                }
                if(last_tx_status & CECxTSTAT_CECTIUR_MASK)
                {
                    if(printf("  Buffer Under Run\n") < 0)
                    {
                        /* no process */
                    }
                }
                if(last_tx_status & CECxTSTAT_CECTIACK_MASK)
                {
                    if(printf("  ACK Error\n") <0)
                    {
                        /* no process */
                    }
                }
                if(last_tx_status & CECxTSTAT_CECTIAL_MASK)
                {
                    if(printf("  Arbitorarion Error\n") < 0)
                    {
                        /* no process */
                    }
                }
            }
            else if (p_tx->state == CEC_IO_SEND_NO_ACK)
            {
                if(printf("\n***No ACK***\n") < 0)
                {
                    /* no process */
                }
                if(last_tx_status & CECxTSTAT_CECTIUR_MASK)
                {
                    if(printf("  Buffer Under Run\n") < 0)
                    {
                        /* no process */
                    }
                }
                if(last_tx_status & CECxTSTAT_CECTIACK_MASK)
                {
                    if(printf("  ACK Error\n") < 0)
                    {
                        /* no process */
                    }
                }
                if(last_tx_status & CECxTSTAT_CECTIAL_MASK)
                {
                    if(printf("  Abitorarion Error\n") < 0)
                    {
                        /* no process */
                    }
                }
            }
            ret = (int)p_tx->wp;
        }
        /* Clear status. */
        p_tx->state = CEC_IO_SEND_IDLE;
    }

    return (ret);
}

/*--------------------------------------------------*/
/** 
  * @brief  The string is acquired from an input stream.
  * @param[in]  s       :Destination address.
  * @param[in]  wp      :Destination address write pointer.
  * @retval     Suceess :Num of input data(bytes).
  * @retval     Failure :-1
  * @note   -
  */
/*--------------------------------------------------*/
int get_s_cec(char *s, uint32_t *wp)
{
    int ret = 0;
    uint8_t i = 0;
    volatile TXZ_WorkState loopBreak = TXZ_BUSY;
    char *data_ptr = s + *wp;
    cec_io_private_rx_t *p_rx = &instance.info.rx;
    if( (p_rx->state == CEC_IO_RECV_SUCCESS) || (p_rx->state == CEC_IO_RECV_RUN) )
    {
        /*>>> Critical information. */
        volatile uint32_t length = p_rx->length;
        /*<<< Critical information. */
        
        /*------------------------------*/
        /*  Check Receive Length        */
        /*------------------------------*/
        if(length < CEC_IO_RECEIVE_MAX)
        {
            if(length > 0)
            {
                char *ptr = (char *)&p_rx->buff[p_rx->rp];
                for(i=0; i < length; ++i)
                {
                    *data_ptr++ = *ptr++;
                }
                ret = (int)length;
                *wp += length;
                /*>>> Critical information. */
                NVIC_DisableIRQ(INTCEC0RX_IRQn);
                p_rx->length = 0x00;
                p_rx->rp = 0;
                NVIC_EnableIRQ(INTCEC0RX_IRQn);
                /*<<< Critical information. */
                if (p_rx->state == CEC_IO_RECV_SUCCESS)
                {
                    *data_ptr = '\0';
                    if(printf("\nCEC Rcv = %s\n", s) < 0)
                    {
                        /* no process */
                    }
                    *wp = 0;
                    p_rx->state = CEC_IO_RECV_RUN;
                }
            }
        }
        else
        {
            p_rx->state = CEC_IO_RECV_FAILURE;
            ret = -1;
        }
    }
    if(p_rx->state == CEC_IO_RECV_OVER_FLOW)
    {
        if(printf("\nCEC Rcv External Buffer Over Flow!!!\n") < 0)
        {
            /* no process */
        }
        p_rx->state = CEC_IO_RECV_RUN;
    }
    if(p_rx->state == CEC_IO_RECV_FAILURE)
    {
        uint32_t status = 0;
        
        if(printf("\n***CEC Rcv Error***\n") < 0)
        {
            /* no process */
        }
        if(cec_get_rx_status(&cec_obj, &status) != TXZ_SUCCESS)
        {
            /* no process */
        }
        if((status & CECxRSTAT_CECRIWAV_MASK) == CECxRSTAT_CECRIWAV_SET)
        {
            if(printf("\nCEC Rcv Wave Error\n") < 0)
            {
                /* no process */
            }
        }
        if((status & CECxRSTAT_CECRIOR_MASK) == CECxRSTAT_CECRIOR_SET)
        {
            if(printf("\nCEC Rcv Over Flow Error\n")< 0)
            {
                /* no process */
            }
        }
        if((status & CECxRSTAT_CECRIACK_MASK) == CECxRSTAT_CECRIACK_SET)
        {
            if(printf("\nCEC Rcv ACK Collision\n") < 0)
            {
                /* no process */
            }
        }
        if((status & CECxRSTAT_CECRIMIN_MASK) == CECxRSTAT_CECRIMIN_SET)
        {
            if(printf("\nCEC Rcv Min. Error\n") < 0)
            {
                /* no process */
            }
        }
        if((status & CECxRSTAT_CECRIMAX_MASK) == CECxRSTAT_CECRIMAX_SET)
        {
            if(printf("\nCEC Rcv Max. Error\n") < 0)
            {
                /* no process */
            }
        }
        p_rx->state = CEC_IO_RECV_RUN;
    }
    return(ret);
}


/*--------------------------------------------------*/
/** 
  * @brief  CEC Receive reception IRQ.
  * @param  -
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_cec_rx(void)
{
    cec_receive_irq_handler(&cec_obj);
}

/*--------------------------------------------------*/
/** 
  * @brief  CEC Transfer reception IRQ.
  * @param  -
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_cec_tx(void)
{
    cec_transmit_irq_handler(&cec_obj);
}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART reception IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_rx(BSPUsbUart uart)
{
    switch (uart)
    {
    /*--- usb uart 0 ---*/
    case BSP_USB_UART_0:
        uart_io_rx_irq_handler();
        break;
    /*--- usb uart 1 ---*/
    case BSP_USB_UART_1:
        uart_io_rx_irq_handler();
        break;
    /*--- default    ---*/
    default:
        /* no processing */
        break;
    }

}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART transmit IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_tx(BSPUsbUart uart)
{
    switch (uart)
    {
        /*--- usb uart 0 ---*/
    case BSP_USB_UART_0:
        uart_io_tx_irq_handler();
        break;
        /*--- usb uart 1 ---*/
    case BSP_USB_UART_1:
        uart_io_tx_irq_handler();
        break;
    /*--- default    ---*/
    default:
        /* no processing */
        break;
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART error IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_err(BSPUsbUart uart)
{
    switch (uart)
    {
        /*--- usb uart 0 ---*/
    case BSP_USB_UART_0:
        uart_io_err_irq_handler();
        break;
        /*--- usb uart 1 ---*/
    case BSP_USB_UART_1:
        uart_io_err_irq_handler();
        break;
    /*--- default    ---*/
    default:
        /* no processing */
        break;
    }
}

#ifdef __DEBUG__
/*--------------------------------------------------*/
/** 
  * @brief  Failure Assert.
  * @note   for debug
  */
/*--------------------------------------------------*/
void assert_failed(char *file, int32_t line)
{
    while (1) {
        __NOP();
    }
}
#endif
/**
 *  @}
 */ /* End of group CEC_CH_TO_CH_Exported_functions */

/**
 *  @}
 */ /* End of group CEC_CH_TO_CH */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
