/** 
 *******************************************************************************
 * @file    main.c
 * @brief   EBIF SRAM Sample Application.
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2020
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif
/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "bsp.h"
#include "bsp_uart_io.h"
#include "ebif_wapper.h"
/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup EBIF_SRAM EBIF_SRAM Sample Appli
 *  @{
 */

/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup EBIF_SRAM_Private_macro EBIF_SRAM Private Macro
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group EBIF_SRAM_Private_macro */


/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup EBIF_SRAM_Private_define EBIF_SRAM Private Define
 *  @{
 */
/* no define */

/** 
 *  @}
 */ /* End of group EBIF_SRAM_Private_define */

/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup EBIF_SRAM_Private_define EBIF_SRAM Private Define
 *  @{
 */

#define MAIN_NULL       ((void *)0)        /*!< Null Pointer.            */
#define STRING_NUM      (0x10)              /*!< String Size. */
#define SRAM_BASE       (uint32_t)(0x61000000)        /*!< Access location address. */
/*----------------------*/
/*  UART Setting        */
/*----------------------*/
/** 
 *  @defgroup UART_Setting UART Setting
 *  @{
 */
#define USER_ID_UART    ((uint32_t)1)   /*!< User ID :USB UART (Any value) */
/** 
 *  @}
 */ /* End of group UART_Setting */

/** 
 *  @}
 */ /* End of group EBIF_SRAM_Private_define */

/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup EBIF_SRAM_Private_define EBIF_SRAM Private Define
 *  @{
 */

/*----------------------------------*/
/** 
 * @enum   Processing
 * @brief  Processing mode.
*/
/*----------------------------------*/
typedef enum
{
    PROC_IDLE,              /*!< Idle.                        */
    PROC_OUTPUT_MSG,        /*!< Output first message.        */
    PROC_INPUT,             /*!< Wait to input.               */
    PROC_COMMAND_PARSE,     /*!< Output Echo back data.       */
    PROC_OUTPUT_ERR,        /*!< Output input error message.  */
    PROC_ERROR              /*!< Error.                       */
} Processing;

/** 
 *  @}
 */ /* End of group EBIF_SRAM_Private_define */

/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup EBIF_SRAM_Private_typedef EBIF_SRAM Private Typedef
 *  @{
 */
/*----------------------------------*/
/** 
 * @struct input_t
 * @brief  Input data information.
*/
/*----------------------------------*/
typedef struct
{
    char    data[(UART_IO_RECEIVE_MAX+1)];  /*!< Input data(+ for NULL 1byte). */
    uint8_t wp;                             /*!< Write pointer.                */
} input_t;

/** 
 *  @}
 */ /* End of group EBIF_SRAM_Private_typedef */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup EBIF_SRAM_Private_variables EBIF_SRAM Private Variables
 *  @{
 */
static EBIF_setting_t ebif_config;
static CS_setting_t   cs1_config;

static Processing proc;         /*!< Processing mode.        */
static input_t input;           /*!< Input data information. */
static gpio_t port;             /*!< Port driver instance. */

#if     defined ( __CC_ARM )    /* ARM Compiler */
    #pragma arm section zidata = "SRAM1"
#elif   defined ( __ICCARM__ )  /* IAR Compiler */
    #pragma location = "SRAM1"
#endif
static uint8_t write_data[STRING_NUM];  /*!< Write data information. */
/**
 *  @}
 */ /* End of group EBIF_SRAM_Private_variables */


/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup EBIF_SRAM_Private_functions EBIF_SRAM Private Fuctions
 *  @{
 */
static void port_error(void);
static void clear_data(void);
static TXZ_Result application_initialize(void);
static void EBIF_port_initialize(void);
static void EBIF_interface_initialize(void);

/*--------------------------------------------------*/
/** 
  * @brief  Error Hook
  * @param  -
  * @return -
  * @retval -
  * @note   for debug
  */
/*--------------------------------------------------*/
static void port_error(void)
{
    while(1)
    {
        __NOP();
    }
}
/*--------------------------------------------------*/
/** 
  * @brief  Clear input data.
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void clear_data(void)
{
    uint8_t i;

    for (i = 0; i < (UART_IO_RECEIVE_MAX+1); i++)
    {
        input.data[i] = 0;
    }
    input.wp = 0;
    
}
/*--------------------------------------------------*/
/** 
  * @brief  Application Initialize.
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result application_initialize(void)
{
   TXZ_Result result = TXZ_SUCCESS;

    /*----------------------*/
    /* Usb Uart             */
    /*----------------------*/
    /* Tx & Rx use same UART channel. */
    {
        uart_io_initial_setting_t param;

        param.id         = USER_ID_UART;
        switch (bsp_get_usb_uart_tx_ch(BSP_USB_UART_1))
        {
        case 0:
            param.p_reg  = TSB_UART0;
            break;
        case 1:
            param.p_reg  = TSB_UART1;
            break;
        case 2:
            param.p_reg  = TSB_UART2;
            break;
        default:
            param.p_reg  = MAIN_NULL;
            break;
        }
        param.boudrate   = bsp_get_usb_uart_tx_baudrate(BSP_USB_UART_1);
        param.irq.tx     = bsp_get_usb_uart_tx_nvic(BSP_USB_UART_1);
        param.irq.rx     = bsp_get_usb_uart_rx_nvic(BSP_USB_UART_1);
        param.irq.err    = bsp_get_usb_uart_err_nvic(BSP_USB_UART_1);
        if (param.p_reg != MAIN_NULL)
        {
            result  = uart_io_initialize(&param);
        }
        else
        {
            result = TXZ_ERROR;
        }
    }
    return (result);
}
/*--------------------------------------------------*/
/** 
  * @brief  Port Initialize.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void EBIF_port_initialize(void)
{    
    /*===============================================================================*/
    /*  [Port D]                                                                     */
    /*-------------------------------------------------------------------------------*/
    /*  Port | Function  | I/O | PDDATA | PDCR | PDFRn | PDOD | PDPUP | PDDN | PDIE  */
    /*-------------------------------------------------------------------------------*/
    /*  PD0  | ED00      | IO  | 0      | 1    | PCFR1 | 0    | 0     | 0    | 1     */
    /*  PD1  | ED01      | IO  | 0      | 1    | PCFR1 | 0    | 0     | 0    | 1     */
    /*  PD2  | ED02      | IO  | 0      | 1    | PCFR1 | 0    | 0     | 0    | 1     */
    /*  PD3  | ED03      | IO  | 0      | 1    | PCFR1 | 0    | 0     | 0    | 1     */
    /*  PD4  | ED04      | IO  | 0      | 1    | PCFR1 | 0    | 0     | 0    | 1     */
    /*  PD5  | ED05      | IO  | 0      | 1    | PCFR1 | 0    | 0     | 0    | 1     */
    /*  PD6  | ED06      | IO  | 0      | 1    | PCFR1 | 0    | 0     | 0    | 1     */
    /*  PD7  | ED07      | IO  | 0      | 1    | PCFR1 | 0    | 0     | 0    | 1     */
    /*===============================================================================*/
    /*--- Construct  ---*/
    /* Register Allocation */
    port.p_pd_instance = TSB_PD;
    /*--- Initialize ---*/
    if (gpio_init(&port, GPIO_PORT_D) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*--- Setting    ---*/
    /*- PD0 -*/
    if (gpio_func(&port, GPIO_PORT_D, GPIO_PORT_0, (uint32_t)GPIO_PD0_ED00, GPIO_PIN_INOUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PD1 -*/
    if (gpio_func(&port, GPIO_PORT_D, GPIO_PORT_1, (uint32_t)GPIO_PD1_ED01, GPIO_PIN_INOUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PD2 -*/
    if (gpio_func(&port, GPIO_PORT_D, GPIO_PORT_2, (uint32_t)GPIO_PD2_ED02, GPIO_PIN_INOUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PD3 -*/
    if (gpio_func(&port, GPIO_PORT_D, GPIO_PORT_3, (uint32_t)GPIO_PD3_ED03, GPIO_PIN_INOUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PD4 -*/
    if (gpio_func(&port, GPIO_PORT_D, GPIO_PORT_4, (uint32_t)GPIO_PD4_ED04, GPIO_PIN_INOUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PD5 -*/
    if (gpio_func(&port, GPIO_PORT_D, GPIO_PORT_5, (uint32_t)GPIO_PD5_ED05, GPIO_PIN_INOUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PD6 -*/
    if (gpio_func(&port, GPIO_PORT_D, GPIO_PORT_6, (uint32_t)GPIO_PD6_ED06, GPIO_PIN_INOUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PD7 -*/
    if (gpio_func(&port, GPIO_PORT_D, GPIO_PORT_7, (uint32_t)GPIO_PD7_ED07, GPIO_PIN_INOUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*===============================================================================*/
    /*  [Port E]                                                                     */
    /*-------------------------------------------------------------------------------*/
    /*  Port | Function  | I/O | PEDATA | PECR | PEFRn | PEOD | PEPUP | PEDN | PEIE  */
    /*-------------------------------------------------------------------------------*/
    /*  PE0  | ED08      | IO  | 0      | 1    | PDFR1 | 0    | 0     | 0    | 1     */
    /*  PE1  | ED09      | IO  | 0      | 1    | PDFR1 | 0    | 0     | 0    | 1     */
    /*  PE2  | ED10      | IO  | 0      | 1    | PDFR1 | 0    | 0     | 0    | 1     */
    /*  PE3  | ED11      | IO  | 0      | 1    | PDFR1 | 0    | 0     | 0    | 1     */
    /*  PE4  | ED12      | IO  | 0      | 1    | PDFR1 | 0    | 0     | 0    | 1     */
    /*  PE5  | ED13      | IO  | 0      | 1    | PDFR1 | 0    | 0     | 0    | 1     */
    /*  PE6  | ED14      | IO  | 0      | 1    | PDFR1 | 0    | 0     | 0    | 1     */
    /*  PE7  | ED15      | IO  | 0      | 1    | PDFR1 | 0    | 0     | 0    | 1     */
    /*===============================================================================*/
    /*--- Construct  ---*/
    /* Register Allocation */
    port.p_pe_instance = TSB_PE;
    /*--- Initialize ---*/
    if (gpio_init(&port, GPIO_PORT_E) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*--- Setting    ---*/
    /*- PE0 -*/
    if (gpio_func(&port, GPIO_PORT_E, GPIO_PORT_0, (uint32_t)GPIO_PE0_ED08, GPIO_PIN_INOUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PE1 -*/
    if (gpio_func(&port, GPIO_PORT_E, GPIO_PORT_1, (uint32_t)GPIO_PE1_ED09, GPIO_PIN_INOUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PE2 -*/
    if (gpio_func(&port, GPIO_PORT_E, GPIO_PORT_2, (uint32_t)GPIO_PE2_ED10, GPIO_PIN_INOUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PE3 -*/
    if (gpio_func(&port, GPIO_PORT_E, GPIO_PORT_3, (uint32_t)GPIO_PE3_ED11, GPIO_PIN_INOUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PE4 -*/
    if (gpio_func(&port, GPIO_PORT_E, GPIO_PORT_4, (uint32_t)GPIO_PE4_ED12, GPIO_PIN_INOUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PE5 -*/
    if (gpio_func(&port, GPIO_PORT_E, GPIO_PORT_5, (uint32_t)GPIO_PE5_ED13, GPIO_PIN_INOUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PE6 -*/
    if (gpio_func(&port, GPIO_PORT_E, GPIO_PORT_6, (uint32_t)GPIO_PE6_ED14, GPIO_PIN_INOUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PE7 -*/
    if (gpio_func(&port, GPIO_PORT_E, GPIO_PORT_7, (uint32_t)GPIO_PE7_ED15, GPIO_PIN_INOUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*===============================================================================*/
    /*  [Port F]                                                                     */
    /*-------------------------------------------------------------------------------*/
    /*  Port | Function  | I/O | PFDATA | PFCR | PFFRn | PFOD | PFPUP | PFDN | PFIE  */
    /*-------------------------------------------------------------------------------*/
    /*  PF0  | ERD_N     | O   | 0      | 1    | PFFR1 | 0    | 0     | 0    | 0     */
    /*  PF1  | EWR_N     | O   | 0      | 1    | PFFR1 | 0    | 0     | 0    | 0     */
    /*  PF2  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PF3  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PF4  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PF5  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PF6  | EBELL_N   | O   | 0      | 1    | PFFR1 | 0    | 0     | 0    | 0     */
    /*  PF7  | EBELH_N   | O   | 0      | 1    | PFFR1 | 0    | 0     | 0    | 0     */
    /*===============================================================================*/
    /*--- Construct  ---*/
    /* Register Allocation */
    port.p_pf_instance = TSB_PF;
    /*--- Initialize ---*/
    if (gpio_init(&port, GPIO_PORT_F) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*--- Setting    ---*/
    /*- PF0 -*/
    if (gpio_func(&port, GPIO_PORT_F, GPIO_PORT_0, (uint32_t)GPIO_PF0_ERD_N, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PF1 -*/
    if (gpio_func(&port, GPIO_PORT_F, GPIO_PORT_1, (uint32_t)GPIO_PF1_EWR_N, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PF6 -*/
    if (gpio_func(&port, GPIO_PORT_F, GPIO_PORT_6, (uint32_t)GPIO_PF6_EBELL_N, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PF7 -*/
    if (gpio_func(&port, GPIO_PORT_F, GPIO_PORT_7, (uint32_t)GPIO_PF7_EBELH_N, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*===============================================================================*/
    /*  [Port G]                                                                     */
    /*-------------------------------------------------------------------------------*/
    /*  Port | Function  | I/O | PGDATA | PGCR | PGFRn | PGOD | PGPUP | PGDN | PGIE  */
    /*-------------------------------------------------------------------------------*/
    /*  PG0  |  EALE     | O   | 0      | 1    | PGFR1 | 0    | 0     | 0    | 0     */
    /*  PG1  |  -        | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PG2  |  -        | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PG3  |  -        | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PG4  |  -        | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PG5  |  -        | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PG6  |  PSW 0    | I   | 0      | 0    | 0     | 0    | 0     | 0    | 1     */
    /*  PG7  |  PSW 1    | I   | 0      | 0    | 0     | 0    | 0     | 0    | 1     */
    /*===============================================================================*/
    /*--- Construct  ---*/
    /* Register Allocation */
    port.p_pg_instance = TSB_PG;
    /*--- Initialize ---*/
    if (gpio_init(&port, GPIO_PORT_G) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*--- Setting    ---*/
    /*- PG0 -*/
    if (gpio_func(&port, GPIO_PORT_G, GPIO_PORT_0, (uint32_t)GPIO_PG0_EALE, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PG1 -*/
    if (gpio_func(&port, GPIO_PORT_G, GPIO_PORT_1, (uint32_t)GPIO_PG1_EWAIT_N, GPIO_PIN_INPUT) != TXZ_SUCCESS)
    {
        port_error();
    }

    /*===============================================================================*/
    /*  [Port H]                                                                     */
    /*-------------------------------------------------------------------------------*/
    /*  Port | Function  | I/O | PHDATA | PHCR | PHFRn | PHOD | PHPUP | PHDN | PHIE  */
    /*-------------------------------------------------------------------------------*/
    /*  PH0  | UT1TXDA   | O   | 0      | 1    | PHFR5 | 0    | 0     | 0    | 0     */
    /*  PH1  | UT1RXD    | I   | 0      | 0    | PHFR5 | 0    | 0     | 0    | 1     */
    /*  PH2  |  -        | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PH3  | TDI       | -   | 0      | 0    | PHFR1 | 0    | 1     | 0    | 1     */
    /*  PH4  | SWDIO     | -   | 0      | 1    | PHFR1 | 0    | 1     | 0    | 1     */
    /*  PH5  | TCK       | -   | 0      | 0    | PHFR1 | 0    | 0     | 1    | 1     */
    /*  PH6  | TDO       | -   | 0      | 1    | PHFR1 | 0    | 0     | 0    | 0     */
    /*  PH7  | TRST_N    | -   | 0      | 0    | PHFR1 | 0    | 1     | 0    | 1     */
    /*===============================================================================*/
    /*--- Construct  ---*/
    /* Register Allocation */
    port.p_ph_instance = TSB_PH;
    /*--- Initialize ---*/
    if (gpio_init(&port, GPIO_PORT_H) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*--- Setting    ---*/
    /*- PH0 -*/
    if (gpio_func(&port, GPIO_PORT_H, GPIO_PORT_0, (uint32_t)GPIO_PH0_UT1TXDA, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PH1 -*/
    if (gpio_func(&port, GPIO_PORT_H, GPIO_PORT_1, (uint32_t)GPIO_PH1_UT1RXD, GPIO_PIN_INPUT) != TXZ_SUCCESS)
    {
        port_error();
    }

    /*===============================================================================*/
    /*  [Port K]                                                                     */
    /*-------------------------------------------------------------------------------*/
    /*  Port | Function  | I/O | PKDATA | PKCR | PKFRn | PKOD | PKPUP | PKDN | PKIE  */
    /*-------------------------------------------------------------------------------*/
    /*  PK0  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PK1  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PK2  | ECS0_N    | O   | 0      | 1    | PKFR1 | 0    | 0     | 0    | 0     */
    /*  PK3  | ECS1_N    | O   | 0      | 1    | PKFR1 | 0    | 0     | 0    | 0     */
    /*  PK4  |PORT_LED[0]| O   | 0      | 1    | 0     | 0    | 0     | 0    | 0     */
    /*  PK5  |PORT_LED[1]| O   | 0      | 1    | 0     | 0    | 0     | 0    | 0     */
    /*  PK6  |PORT_LED[2]| O   | 0      | 1    | 0     | 0    | 0     | 0    | 0     */
    /*  PK7  |PORT_LED[3]| O   | 0      | 1    | 0     | 0    | 0     | 0    | 0     */
    /*===============================================================================*/
    /*--- Construct  ---*/
    /* Register Allocation */
    port.p_pk_instance = TSB_PK;
    /*--- Initialize ---*/
    if (gpio_init(&port, GPIO_PORT_K) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*--- Setting    ---*/
    /*- PK3 -*/
    if (gpio_func(&port, GPIO_PORT_K, GPIO_PORT_3, (uint32_t)GPIO_PK3_ECS1_N, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        port_error();
    }
}


/*--------------------------------------------------*/
/** 
  * @brief  EBIF Initialize.
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/

static void EBIF_interface_initialize(void)
{
    ebif_config.exbmod_wait   = EXBMOD_WAIT_0MULTI;
    ebif_config.exbmod_select = EXBMODE_SEL_MULTIPLEXEER;
    ebif_config.cs0_reg = MAIN_NULL;
    ebif_config.cs1_reg = &cs1_config;
    ebif_config.cs0_reg = MAIN_NULL;
    
    cs1_config.exbas_start_address = SRAM_BASE;
    cs1_config.exbas_area          = ECBAS_EXAR_1M;
    
    cs1_config.exbcs_csr  = EXBCS_CSR_0CYCLE;
    cs1_config.exbcs_wrr  = EXBCS_WRR_1CYCLE;
    cs1_config.exbcs_rdr  = EXBCS_RDR_0CYCLE;
    cs1_config.exbcs_alew = EXBCS_ALEW_3CYCLE;          /* 3cycle : 3 * 1/192 * 1000 = 15.6ns */
    cs1_config.exbcs_wrs  = EXBCS_WRS_1CYCLE;
    cs1_config.exbcs_rds  = EXBCS_RDS_0CYCLE;
    cs1_config.exbcs_wsel = EXBCS_WSEL_LOW_ACTIVE;
    cs1_config.exbcs_wait = EXBCS_WAIT_INTERNAL;
    cs1_config.exbcs_csiw = EXBCS_CSIW_11WAIT;           /* 11wait = 11 * 1/192 * 1000 = 57.29ns */
    cs1_config.exbcs_csw  = EXBCS_CSW_16BIT;
    cs1_config.exbcs_csw0 = EXBCS_CSW0_ENABLE;
    
    EBIF_init(&ebif_config);
}

/**
 *  @}
 */ /* End of group EBIF_SRAM_Private_functions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup EBIF_SRAM_Exported_functions EBIF_SRAM Exported Functions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
int main(void)
{
    int num;
    int len = 0;
    int cnum = 0;
    /*----------------------*/
    /* Set Processing Mode  */
    /*----------------------*/
    proc = PROC_OUTPUT_MSG;
    /*----------------------*/
    /* Initialize           */
    /*----------------------*/
    /*----------------------*/
    /* BSP initialize       */
    /*----------------------*/
    bsp_initialize();
    EBIF_port_initialize();
    EBIF_interface_initialize();
    /* Application initialization */
    if(application_initialize() != TXZ_SUCCESS)
    {
        proc = PROC_ERROR;
    }
    {
        int i;
        for (i = 0; i < STRING_NUM; i++)
        {
            write_data[i] = 0;
        }
    }
    
    /*----------------------*/
    /* Main Processing      */
    /*----------------------*/
    while(1)
    {
        switch(proc)
        {
        /*--- Output "First Message" ---*/
        case PROC_OUTPUT_MSG:
            {
                /* Clear input data.     */
                clear_data();
                /* Output first message. */
                if (printf("command > ") != -1)
                {
                    proc = PROC_INPUT;
                }
                else
                {
                    proc = PROC_ERROR;
                }
            }
            break;
        /*--- Wait to Input ---*/
        case PROC_INPUT:
            {   
                int data;
                
                data = getchar();
                if ((data != -1) && (input.wp < UART_IO_RECEIVE_MAX))
                {
                    input.data[input.wp] = (char)data;
                    input.wp++;
                    switch ((char)data)
                    {
                    /*--- C.dataR: carriage return ---*/
                    case '\r':
                        /* no process */
                        break;
                    /*--- LF: line feed ---*/
                    case '\n':
                        len = input.wp;
                        proc = PROC_COMMAND_PARSE;
                        break;
                    /*--- default ---*/
                    default:
                        /* no process */
                        break;
                    }
                }
                else
                {
                    proc = PROC_OUTPUT_ERR;
                }
            }
            break;
        /*--- Output "Echo Back Data" ---*/
        case PROC_COMMAND_PARSE:
            {
                int i;
                int32_t res = 0;

                if ((strncmp((const char *)&input.data[0], "write", 5)) == 0)
                {
                    /* WRITE Command */
                    if (len >= 9)
                    {
                        if(printf("write data > ") < 0)
                        {
                            proc = PROC_ERROR;
                        }
                        num = (len - (6 + 2));
                        for (i = 0; (i < num) && (i < (STRING_NUM-2)); i++)
                        {
                            write_data[i] = (uint8_t)(input.data[6 + i]);
                        }
                        write_data[i] = 0x00;
                       
                        cnum = i;
                        for (i = 0; (i < num) && (i < (STRING_NUM-2)); i++)
                        {
                            if(write_data[i] != (uint8_t)(input.data[6 + i]))
                            {
                                res = -1;
                                break;
                            }
                        }
                        cnum = i;
                        
                        if (res >= 0)
                        {
                            if(printf("%s\n", (const char *)&write_data[0]) < 0)
                            {
                                proc = PROC_ERROR;
                            }
                            else
                            {
                                proc = PROC_OUTPUT_MSG;
                            }
                        }
                        else
                        {
                            if(printf("\n write error !! \n") < 0)
                            {
                                /* no process */
                            }
                            proc = PROC_ERROR;
                        }
                        if(printf("\n") < 0)
                        {
                            proc = PROC_ERROR;
                        }
                    }
                    else
                    {
                        proc = PROC_OUTPUT_ERR;
                    }
                    break;
                }
                else if ((strncmp((const char *)&input.data[0], "read", 4)) == 0)
                {
                    /* READ Command */
                    if(printf("read data > ") < 0)
                    {
                        proc = PROC_ERROR;
                    }
                    if (cnum > 0)
                    {
                        if(printf("%s\n", (const char*)&write_data[0]) < 0)
                        {
                            proc = PROC_ERROR;
                        }
                        else
                        {
                            proc = PROC_OUTPUT_MSG;
                        }
                    }
                    else
                    {
                        if(printf("\n read error !! \n") < 0)
                        {
                            /* no process */
                        }
                        proc = PROC_ERROR;
                    }
                    if(printf("\n") < 0)
                    {
                        proc = PROC_ERROR;
                    }
                }
                else if ((strncmp((const char *)&input.data[0], "bw", 2)) == 0)
                {
                    /* BYTE WRITE Command */
                    if (len > 10)
                    {
                        char str[6] = {0};
                        int32_t address;
                        char pr_data[2] = {0};
                        volatile char *ptr;

                        if(printf("byte write data > ") < 0)
                        {
                            proc = PROC_ERROR;
                        }
                        str[0]= input.data[3];
                        str[1]= input.data[4];
                        str[2]= input.data[5];
                        str[3]= input.data[6];
                        address = strtol(str, 0, 16);
                        if((address < 0) || (address > 0xFFFF))
                        {
                            if(printf("\n address error !! \n") < 0)
                            {
                                /* no process */
                            }
                            proc = PROC_ERROR;
                            break;
                        }
                        pr_data[0] = input.data[8];
                        pr_data[1] = 0;
                        ptr = (char *)((uint32_t)address+SRAM_BASE);
                        *ptr = input.data[8];
                        
                        if (*ptr == input.data[8])
                        {
                            if(printf("0x%x = %s\n", address, (const char *)&pr_data[0]) < 0)
                            {
                                proc = PROC_ERROR;
                            }
                            else
                            {
                                proc = PROC_OUTPUT_MSG;
                            }
                        }
                        else
                        {
                            if(printf("\n write error !! \n") < 0)
                            {
                                /* no process  */
                            }
                            proc = PROC_ERROR;
                        }
                        if(printf("\n") < 0)
                        {
                            proc = PROC_ERROR;
                        }
                    }
                    else
                    {
                        proc = PROC_OUTPUT_ERR;
                    }
                }
                else if ((strncmp((const char *)&input.data[0], "br", 2)) == 0)
                {
                    /* BYTE READ Command */
                    if (len > 8)
                    {
                        char str[6] = {0};
                        int32_t address;
                        char pr_data[2] = {0};
                        volatile char *ptr;

                        if(printf("byte read data > ") < 0)
                        {
                            proc = PROC_ERROR;
                        }
                        str[0]= input.data[3];
                        str[1]= input.data[4];
                        str[2]= input.data[5];
                        str[3]= input.data[6];
                        address = strtol(str, 0, 16);
                        if((address < 0) || (address > 0xFFFF))
                        {
                            if(printf("\n address error !! \n") < 0)
                            {
                                /* no process */
                            }
                            proc = PROC_ERROR;
                            break;
                        }
                        ptr = (char *)((uint32_t)address+SRAM_BASE);
                        pr_data[0] = *ptr;
                        pr_data[1] = 0;
                        if (*ptr == pr_data[0])
                        {
                            if(printf("0x%x = %s\n", address, (const char*)&pr_data[0]) < 0)
                            {
                                proc = PROC_ERROR;
                            }
                            else
                            {
                                proc = PROC_OUTPUT_MSG;
                            }
                        }
                        else
                        {
                            if(printf("\n read error !! \n") < 0)
                            {
                                /* no process */
                            }
                            proc = PROC_ERROR;
                        }
                        if(printf("\n") < 0)
                        {
                            proc = PROC_ERROR;
                        }
                    }
                    else
                    {
                        proc = PROC_OUTPUT_ERR;
                    }
                }
                else
                {
                    proc = PROC_OUTPUT_ERR;
                }
            }
            break;
                
        /*--- Output "Input Error Message" ---*/
        case PROC_OUTPUT_ERR:
            if(printf("? \n") < 0)
            {
                proc = PROC_ERROR;
            }
            else
            {
                proc = PROC_OUTPUT_MSG;
            }
            break;
        /*--- Error ---*/
        case PROC_ERROR:
            /* System Error. Please Debug. */
#ifdef __DEBUG__
            assert_failed(MAIN_NULL, 0);
#endif
            proc = PROC_OUTPUT_MSG;
            break;
        /*--- Default ---*/
        default:
            proc = PROC_ERROR;
            break;
        }
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART reception IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_rx(BSPUsbUart uart)
{
    switch (uart)
    {
    /*--- usb uart 0 ---*/
    case BSP_USB_UART_0:
        uart_io_rx_irq_handler();
        break;
    /*--- usb uart 1 ---*/
    case BSP_USB_UART_1:
        uart_io_rx_irq_handler();
        break;
    /*--- default    ---*/
    default:
        /* no processing */
        break;
    }

}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART transmit IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_tx(BSPUsbUart uart)
{
    switch (uart)
    {
      /*--- usb uart 0 ---*/
      case BSP_USB_UART_0:
        uart_io_tx_irq_handler();
        break;
     /*--- usb uart 1 ---*/
     case BSP_USB_UART_1:
        uart_io_tx_irq_handler();
        break;
     /*--- default    ---*/
     default:
        /* no processing */
        break;
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART error IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_err(BSPUsbUart uart)
{
    switch (uart)
    {
    /*--- usb uart 0 ---*/
    case BSP_USB_UART_0:
        uart_io_err_irq_handler();
        break;
    /*--- usb uart 1 ---*/
    case BSP_USB_UART_1:
        uart_io_err_irq_handler();
        break;
    /*--- default    ---*/
    default:
        /* no processing */
        break;
    }
}

#ifdef __DEBUG__
/*--------------------------------------------------*/
/** 
  * @brief  Failure Assert.
  * @note   for debug
  */
/*--------------------------------------------------*/
void assert_failed(char *file, int32_t line)
{
    while (1) {
     /*   __NOP(); */
    }
}
#endif

/**
 *  @}
 */ /* End of group EBIF_SRAM_Exported_functions */


/**
 *  @} 
 */ /* End of group EBIF_SRAM */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
