/** 
 *******************************************************************************
 * @file    main.c
 * @brief   HDMA HDMA_MEM_MEM Sample Application.
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2020
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include <stdio.h>
#include <string.h>
   
#include "txz_sample_def.h"
#include "bsp.h"
#include "bsp_uart_io.h"
#include "bsp_timer.h"
/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup HDMA_MEM_MEM HDMA Memory to Memory Sample Appli
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup HDMA_MEM_MEM_Private_define HDMA Memory to Memory Private Define
 *  @{
 */

/*----------------------*/
/*  DMA Setting         */
/*----------------------*/
#define CFG_DATA_SIZE         ((uint32_t)127)            /* DMA assigned data size */
#define CFG_DMA_CH            ((uint32_t)0)              /* DMA Using Channel      */
/** 
 *  @}
 */ /* End of group HDMA_MEM_MEM_Private_define */

/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup HDMA_MEM_MEM_Private_define HDMA Memory to Memory Private Define
 *  @{
 */

#define MAIN_NULL               ((void *)0) /*!< Null Pointer */

/** 
 *  @}
 */ /* End of group HDMA_MEM_MEM_Private_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup HDMA_MEM_MEM_Private_define HDMA Memory to Memory Private Define
 *  @{
 */
/*----------------------------------*/
/** 
 * @brief  Processing mode.
*/
/*----------------------------------*/
typedef enum
{
    PROC_IDLE,              /*!< Idle.                        */
    PROC_START,             /*!< Start.                       */
    PROC_STOP,              /*!< Stop.                        */
    PROC_OUTPUT_MSG,        /*!< Output first message.        */
    PROC_INPUT,             /*!< Wait to input.               */
    PROC_RUNNING,           /*!< DMA Running.                 */
    PROC_WAIT,              /*!< DMA Transfer Complete Wait.  */
    PROC_OUTPUT_ECHO,       /*!< Output Echo back data.       */
    PROC_OUTPUT_ERR,        /*!< Output input error message.  */
    PROC_END,               /*!< End.                         */
    PROC_ERROR              /*!< Error.                       */
} Processing;

typedef enum
{
    TRANS_STATE_IDLE,       /* IDLE      */
    TRANS_STATE_RUNNING,    /* Running   */
    TRANS_STATE_SUCCESS,    /* Success   */
    TRANS_STATE_FAILURE     /* Failure   */
} TransState;
/** 
 *  @}
 */ /* End of group HDMA_MEM_MEM_Private_define */



/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup HDMA_MEM_MEM_Private_typedef HDMA Memory to Memory Private Typedef
 *  @{
 */
/*----------------------------------*/
/** 
 * @brief Instance Information.
*/
/*----------------------------------*/
typedef struct
{
   t32a_t        t32a;      /*!< Driver      :T32A.              */
   hdma_t        hdma;      /*!< Driver      :HDMA.               */
   timer_t       timer;     /*!< Application :Timer (1ms timer). */
   uart_io_t     *p_uart;   /*!< Application :USB UART.          */
} instance_t;

/*----------------------------------*/
/** 
 * @brief  Input data information.
*/
/*----------------------------------*/
typedef struct
{
    char    data[(UART_IO_RECEIVE_MAX+1)];  /*!< Input data(+ for NULL 1byte). */
    uint8_t wp;                                 /*!< Write pointer.                */
} input_t;
/**
 *  @}
 */ /* End of group HDMA_MEM_MEM_Private_typedef */


/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup HDMA_MEM_MEM_Private_variables HDMA Memory to Memory Private Variables
 *  @{
 */
static instance_t instance;    /*!< Instance.               */
static Processing proc;        /*!< Processing mode.        */
static input_t    input;       /*!< Input data information. */
static TransState status;      /*!< DMA Status.             */
static uint32_t transsize;

#if defined ( __CC_ARM   )      /* RealView Compiler */
    static __align(4) uint8_t src[CFG_DATA_SIZE];
    static __align(4) uint8_t dst[CFG_DATA_SIZE];
#elif defined ( __ICCARM__ )    /*IAR Compiler */
    #pragma location="dma_work"
    #pragma data_alignment=4
    static uint8_t src[CFG_DATA_SIZE];
    static uint8_t dst[CFG_DATA_SIZE];
#endif

/**
 *  @}
 */ /* End of group HDMA_MEM_MEM_Private_variables */


/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup HDMA_MEM_MEM_Private_fuctions HDMA Memory to Memory Private Fuctions
 *  @{
 */
static void event_handler(uint32_t id, TXZ_Result result);
static void variable_initialize(void);
static TXZ_Result driver_hdma_initialize(void);
static TXZ_Result driver_hdma_ch_initialize(uint32_t size);
static TXZ_Result driver_hdma_finalize(void);
static TXZ_Result driver_initialize(void);
static TXZ_Result driver_finalize(void);
static TXZ_Result application_initialize(void);
static TXZ_Result application_finalize(void);
static void timer_interval_handler(uint32_t id);
static void clear_data(void);

/*--------------------------------------------------*/
/** 
  * @brief  DMA Evwnt Handler
  * @param  id        :User ID
  * @return Result.
  * @retval TXZ_SUCCESS :Success
  * @retval TXZ_ERROR   :Failure
  * @note   Call by HDMA Module
  */
/*--------------------------------------------------*/
static void event_handler(uint32_t id, TXZ_Result result)
{
    if (result == TXZ_SUCCESS)
    {
        status = TRANS_STATE_SUCCESS;
    }
    else
    {
        status = TRANS_STATE_FAILURE;
    }
}
/*--------------------------------------------------*/
/** 
  * @brief  Variable Initialize
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void variable_initialize(void)
{
    /* Instance. */
    instance.p_uart             = MAIN_NULL;
    instance.t32a.p_instance    = MAIN_NULL;
    instance.timer.init.p_t32a  = MAIN_NULL;
    instance.hdma.p_instance     = MAIN_NULL;
    /* Processing mode. */
    proc = PROC_START;
    /* Input data information. */
    clear_data();
}

/*--------------------------------------------------*/
/** 
  * @brief  DMA Driver Initialize.
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success
  * @retval TXZ_ERROR   :Failure
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result driver_hdma_initialize(void)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*--- Construct (Register Allocation) ---*/
    instance.hdma.p_instance = TSB_DMACA;
    /*--- Initialize ---*/
    if (hdma_init(&instance.hdma) != TXZ_SUCCESS)
    {
        result = TXZ_ERROR;
    }

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  DMA Driver Ch Setting Initialize.
  * @param  size        : TransferSize
  * @return Result.
  * @retval TXZ_SUCCESS :Success
  * @retval TXZ_ERROR   :Failure
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result driver_hdma_ch_initialize(uint32_t size)
{
    TXZ_Result result = TXZ_SUCCESS;
    uint32_t dma_ch = CFG_DMA_CH;

    /*--- Construct (Register Allocation) ---*/
    instance.hdma.p_instance = TSB_DMACA;
    /*------------------------*/
    /* Channel Initialize     */
    /*------------------------*/
    instance.hdma.ch[dma_ch].handler       = event_handler;
    instance.hdma.ch[dma_ch].id            = (uint32_t)&instance.hdma;
    instance.hdma.ch[dma_ch].cnt.dstInc    = HDMA_DST_ADDR_INC;
    instance.hdma.ch[dma_ch].cnt.dstSize   = HDMA_DST_BITWIDTH_1BYTE;
    instance.hdma.ch[dma_ch].cnt.dstBurstSize = HDMA_DST_BURSTSIZE_1BEAT;
    instance.hdma.ch[dma_ch].cnt.srcInc    = HDMA_SRC_ADDR_INC;
    instance.hdma.ch[dma_ch].cnt.srcSize   = HDMA_SRC_BITWIDTH_1BYTE;
    instance.hdma.ch[dma_ch].cnt.srcBurstSize = HDMA_SRC_BURSTSIZE_1BEAT;
    instance.hdma.ch[dma_ch].cnt.transsize = size;
    instance.hdma.ch[dma_ch].type          = HDMA_TRANS_TYPE_SINGLE;
    instance.hdma.ch[dma_ch].srcReqCh      = HDMA_SRC_REQ_CH_MEM_USE;
    instance.hdma.ch[dma_ch].dstReqCh      = HDMA_DST_REQ_CH_MEM_USE;
    if (hdma_FlowType_Set(&instance.hdma, dma_ch, HDMA_CH_FLOWCNT_MEM_MEM) != TXZ_SUCCESS)
    {
        result = TXZ_ERROR;
    }
    if (hdma_ITC_enable(&instance.hdma, dma_ch) != TXZ_SUCCESS)
    {
        result = TXZ_ERROR;
    }
    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  DMA Driver Finalize.
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success
  * @retval TXZ_ERROR   :Failure
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result driver_hdma_finalize(void)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*--- Finalize ---*/
    if (hdma_deinit(&instance.hdma) != TXZ_SUCCESS)
    {
        result = TXZ_ERROR;
    }
    /*--- Destruct (Register Release) ---*/
    /* Register Allocation */
    instance.hdma.p_instance = MAIN_NULL;

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Driver Initialize
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success
  * @retval TXZ_ERROR   :Failure
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result driver_initialize(void)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*----------------------*/
    /* GPIO                 */
    /*----------------------*/
    /* Driver initialize has done within BSP. */
    /*----------------------*/
    /* T32A                 */
    /*----------------------*/
    {
        t32a_t *p_t32a = &instance.t32a;

        /*--- Construct  ---*/
        /* Register Allocation */
        switch (bsp_get_timer_ch(BSP_TIMER_1MS))
        {
        case 0:
            p_t32a->p_instance = TSB_T32A0;
            break;
        case 1:
            p_t32a->p_instance = TSB_T32A1;
            break;
        case 2:
            p_t32a->p_instance = TSB_T32A2;
            break;
        case 3:
            p_t32a->p_instance = TSB_T32A3;
            break;
        case 4:
            p_t32a->p_instance = TSB_T32A4;
            break;
        case 5:
            p_t32a->p_instance = TSB_T32A5;
            break;
        default:
            p_t32a->p_instance = MAIN_NULL;
            break;
        }
        /*--- Initialize ---*/
        p_t32a->init_mode.mode.halt = T32A_DBG_HALT_STOP;
        p_t32a->init_mode.mode.mode = T32A_MODE_16;
        if (p_t32a->p_instance != MAIN_NULL)
        {
            if (t32a_mode_init(p_t32a) != TXZ_SUCCESS)
            {
                result = TXZ_ERROR;
            }
        }
    }
    /*----------------------*/
    /* DMA                  */
    /*----------------------*/
    /* Driver initialize will process within DMA. */
    /*----------------------*/
    /* Uart                 */
    /*----------------------*/
    /* Driver initialize will process within USB UART. */

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Driver Finalize
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success
  * @retval TXZ_ERROR   :Failure
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result driver_finalize(void)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*----------------------*/
    /* Uart                 */
    /*----------------------*/
    /* Driver finalize has done within USB UART.    */
    /*----------------------*/
    /* DMA                  */
    /*----------------------*/
    if (driver_hdma_finalize() != TXZ_SUCCESS)
    {
        result = TXZ_ERROR;
    }
    /*----------------------*/
    /* T32A                 */
    /*----------------------*/
    {
        t32a_t *p_t32a = &instance.t32a;

        /*--- Finalize ---*/
        /* no processing */
        /*--- Destruct ---*/
        /* Register Release */
        p_t32a->p_instance = MAIN_NULL;
    }
    /*----------------------*/
    /* GPIO                 */
    /*----------------------*/
    /* Driver finalize will process within BSP.     */

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Application Initialize
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success
  * @retval TXZ_ERROR   :Failure
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result application_initialize(void)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*----------------------*/
    /* Timer                */
    /*----------------------*/
    /* 1ms timer */
    {
        timer_t *p_timer = &instance.timer;

        p_timer->init.id       = (uint32_t)&instance.timer;
        p_timer->init.p_t32a   = &instance.t32a;
        p_timer->init.type     = bsp_get_timer_type(BSP_TIMER_1MS);
        p_timer->init.irq      = bsp_get_timer_nvic(BSP_TIMER_1MS);
        p_timer->init.interval = 1000;
        p_timer->init.handler  = timer_interval_handler;
        timer_initialize(p_timer);
    }
    /*----------------------*/
    /* Usb Uart             */
    /*----------------------*/
    /* Tx & Rx are same channel */
    {
       uart_io_initial_setting_t param;

        param.id         = 0;   /* User ID :any value. */
        switch (bsp_get_usb_uart_tx_ch(BSP_USB_UART_0))
        {
        case 0:
            param.p_reg  = TSB_UART0;
            break;
        case 1:
            param.p_reg  = TSB_UART1;
            break;
        case 2:
            param.p_reg  = TSB_UART2;
            break;
        default:
            param.p_reg  = MAIN_NULL;
            break;
        }
        param.boudrate   = bsp_get_usb_uart_tx_baudrate(BSP_USB_UART_0);
        param.irq.tx     = bsp_get_usb_uart_tx_nvic(BSP_USB_UART_0);
        param.irq.rx     = bsp_get_usb_uart_rx_nvic(BSP_USB_UART_0);
        param.irq.err    = bsp_get_usb_uart_err_nvic(BSP_USB_UART_0);
        if (param.p_reg != MAIN_NULL)
        {
            result  = uart_io_initialize(&param);
        }
        else
        {
            result = TXZ_ERROR;
        }
    }
    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Application Finalize
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success
  * @retval TXZ_ERROR   :Failure
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result application_finalize(void)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*----------------------*/
    /* USB UART             */
    /*----------------------*/
    if (instance.p_uart != MAIN_NULL)
    {
        uart_io_finalize();
    }
    /*----------------------*/
    /* Timer                */
    /*----------------------*/
    /* 1ms timer */
    timer_finalize(&instance.timer);

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Interval Time Handler.
  * @param  id        :User ID
  * @return -
  * @retval -
  * @note   for 1ms Timer.
  * @note   Call by Timer.
  */
/*--------------------------------------------------*/
static void timer_interval_handler(uint32_t id)
{
    if (id == (uint32_t)&instance.timer)
    {
        /* Increment Tick. Tick is used to driver's Timeout. */
        hal_inc_tick();
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  Clear input data.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void clear_data(void)
{
    uint8_t i;

    for (i=0; i<(UART_IO_RECEIVE_MAX+1); i++)
    {
        input.data[i] = 0;
    }
    input.wp = 0;
    for (i=0; i < CFG_DATA_SIZE; i++)
    {
        src[i] = 0;
    }
    for (i=0; i < CFG_DATA_SIZE; i++)
    {
        dst[i] = 0;
    }
}
/**
 *  @}
 */ /* End of group HDMA_MEM_MEM_Private_fuctions */


/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup HDMA_MEM_MEM_Exported_functions HDMA Memory to Memory Exported Functions
 *  @{
 */
void dmac_done(uint32_t ch);
void dmac_error(void);
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
int main(void)
{
    /*----------------------*/
    /* BSP initialize       */
    /*----------------------*/
    bsp_initialize();
    /*----------------------*/
    /* Initialize           */
    /*----------------------*/
    /*--- variable    ---*/
    variable_initialize();
    /*--- driver      ---*/
    if (driver_initialize() != TXZ_SUCCESS)
    {
        proc = PROC_ERROR;
    }
    /*--- application ---*/
    if (application_initialize() != TXZ_SUCCESS)
    {
        proc = PROC_ERROR;
    }
    /*----------------------*/
    /* Main Processing      */
    /*----------------------*/
    while(proc != PROC_ERROR)
    {
        switch(proc)
        {
        /*--- Start ---*/
        case PROC_START:
            {
                /* DMA  */
                status = TRANS_STATE_IDLE;
                NVIC_EnableIRQ(INTHDMAATC_IRQn);
                NVIC_EnableIRQ(INTHDMAAERR_IRQn);
                /* Timer */
                timer_start(&instance.timer);
                proc = PROC_OUTPUT_MSG;
            }
            break;
        /*--- Stop ---*/
        case PROC_STOP:
            {
                /* DMA  */
                NVIC_DisableIRQ(INTHDMAATC_IRQn);
                NVIC_DisableIRQ(INTHDMAAERR_IRQn);
                /* Timer */
                timer_stop(&instance.timer);
                proc = PROC_ERROR;
            }
            break;
        /*--- Output "First Message" ---*/
        case PROC_OUTPUT_MSG:
            {
                /* Clear data.     */
                clear_data();
                /* Output first message. */
                if (printf("Input = ") != -1 )
                {
                    proc = PROC_INPUT;
                }
                else
                {
                    proc = PROC_STOP;
                }
            }
            break;
        /*--- Wait to Input ---*/
        case PROC_INPUT:
            {
                int data;

                data = getchar();
                if ((data != -1) && (input.wp < UART_IO_RECEIVE_MAX))
                {
                    input.data[input.wp] = (char)data;
                    input.wp++;
                    switch ((char)data)
                    {
                    /*--- CR: carriage return ---*/
                    case '\r':
                        /* no process */
                        break;
                    /*--- LF: line feed ---*/
                    case '\n':
                        proc = PROC_RUNNING;
                        break;
                    /*--- default ---*/
                    default:
                        /* no process */
                        break;
                    }
                }
                else
                {
                    proc = PROC_OUTPUT_ERR;
                }
            }
            break;
        /*--- DMA Start ---*/
        case PROC_RUNNING:
            {
                uint32_t i = 0;
                uint32_t dma_ch = CFG_DMA_CH;
                
                transsize = input.wp; 
                for(i = 0; i < transsize ; i++){
                   src[i] = (uint8_t)input.data[i];
                }
                status = TRANS_STATE_RUNNING;
                {
                    /* DMA Driver Initialize & Channel Setting */
                    if (driver_hdma_initialize() != TXZ_SUCCESS)
                    {
                      proc = PROC_ERROR;
                    }else{
                      /* No Proccess */
                    }
                    if (driver_hdma_ch_initialize(transsize) != TXZ_SUCCESS)
                    {
                      proc = PROC_ERROR;
                    }else{
                      /* No Proccess */
                    }
                    /* DMA Transfer Start */
                    instance.hdma.init.srcAdd = (uint32_t)&src[0];
                    instance.hdma.init.dstAdd = (uint32_t)&dst[0];
                    if (hdma_startIt(&instance.hdma, dma_ch) != TXZ_SUCCESS)
                    {
                      proc = PROC_ERROR;
                    }else{
                      proc = PROC_WAIT;
                    }
                }
                break;
            }
        /*--- Wait DMA Transfer & Output Transfer data ---*/
        case PROC_WAIT:
            {
                if (status == TRANS_STATE_SUCCESS)
                {
                    if (src[0] != dst[0]){
                        proc = PROC_ERROR;
                    }else{
                        if(printf("transdata = %s\n", (const char*)&dst[0]) != -1)
                        {
                            proc = PROC_OUTPUT_MSG;
                        }
                        else
                        {
                            proc = PROC_STOP;
                        }
                    }
                }
            }
            break;
        /*--- Output "Input Error Message" ---*/
        case PROC_OUTPUT_ERR:
            {
                if (printf("\n Input Error !! \n") != -1 )
                {
                    proc = PROC_OUTPUT_MSG;
                }
                else
                {
                    proc = PROC_STOP;
                }
            }
            break;
        /*--- Error ---*/
        case PROC_ERROR:
            /* no processing */
            break;
        /*--- Default ---*/
        default:
            proc = PROC_ERROR;
            break;
        }
    }
    /*>>> System Error. Please Debug. */
    /*----------------------*/
    /* Finalize             */
    /*----------------------*/
    /*--- application ---*/
    if (application_finalize() != TXZ_SUCCESS)
    {
        /* no processing */
    }
    /*--- driver      ---*/
    if (driver_finalize() != TXZ_SUCCESS)
    {
        /* no processing */
    }
    /*----------------------*/
    /* BSP finalize         */
    /*----------------------*/
    bsp_finalize();
#ifdef __DEBUG__
    assert_failed(MAIN_NULL, 0);
#endif

    return (0);
}
/*--------------------------------------------------*/
/** 
  * @brief  USB UART reception IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_rx(BSPUsbUart uart)
{
    switch (uart)
    {
    /*--- usb uart 0 ---*/
    case BSP_USB_UART_0:
        uart_io_rx_irq_handler();
        break;
    /*--- usb uart 1 ---*/
    case BSP_USB_UART_1:
        uart_io_rx_irq_handler();
        break;
    /*--- default    ---*/
    default:
        /* no processing */
        break;
    }

}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART transmit IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_tx(BSPUsbUart uart)
{
    switch (uart)
    {
        /*--- usb uart 0 ---*/
    case BSP_USB_UART_0:
        uart_io_tx_irq_handler();
        break;
        /*--- usb uart 1 ---*/
    case BSP_USB_UART_1:
        uart_io_tx_irq_handler();
        break;
    /*--- default    ---*/
    default:
        /* no processing */
        break;
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART error IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_err(BSPUsbUart uart)
{
    switch (uart)
    {
        /*--- usb uart 0 ---*/
    case BSP_USB_UART_0:
        uart_io_err_irq_handler();
        break;
        /*--- usb uart 1 ---*/
    case BSP_USB_UART_1:
        uart_io_err_irq_handler();
        break;
    /*--- default    ---*/
    default:
        /* no processing */
        break;
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  Timer.
  * @param  timer :Timer.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_timer(BSPTimer timer)
{
    switch (timer)
    {
    /*--- 1ms timer 0 ---*/
    case BSP_TIMER_1MS:
        timer_irq_handler(&instance.timer);
        break;
    /*--- default    ---*/
    default:
        /* no processing */
        break;
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  HDMAC Transfer Complete IRQ.
  * @param  -
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_dmac_done(void)
{
    if(hdma_irq_handler(&instance.hdma) != TXZ_SUCCESS)
    {
        /* no process */
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  HDMAC ERROR IRQ.
  * @param  -
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_dmac_err(void)
{
    if(hdma_error_irq_handler(&instance.hdma) != TXZ_SUCCESS)
    {
        /* no process */
    }
}

#ifdef __DEBUG__
/*--------------------------------------------------*/
/** 
  * @brief  Failure Assert.
  * @note   for debug
  */
/*--------------------------------------------------*/
void assert_failed(char *file, int32_t line)
{
    while (1) {
        __NOP();
    }
}
#endif
/**
 *  @}
 */ /* End of group HDMA_MEM_MEM_Exported_functions */

/**
 *  @}
 */ /* End of group HDMA_MEM_MEM */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
