/** 
 *******************************************************************************
 * @file    main.c
 * @brief   I2C EEPROM Sample Application.
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2020
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif
/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "txz_sample_def.h"
#include "i2c_eeprom.h"
#include "bsp.h"
#include "bsp_uart_io.h"

/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup I2C_EEPROM I2C_EEPROM Sample Appli
 *  @{
 */

/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup I2C_EEPROM_Private_macro I2C_EEPROM Private Macro
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group I2C_EEPROM_Private_macro */


/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup I2C_EEPROM_Private_define I2C_EEPROM Private Define
 *  @{
 */
/* no define */

/** 
 *  @}
 */ /* End of group I2C_EEPROM_Private_define */

/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup I2C_EEPROM_Private_define I2C_EEPROM Private Define
 *  @{
 */

#define MAIN_NULL       ((void *)0)         /*!< Null Pointer.            */
#define SAMPLE_ADDRESS  (0x0000)            /*!< Access location address. */
/*----------------------*/
/*  UART Setting        */
/*----------------------*/
/** 
 *  @defgroup UART_Setting UART Setting
 *  @{
 */
#define USER_ID_UART    ((uint32_t)1)   /*!< User ID :USB UART (Any value) */
/** 
 *  @}
 */ /* End of group UART_Setting */

/** 
 *  @}
 */ /* End of group I2C_EEPROM_Private_define */

/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup I2C_EEPROM_Private_define I2C_EEPROM Private Define
 *  @{
 */

/*----------------------------------*/
/** 
 * @enum   Processing
 * @brief  Processing mode.
*/
/*----------------------------------*/
typedef enum
{
    PROC_IDLE,              /*!< Idle.                        */
    PROC_OUTPUT_MSG,        /*!< Output first message.        */
    PROC_INPUT,             /*!< Wait to input.               */
    PROC_COMMAND_PARSE,     /*!< Output Echo back data.       */
    PROC_OUTPUT_ERR,        /*!< Output input error message.  */
    PROC_ERROR              /*!< Error.                       */
} Processing;

/** 
 *  @}
 */ /* End of group I2C_EEPROM_Private_define */

/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup I2C_EEPROM_Private_typedef I2C_EEPROM Private Typedef
 *  @{
 */
/*----------------------------------*/
/** 
 * @struct input_t
 * @brief  Input data information.
*/
/*----------------------------------*/
typedef struct
{
    char    data[(UART_IO_RECEIVE_MAX+1)];  /*!< Input data(+ for NULL 1byte). */
    uint8_t wp;                             /*!< Write pointer.                */
} input_t;

/** 
 *  @}
 */ /* End of group I2C_EEPROM_Private_typedef */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup I2C_EEPROM_Private_variables I2C_EEPROM Private Variables
 *  @{
 */
static Processing proc;         /*!< Processing mode.        */
static input_t input;           /*!< Input data information. */
static uint8_t write_data[10];  /*!< Write data information. */
static uint8_t read_data[10];   /*!< Read data information. */
/**
 *  @}
 */ /* End of group I2C_EEPROM_Private_variables */


/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup I2C_EEPROM_Private_functions I2C_EEPROM Private Fuctions
 *  @{
 */
static void clear_data(void);
static TXZ_Result application_initialize(void);

/*--------------------------------------------------*/
/** 
  * @brief  Clear input data.
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void clear_data(void)
{
    uint8_t i;

    for (i = 0; i < (UART_IO_RECEIVE_MAX+1); i++)
    {
        input.data[i] = 0;
    }
    input.wp = 0;
    
    for (i = 0; i < 10; i++)
    {
        read_data[i] = 0;
        write_data[i] = 0;
    }
}
/*--------------------------------------------------*/
/** 
  * @brief  Application Initialize.
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result application_initialize(void)
{
   TXZ_Result result = TXZ_SUCCESS;

    /*----------------------*/
    /* Usb Uart             */
    /*----------------------*/
    /* Tx & Rx use same UART channel. */
    {
        uart_io_initial_setting_t param;

        param.id         = USER_ID_UART;
        switch (bsp_get_usb_uart_tx_ch(BSP_USB_UART_0))
        {
        case 0:
            param.p_reg  = TSB_UART0;
            break;
        case 1:
            param.p_reg  = TSB_UART1;
            break;
        case 2:
            param.p_reg  = TSB_UART2;
            break;
        default:
            param.p_reg  = MAIN_NULL;
            break;
        }
        param.boudrate   = bsp_get_usb_uart_tx_baudrate(BSP_USB_UART_0);
        param.irq.tx     = bsp_get_usb_uart_tx_nvic(BSP_USB_UART_0);
        param.irq.rx     = bsp_get_usb_uart_rx_nvic(BSP_USB_UART_0);
        param.irq.err    = bsp_get_usb_uart_err_nvic(BSP_USB_UART_0);
        if (param.p_reg != MAIN_NULL)
        {
            result  = uart_io_initialize(&param);
        }
        else
        {
            result = TXZ_ERROR;
        }
    }
    return (result);
}
/**
 *  @}
 */ /* End of group I2C_EEPROM_Private_functions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup I2C_EEPROM_Exported_functions I2C_EEPROM Exported Functions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
int main(void)
{
    int num;
    int len = 0;
    int cnum = 0;
    /*----------------------*/
    /* Set Processing Mode  */
    /*----------------------*/
    proc = PROC_OUTPUT_MSG;
    /*----------------------*/
    /* Initialize           */
    /*----------------------*/
    /*----------------------*/
    /* BSP initialize       */
    /*----------------------*/
    bsp_initialize();
    /* Application initialization */
    application_initialize();  
    EEPROM_i2c_init();
    
    /*----------------------*/
    /* Main Processing      */
    /*----------------------*/
    while(1)
    {
        switch(proc)
        {
        /*--- Output "First Message" ---*/
        case PROC_OUTPUT_MSG:
            {
                /* Clear input data.     */
                clear_data();
                /* Output first message. */
                if (printf("command > ") != -1)
                {
                    proc = PROC_INPUT;
                }
                else
                {
                    proc = PROC_ERROR;
                }
            }
            break;
        /*--- Wait to Input ---*/
        case PROC_INPUT:
            {   
                int data;
                
                data = getchar();
                if ((data != -1) && (input.wp < UART_IO_RECEIVE_MAX))
                {
                    input.data[input.wp] = (char)data;
                    input.wp++;
                    switch ((char)data)
                    {
                    /*--- C.dataR: carriage return ---*/
                    case '\r':
                        /* no process */
                        break;
                    /*--- LF: line feed ---*/
                    case '\n':
                        len = input.wp;
                        proc = PROC_COMMAND_PARSE;
                        break;
                    /*--- default ---*/
                    default:
                        /* no process */
                        break;
                    }
                }
                else
                {
                    proc = PROC_OUTPUT_ERR;
                }
            }
            break;
        /*--- Output "Echo Back Data" ---*/
        case PROC_COMMAND_PARSE:
            {
                int i;
                int32_t res;

                if ((strncmp((const char *)&input.data[0], "write", 5)) == 0)
                {
                    /* WRITE Command */
                    if (len >= 9)
                    {
                        printf("write data > ");
                        num = (len - (6 + 2));
                        for (i = 0; (i < num) && (i < 8); i++)
                        {
                            write_data[i] = (uint8_t)(input.data[6 + i]);
                        }
                        cnum = i;
                        res = EEPROM_i2c_WritePage((int32_t)SAMPLE_ADDRESS, write_data, cnum);
                        if (res > 0)
                        {
                            printf("%s\n", (const char *)&write_data[0]);
                            proc = PROC_OUTPUT_MSG;
                        }
                        else
                        {
                            printf("\n write error !! \n");
                            proc = PROC_ERROR;
                        }
                        printf("\n");
                    }
                    else
                    {
                        proc = PROC_OUTPUT_ERR;
                    }
                    break;
                }
                else if ((strncmp((const char *)&input.data[0], "read", 4)) == 0)
                {
                    /* READ Command */
                    printf("read data > ");
                    res = EEPROM_i2c_ReadData((int32_t)SAMPLE_ADDRESS, read_data, cnum);
                    if (res > 0)
                    {
                        for (i = res; i < 10; i++)
                        {
                             read_data[i] = 0;
                        }
                        printf("%s\n", (const char*)&read_data[0]);
                        proc = PROC_OUTPUT_MSG;
                    }
                    else
                    {
                        printf("\n read error !! \n");
                        proc = PROC_ERROR;
                    }
                    printf("\n");
                }
                else if ((strncmp((const char *)&input.data[0], "bw", 2)) == 0)
                {
                    /* BYTE WRITE Command */
                    if (len >= 9)
                    {
                        char str[5] = {0};
                        int32_t address;

                        printf("byte write data > ");
                        str[0]= input.data[3];
                        str[1]= input.data[4];
                        str[2]= input.data[5];
                        str[3]= input.data[6];
                        address = strtol(str, 0, 16);
                        write_data[0] = (uint8_t)(input.data[8]);
                        write_data[1] = 0;
                        if (EEPROM_i2c_WriteByte(address, write_data[0]) == TXZ_SUCCESS)
                        {
                            printf("%s\n", (const char *)&write_data[0]);
                            proc = PROC_OUTPUT_MSG;
                        }
                        else
                        {
                            printf("\n write error !! \n");
                            proc = PROC_ERROR;
                        }
                        printf("\n");
                    }
                    else
                    {
                        proc = PROC_OUTPUT_ERR;
                    }
                }
                else if ((strncmp((const char *)&input.data[0], "br", 2)) == 0)
                {
                    /* BYTE READ Command */
                    if (len >= 7)
                    {
                        char str[5] = {0};
                        int32_t address;

                        printf("byte read data > ");
                        str[0]= input.data[3];
                        str[1]= input.data[4];
                        str[2]= input.data[5];
                        str[3]= input.data[6];
                        address = strtol(str, 0, 16);
                        read_data[0] = 0;
                        read_data[1] = 0;
                        if (EEPROM_i2c_ReadByte(address, &read_data[0]) == TXZ_SUCCESS)
                        {
                            printf("%s\n", (const char*)&read_data[0]);
                            proc = PROC_OUTPUT_MSG;         
                        }
                        else
                        {
                            printf("\n read error !! \n");
                            proc = PROC_ERROR;
                        }
                        printf("\n");
                    }
                    else
                    {
                        proc = PROC_OUTPUT_ERR;
                    }
                }
                else
                {
                    proc = PROC_OUTPUT_ERR;
                }
            }
            break;
                
        /*--- Output "Input Error Message" ---*/
        case PROC_OUTPUT_ERR:
            printf("? \n");
            proc = PROC_OUTPUT_MSG;
            break;
        /*--- Error ---*/
        case PROC_ERROR:
            /* System Error. Please Debug. */
#ifdef __DEBUG__
            assert_failed(MAIN_NULL, 0);
#endif
            proc = PROC_OUTPUT_MSG;
            break;
        /*--- Default ---*/
        default:
            proc = PROC_ERROR;
            break;
        }
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART reception IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_rx(BSPUsbUart uart)
{
    switch (uart)
    {
    /*--- usb uart 0 ---*/
    case BSP_USB_UART_0:
        uart_io_rx_irq_handler();
        break;
    /*--- usb uart 1 ---*/
    case BSP_USB_UART_1:
        uart_io_rx_irq_handler();
        break;
    /*--- default    ---*/
    default:
        /* no processing */
        break;
    }

}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART transmit IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_tx(BSPUsbUart uart)
{
    switch (uart)
    {
    /*--- usb uart 0 ---*/
    case BSP_USB_UART_0:
        uart_io_tx_irq_handler();
        break;
    /*--- usb uart 1 ---*/
    case BSP_USB_UART_1:
        uart_io_tx_irq_handler();
        break;
    /*--- default    ---*/
    default:
        /* no processing */
        break;
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART error IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_err(BSPUsbUart uart)
{
    switch (uart)
    {
    /*--- usb uart 0 ---*/
    case BSP_USB_UART_0:
        uart_io_err_irq_handler();
        break;
    /*--- usb uart 1 ---*/
    case BSP_USB_UART_1:
        uart_io_err_irq_handler();
        break;
    /*--- default    ---*/
    default:
        /* no processing */
        break;
    }
}

#ifdef __DEBUG__
/*--------------------------------------------------*/
/** 
  * @brief  Failure Assert.
  * @note   for debug
  */
/*--------------------------------------------------*/
void assert_failed(char *file, int32_t line)
{
    while (1) {
     /*   __NOP(); */
    }
}
#endif

/**
 *  @}
 */ /* End of group I2C_EEPROM_Exported_functions */


/**
 *  @} 
 */ /* End of group I2C_EEPROM */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
