/** 
 *******************************************************************************
 * @file    main.c
 * @brief   I2C MASTER and SLAVE Sample Application.
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2020
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif
/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "txz_sample_def.h"
#include "bsp.h"
#include "bsp_uart_io.h"
#include "bsp_i2c.h"

/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup I2C_MASTER_SLAVE I2C_MASTER_SLAVE Sample Appli
 *  @{
 */

/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup I2C_MASTER_SLAVE_Private_macro I2C_MASTER_SLAVE Private Macro
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group I2C_MASTER_SLAVE_Private_macro */

/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup I2C_MASTER_SLAVE_Private_define I2C_MASTER_SLAVE Private Define
 *  @{
 */

/** 
 *  @}
 */ /* End of group I2C_MASTER_SLAVE_Private_define */

/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup I2C_MASTER_SLAVE_Private_define I2C_MASTER_SLAVE Private Define
 *  @{
 */

#define MAIN_NULL       ((void *)0)         /*!< Null Pointer.  */
#define SELF_ADDR       ((uint8_t)0x60)     /*!< Self address for test.        */
#define SLAVE_ADDR      ((uint8_t)0x60)     /*!< Slave address for test.       */
#define PARAM_A_SIZE_MAX (0x04)             /*!< Parameter A Size Max          */
#define PARAM_B_SIZE_MAX (0x02)             /*!< Parameter B Size Max          */
#define PARAM_B_DATA_MAX (0x04)             /*!< Parameter B Size Max          */
#define INIT_RNUM       (2)                 /*!< Init RNUM                     */
#define DATA_LENGTH     (16)                /*!< Length of base data.          */
#define SLAVE_CONTINUE  (10)                /*!< Number of keeping slave mode. */
#define TEST_TIMEOUT    (16000000)          /*!< for verification.             */
#define ASCII_0         (0x30)              /*!< ASCII code 0*/
#define ASCII_9         (0x39)              /*!< ASCII code 9*/
#define ASCII_A         (0x41)              /*!< ASCII code A*/
#define ASCII_F         (0x46)              /*!< ASCII code F*/
#define ASCII_a         (0x61)              /*!< ASCII code a*/
#define ASCII_f         (0x66)              /*!< ASCII code f*/
#define WAITTIME        (30)                /*!< Send wait time*/

#define CMD_MASTER_WRITE  (0x00)            /*!< Master writing                */
#define CMD_MASTER_READ   (0x01)            /*!< Master reading                */
/*----------------------*/
/*  UART Setting        */
/*----------------------*/
/** 
 *  @defgroup UART_Setting UART Setting
 *  @{
 */
#define USER_ID_UART    ((uint32_t)1)   /*!< User ID :USB UART (Any value) */
/** 
 *  @}
 */ /* End of group UART_Setting */

/** 
 *  @}
 */ /* End of group I2C_MASTER_SLAVE_Private_define */

/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup I2C_MASTER_SLAVE_Private_define I2C_MASTER_SLAVE Private Define
 *  @{
 */

/*----------------------------------*/
/** 
 * @brief  i2c mode.
*/
/*----------------------------------*/
typedef enum
{
    MODE_INIT,
    MODE_MASTER,
    MODE_SLAVE
} Mode;

/*----------------------------------*/
/** 
 * @enum   Processing
 * @brief  Processing mode.
*/
/*----------------------------------*/
typedef enum
{
    PROC_IDLE,              /*!< Idle.                        */
    PROC_MASTER_START,      /*!< Master start                 */
    PROC_COMMAND_MSG,       /*!< Output first message.        */
    PROC_INPUT,             /*!< Wait to input.               */
    PROC_COMMAND_PARSE,     /*!< Output Echo back data.       */
    PROC_INPUT_ERR,         /*!< Output input error message.  */
    PROC_COMMAND_END,       /*!< Command end.                 */
    PROC_MASTER_WRITE,      /*!< Master write.                */
    PROC_MASTER_WRITE_READ, /*!< Master write and read.       */
    PROC_MASTER_END,        /*!< Master end.                  */
    PROC_SLAVE_START,       /*!< Slave start.                 */
    PROC_SLAVE_RECEIVE,     /*!< Slave receive.               */
    PROC_SLAVE_END,         /*!< Slave end.                   */
    PROC_ERROR              /*!< Error.                       */
} Processing;

/** 
 *  @}
 */ /* End of group I2C_MASTER_SLAVE_Private_define */

/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup I2C_MASTER_SLAVE_Private_typedef I2C_MASTER_SLAVE Private Typedef
 *  @{
 */
/*----------------------------------*/
/** 
 * @struct input_t
 * @brief  Input data information.
*/
/*----------------------------------*/
typedef struct
{
    char    data[(UART_IO_RECEIVE_MAX+1)];  /*!< Input data(+ for NULL 1byte). */
    uint8_t wp;                             /*!< Write pointer.                */
} input_t;
/** 
 *  @}
 */ /* End of group I2C_MASTER_SLAVE_Private_typedef */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup I2C_MASTER_SLAVE_Private_variables I2C_MASTER_SLAVE Private Variables
 *  @{
 */

static Processing proc;                         /*!< Processing mode.         */
static input_t  input;                          /*!< Input data information.  */

static i2c_t obj;                               /*!< I2C driver instance.     */
static uint32_t self_address;                   /*!< I2C Self Address.        */
static uint32_t slave_address;                  /*!< I2C Slave Address.       */
static volatile uint32_t transfer_event;        /*!< I2C Transfer Event       */
static volatile Mode      mode;                 /*!< I2C mode.                */

static int32_t frequency;                       /*!< Frequency.               */
static int32_t data_form1;                      /*!< Length of 1st Data form. */
static int32_t data_form2;                      /*!< Length of 2nd Data form. */
static int32_t master_blocking;                 /*!< Selection of the api.    */
static int32_t slave_blocking;                  /*!< Selection of the api.    */
static int32_t slave_continue;                  /*!< Counts of the slave.     */
static int32_t timeout;                         /*!< Timeout work.            */

static uint32_t master_rx_length;               /*!< Length of Master Rx Data. */
static uint8_t  master_rx_data[DATA_LENGTH];    /*!< Rx Data of Master.        */
static uint32_t master_tx_length;               /*!< Length of Master Tx Data. */
static uint8_t  master_tx_data[DATA_LENGTH];    /*!< Tx Data of Master.        */

static uint32_t slave_rx_length;                /*!< Length of Slave Rx Data.  */
static uint8_t  slave_rx_data[DATA_LENGTH];     /*!< Rx Data of Slave.         */
static uint32_t slave_tx_length;                /*!< Length of Slave Tx Data.  */
static uint8_t  slave_tx_data[DATA_LENGTH];     /*!< Tx Data of Slave.         */

static uint8_t  MEM[DATA_LENGTH] ;              /*!< Memory */
/**
 *  @}
 */ /* End of group I2C_MASTER_SLAVE_Private_variables */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup I2C_MASTER_SLAVE_Private_functions I2C_MASTER_SLAVE Private Functions
 *  @{
 */
static TXZ_Result application_initialize(void);
static void configuration(void);
static void clear_input_data(void);
static void clear_master_rx_data(void);
static void clear_master_tx_data(void);
static void clear_slave_rx_data(void);
static void clear_slave_tx_data(void);
static uint8_t get_param_hex_byte(char *p_src);
static void get_param_hex_array(uint8_t *p_dst, char *p_src, int len);
static void ts_i2c_initialize(void);
static void ts_i2c_set_master_mode(void);
static void ts_i2c_set_slave_mode(void);
static void ts_i2c_master_blocking_write_read(void);
static void ts_i2c_master_blocking_write(void);
static void ts_i2c_master_non_blocking_write_read(void);
static void ts_i2c_master_non_blocking_write(void);
static void ts_i2c_slave_blocking_receive(void);
static void ts_i2c_slave_non_blocking_receive(void);
static void ts_i2c_slave_abort_check(void);

/*--------------------------------------------------*/
/** 
  * @brief  Application Initialize.
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result application_initialize(void)
{
   TXZ_Result result = TXZ_SUCCESS;

    /*----------------------*/
    /* Usb Uart             */
    /*----------------------*/
    /* Tx & Rx use same UART channel. */
    {
        uart_io_initial_setting_t param;

        param.id         = USER_ID_UART;
        switch (bsp_get_usb_uart_tx_ch(BSP_USB_UART_0))
        {
        case 0:
            param.p_reg  = TSB_UART0;
            break;
        case 1:
            param.p_reg  = TSB_UART1;
            break;
        case 2:
            param.p_reg  = TSB_UART2;
            break;
        default:
            param.p_reg  = MAIN_NULL;
            break;
        }
        param.boudrate   = bsp_get_usb_uart_tx_baudrate(BSP_USB_UART_0);
        param.irq.tx     = bsp_get_usb_uart_tx_nvic(BSP_USB_UART_0);
        param.irq.rx     = bsp_get_usb_uart_rx_nvic(BSP_USB_UART_0);
        param.irq.err    = bsp_get_usb_uart_err_nvic(BSP_USB_UART_0);
        if (param.p_reg != MAIN_NULL)
        {
            result  = uart_io_initialize(&param);
        }
        else
        {
            result = TXZ_ERROR;
        }
    }
    return (result);
}
/*--------------------------------------------------*/
/** 
  * @brief  Configuration
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void configuration(void)
{
    mode            = MODE_INIT;

    self_address    = (uint32_t)SELF_ADDR;
    slave_address   = (uint32_t)SLAVE_ADDR;

    frequency       = 400000;
    data_form1      = 2;
    data_form2      = 2;
    slave_continue  = SLAVE_CONTINUE;
    master_blocking = 0;
    slave_blocking  = 0;
}

/*--------------------------------------------------*/
/** 
  * @brief  Clear input data
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void clear_input_data(void)
{
    uint8_t i;

    for (i = 0; i < (UART_IO_RECEIVE_MAX+1); i++)
    {
        input.data[i] = 0;
    }
    input.wp = 0;
}

/*--------------------------------------------------*/
/** 
  * @brief  clear_master_rx_data
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void clear_master_rx_data(void)
{
    int i;

    for (i = 0; i < DATA_LENGTH; i++)
    {
        master_rx_data[i] = 0;
    }
    master_rx_length = 0;
}

/*--------------------------------------------------*/
/** 
  * @brief  clear_master_tx_data
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void clear_master_tx_data(void)
{
    int i;

    for (i = 0; i < DATA_LENGTH; i++)
    {
        master_tx_data[i] = i;
    }
    master_tx_length = 0;
}

/*--------------------------------------------------*/
/** 
  * @brief  clear_slave_rx_data
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void clear_slave_rx_data(void)
{
    int i;

    for (i = 0; i < DATA_LENGTH; i++)
    {
        slave_rx_data[i] = 0;
    }
    slave_rx_length = 0;
}

/*--------------------------------------------------*/
/** 
  * @brief  clear_slave_tx_data
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void clear_slave_tx_data(void)
{
    int i;

    for (i = 0; i < DATA_LENGTH; i++)
    {
        slave_tx_data[i] = (i | 0x80);
    }
    slave_tx_length = 0;
}

/*--------------------------------------------------*/
/** 
  * @brief  Getting parameter from input data
  * @param  p_src       :Input source address.
  * @retval 1byte data
  * @note   -
  */
/*--------------------------------------------------*/
static uint8_t get_param_hex_byte(char *p_src)
{
    uint32_t data = 0;
    char str[2+1] = {'\0'};

    str[0]= p_src[0];
    str[1]= p_src[1];
    data = strtol(str, 0, 16);
    data &= 0x000000FF;
    return ((uint8_t)data);
}

/*--------------------------------------------------*/
/** 
  * @brief  Getting parameter from input data
  * @param  p_dst       :Destination address.
  * @param  p_src       :Source address.
  * @param  len         :Source length.
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void get_param_hex_array(uint8_t *p_dst, char *p_src, int len)
{
    int i;

    for (i = 0; (((i * 2) < (len - 1)) && (i < DATA_LENGTH)); i++)
    {
        p_dst[i] = get_param_hex_byte(p_src + (i * 2));
    }
}
/*--------------------------------------------------*/
/** 
  * @brief  Parameter check 
  * @param  p_src       :Source address.
  * @param  len         :Source length.
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result check_param_hex_array(char *p_src, int len)
{
    int i;
    char c;
    TXZ_Result res = TXZ_SUCCESS ;

    for (i = 0; ((i < len - 1) && (i < DATA_LENGTH)); i++)
    {
        c = p_src[i] ;
        if(((c>=ASCII_0)&&(c<=ASCII_9))||((c>=ASCII_A)&&(c<=ASCII_F))||((c>=ASCII_a)&&(c<=ASCII_f)))
    {
    }
    else
    {
        res = TXZ_ERROR ;
    }
    }
    return res ;
}
/*--------------------------------------------------*/
/**
  * @brief     Initializing of I2C Driver
  * @param     -
  * @retval    -
  */
/*--------------------------------------------------*/
static void ts_i2c_initialize(void)
{
    mode = MODE_MASTER;

    /* Initializing of I2C */
    if (i2c_init(&obj, I2C_CFG_PORT_SDA, I2C_CFG_PORT_SCL) == TXZ_SUCCESS)
    {
        if (frequency != 100000)
        {
            /* Update frequency */
            if (i2c_frequency(&obj, frequency) != TXZ_SUCCESS)
            {
                printf("i2c frequency(%d) error!!\r\n", frequency);
            }
        }
    }
    else
    {
        /* fatal error */
        printf("i2c init error!!\r\n");
    }
}

/*--------------------------------------------------*/
/**
  * @brief     Changing to Master Mode
  * @param     -
  * @retval    -
  */
/*--------------------------------------------------*/
static void ts_i2c_set_master_mode(void)
{
    i2c_slave_mode(&obj, 0);

    mode = MODE_MASTER;
}

/*--------------------------------------------------*/
/**
  * @brief     Changing to Slave Mode
  * @param     -
  * @retval    -
  */
/*--------------------------------------------------*/
static void ts_i2c_set_slave_mode(void)
{
    i2c_slave_address(&obj, self_address);
    i2c_slave_mode(&obj, 1);

    mode = MODE_SLAVE;
}

/*--------------------------------------------------*/
/**
  * @brief     Writing and Reading by Master Blocking
  * @param     -
  * @retval    -
  */
/*--------------------------------------------------*/
static void ts_i2c_master_blocking_write_read(void)
{
    int32_t res;

    master_tx_length = 0;
    master_rx_length = 0;

    if (data_form1 != 0)
    {
        /* write with no stop condition */
        res = i2c_write(&obj, (int32_t)slave_address, master_tx_data, data_form1, 0);
        if (res > 0)
        {
            master_tx_length = res;
        }
        else if (res == I2C_ERROR_NO_SLAVE)
        {
            printf("NO_SLAVE!!\r\n");
        }
        else
        {
            printf("write error!!\r\n");
        }
        if (master_tx_length < data_form1)
        {
            /* early nack? */
            i2c_stop(&obj);
            return;
        }
    }
    if (data_form2 != 0)
    {
        /* read with stop condition */
        res = i2c_read(&obj, (int32_t)slave_address, master_rx_data, data_form2, 1);
        if (res > 0)
        {
            master_rx_length = res;
        }
        else if (res == I2C_ERROR_NO_SLAVE)
        {
            printf("NO_SLAVE!!\r\n");
        }
        else if ((res == 0) && (data_form2 == 0))
        {
            /* no request */
        }
        else
        {
            printf("read error!!\r\n");
        }
     }
}

/*--------------------------------------------------*/
/**
  * @brief     Writing by Master Blocking
  * @param     -
  * @retval    -
  */
/*--------------------------------------------------*/
static void ts_i2c_master_blocking_write(void)
{
    int32_t res;

    master_tx_length = 0;
    master_rx_length = 0;

    /* write with stop condition */
    res = i2c_write(&obj, (int32_t)slave_address, master_tx_data, (data_form1 + data_form2), 1);
    if (res < 0)
    {
        if (res == I2C_ERROR_NO_SLAVE)
        {
            printf("NO_SLAVE!!\r\n");
        }
        else
        {
            printf("write error!!\r\n");
        }
    }
    else
    {
        master_tx_length = res;
    }
}

/*--------------------------------------------------*/
/**
  * @brief     Writing and Reading by Master Non-Blocking
  * @param     -
  * @retval    -
  */
/*--------------------------------------------------*/
static void ts_i2c_master_non_blocking_write_read(void)
{
    master_tx_length = 0;
    master_rx_length = 0;

    transfer_event = 0;
    if (i2c_transfer_asynch(&obj, 
                            master_tx_data, 
                            2,
                            I2C_NULL,
                            0,
                            (int32_t)slave_address, 
                            1
                            ) != TXZ_SUCCESS)
    {
        printf("read error!!\r\n");
    }
    else
    {
        timeout = TEST_TIMEOUT;
        while (1)
        {
            volatile uint32_t event = transfer_event;   /* asynch information */

            if ((event & I2C_EVENT_ALL) != 0)
            {
                /* event occured */
                if (event & I2C_EVENT_TRANSFER_COMPLETE)
                {
                    /* complete */
                    master_tx_length = obj.tx_buff.pos;
                    master_rx_length = obj.rx_buff.pos;
                }
                else if (event & I2C_EVENT_TRANSFER_EARLY_NACK)
                {
                    /* early complete */
                    i2c_abort_asynch(&obj);
                    master_tx_length = obj.tx_buff.pos;
                    master_rx_length = obj.rx_buff.pos;
                    printf("EARLY_NACK!!\r\n");
                }
                else if (event & I2C_EVENT_ERROR_NO_SLAVE)
                {
                    /* no slave */
                    i2c_abort_asynch(&obj);
                    printf("NO_SLAVE!!\r\n");
                }
                else  /* if (event & I2C_EVENT_ERROR) */
                {
                    /* other error */
                    i2c_abort_asynch(&obj);
                    printf("ERROR!!\r\n");
                }
                break;
            }
            else
            {
                /* no event */
                if ((timeout--) == 0)
                {
                    i2c_abort_asynch(&obj);
                    printf("timeout error!!\r\n");
                    break;
                }
            }
        }
        while (i2c_active(&obj))
        {
            /* checking bus release */
        }
    }

    wait(WAITTIME);

    transfer_event = 0;
    if (i2c_transfer_asynch(&obj, 
                            I2C_NULL,
                            0,
                            master_rx_data,
                            data_form2,
                            (int32_t)slave_address, 
                            1
                            ) != TXZ_SUCCESS)
    {
        printf("Reading Error!!\n");
    }
    else
    {
        timeout = TEST_TIMEOUT;
        while (1)
        {
            volatile uint32_t event = transfer_event;   /* asynch information */

            if ((event & I2C_EVENT_ALL) != 0)
            {
                /* event occured */
                if (event & I2C_EVENT_TRANSFER_COMPLETE)
                {
                    /* complete */
                    master_tx_length = obj.tx_buff.pos;
                    master_rx_length = obj.rx_buff.pos;
                }
                else if (event & I2C_EVENT_TRANSFER_EARLY_NACK)
                {
                    /* early complete */
                    i2c_abort_asynch(&obj);
                    master_tx_length = obj.tx_buff.pos;
                    master_rx_length = obj.rx_buff.pos;
                    printf("EARLY NACK!!\n");
                }
                else if (event & I2C_EVENT_ERROR_NO_SLAVE)
                {
                    /* no slave */
                    i2c_abort_asynch(&obj);
                    printf("NO SLAVE!!\n");
                }
                else  /* if (event & I2C_EVENT_ERROR) */
                {
                    /* other error */
                    i2c_abort_asynch(&obj);
                    printf("ERROR!!\n");
                }
                break;
            }
            else
            {
                /* no event */
                if ((timeout--) == 0)
                {
                    i2c_abort_asynch(&obj);
                    printf("time error!!\n");
                    break;
                }
            }
        }
        while (i2c_active(&obj))
        {
            /* checking bus release */
        }
    }




}

/*--------------------------------------------------*/
/**
  * @brief     Writing by Master Non-Blocking
  * @param     -
  * @retval    -
  */
/*--------------------------------------------------*/
static void ts_i2c_master_non_blocking_write(void)
{
    master_tx_length = 0;
    master_rx_length = 0;
    transfer_event = 0;
    if (i2c_transfer_asynch(&obj, 
                            master_tx_data, 
                            2,
                            I2C_NULL, 
                            0, 
                            (int32_t)slave_address, 
                            1
                            ) != TXZ_SUCCESS)
    {
        master_tx_length = 0;
        printf("Writing Error!!\n");
    }
    else
    {
        timeout = TEST_TIMEOUT;
        while (1)
        {
            volatile uint32_t event = transfer_event;    /* asynch information. */

            if ((event & I2C_EVENT_ALL) != 0)
            {
                /* event occured */
                if (event & I2C_EVENT_TRANSFER_COMPLETE)
                {
                    /* complete */
                    master_tx_length = obj.tx_buff.pos;
                    master_rx_length = obj.rx_buff.pos;
                }
                else if (event & I2C_EVENT_TRANSFER_EARLY_NACK)
                {
                    /* early complete */
                    i2c_abort_asynch(&obj);
                    master_tx_length = obj.tx_buff.pos;
                    master_rx_length = obj.rx_buff.pos;
                    printf("EARLY NACK!!\n");
                }
                else if (event & I2C_EVENT_ERROR_NO_SLAVE)
                {
                    /* no slave */
                    i2c_abort_asynch(&obj);
                    printf("NO SLAVE!!\n");
                }
                else  /* if (event & I2C_EVENT_ERROR) */
                {
                    /* other error */
                    i2c_abort_asynch(&obj);
                    printf("ERROR!!\n");
                }
                break;
            }
            else
            {
                /* no event */
                if ((timeout--) == 0)
                {
                    i2c_abort_asynch(&obj);
                    printf("time error!!\n");
                    break;
                }
            }
        }
        while (i2c_active(&obj))
        {
            /* checking bus release */
        }
    }

    wait(WAITTIME);

    transfer_event = 0;
    if (i2c_transfer_asynch(&obj, 
                            &master_tx_data[2], 
                            master_tx_data[1],
                            I2C_NULL, 
                            0, 
                            (int32_t)slave_address, 
                            1
                            ) != TXZ_SUCCESS)
    {
        master_tx_length = 0;
        printf("Writing Error!!\n");
    }
    else
    {
        timeout = TEST_TIMEOUT;
        while (1)
        {
            volatile uint32_t event = transfer_event;    /* asynch information. */

            if ((event & I2C_EVENT_ALL) != 0)
            {
                /* event occured */
                if (event & I2C_EVENT_TRANSFER_COMPLETE)
                {
                    /* complete */
                    master_tx_length = obj.tx_buff.pos;
                    master_rx_length = obj.rx_buff.pos;
                }
                else if (event & I2C_EVENT_TRANSFER_EARLY_NACK)
                {
                    /* early complete */
                    i2c_abort_asynch(&obj);
                    master_tx_length = obj.tx_buff.pos;
                    master_rx_length = obj.rx_buff.pos;
                    printf("EARLY_NACK!!\r\n");
                }
                else if (event & I2C_EVENT_ERROR_NO_SLAVE)
                {
                    /* no slave */
                    i2c_abort_asynch(&obj);
                    printf("NO_SLAVE!!\r\n");
                }
                else  /* if (event & I2C_EVENT_ERROR) */
                {
                    /* other error */
                    i2c_abort_asynch(&obj);
                    printf("ERROR!!\r\n");
                }
                break;
            }
            else
            {
                /* no event */
                if ((timeout--) == 0)
                {
                    i2c_abort_asynch(&obj);
                    printf("timeout error!!\r\n");
                    break;
                }
            }
        }
        while (i2c_active(&obj))
        {
            /* checking bus release */
        }
    }
}

/*--------------------------------------------------*/
/**
  * @brief     Receiving by Slave blocking
  * @param     -
  * @retval    -
  */
/*--------------------------------------------------*/
static void ts_i2c_slave_blocking_receive(void)
{
    int32_t res;

    slave_rx_length = 0;
    slave_tx_length = 0;

    while (1)
    {
        /* slave receive */
        res = i2c_slave_receive(&obj);
        if (res == I2C_READ_ADDRESSED)
        {
            /* read */
            res = i2c_slave_read(&obj, &slave_rx_data[slave_rx_length], (data_form1 + data_form2));
            if (res > 0)
            {
                slave_rx_length += res;
            }
        }
        else if (res == I2C_WRITE_ADDRESSED)
        {
            /* write */
            res = i2c_slave_write(&obj, &slave_tx_data[slave_tx_length], data_form2);
            if (res > 0)
            {
                slave_tx_length += res;
            }
            break;
        }
        else
        {
            if ((slave_rx_length + slave_tx_length) >= (data_form1 + data_form2))
            {
                break;
            }
        }
    }
}

/*--------------------------------------------------*/
/**
  * @brief     Receiving by Slave non-blocking
  * @param     -
  * @retval    -
  */
/*--------------------------------------------------*/
static void ts_i2c_slave_non_blocking_receive(void)
{
    TXZ_Result result_mem ;
    uint8_t write_read_mode ;
    uint8_t write_read_size ;
    uint32_t i;

    slave_rx_length = 0;
    slave_tx_length = 0;

    transfer_event = 0;
    if (i2c_slave_transfer_asynch(&obj, 
                            I2C_NULL,
                            0,
                            slave_rx_data, 
                            2
                            ) != TXZ_SUCCESS)
    {
        printf("slave error!!\r\n");
    }
    else
    {
        while (1)
        {
            volatile uint32_t event = transfer_event;   /* asynch information */

            if ((event & I2C_EVENT_ALL) != 0)
            {
                /* event occured */
                if (event & I2C_EVENT_TRANSFER_COMPLETE)
                {
                    /* complete */
                    slave_tx_length = obj.tx_buff.pos;
                    slave_rx_length = obj.rx_buff.pos;
                }
                else if (event & I2C_EVENT_TRANSFER_EARLY_NACK)
                {
                    /* early complete */
                    i2c_slave_abort_asynch(&obj);
                    slave_tx_length = obj.tx_buff.pos;
                    slave_rx_length = obj.rx_buff.pos;
                    printf("EARLY_NACK!!\r\n");
                }
                else  /* if (event & I2C_EVENT_ERROR) */
                {
                    /* other error */
                    i2c_slave_abort_asynch(&obj);
                    printf("ERROR!!\r\n");
                }
                break;
            }
            else
            {
                /* no event */
                /* wait forever */
            }
        }   
    }

    ts_i2c_slave_abort_check() ;
    write_read_mode = slave_rx_data[0] ;
    write_read_size = slave_rx_data[1] ;
    transfer_event = 0;

     result_mem = TXZ_ERROR ;
    if ( write_read_mode == CMD_MASTER_WRITE )           /* Master is writing */
    {
         result_mem = i2c_slave_transfer_asynch(&obj,
                            I2C_NULL,
                            0,
                            slave_rx_data,
                            write_read_size
                            ) ;
    }
    else if ( write_read_mode == CMD_MASTER_READ )       /* Master is reading */
    {
         for(i=0;i<write_read_size;i++)
         {
             slave_tx_data[i] = MEM[i] ;
         }
         result_mem = i2c_slave_transfer_asynch(&obj,
                            slave_tx_data,
                            write_read_size,
                            I2C_NULL,
                            0
                            ) ;
    }
    else
    {
        result_mem = TXZ_ERROR ;
    }

    if ( result_mem != TXZ_SUCCESS )    
    {                                   
        printf("slave error!!\n");      
    }
    else
    {
        while (1)
        {
            volatile uint32_t event = transfer_event;   /* asynch information */

            if ((event & I2C_EVENT_ALL) != 0)
            {
                /* event occured */
                if (event & I2C_EVENT_TRANSFER_COMPLETE)
                {
                    /* complete */
                    slave_tx_length = obj.tx_buff.pos;
                    slave_rx_length = obj.rx_buff.pos;

                    if ( write_read_mode == CMD_MASTER_WRITE )           /* Master is writing */
                    {
                        for(i=0;i<write_read_size;i++)
                        {
                            MEM[i] = slave_rx_data[i] ;
                        }
                    }
                }
                else if (event & I2C_EVENT_TRANSFER_EARLY_NACK)
                {
                    /* early complete */
                    i2c_slave_abort_asynch(&obj);
                    slave_tx_length = obj.tx_buff.pos;
                    slave_rx_length = obj.rx_buff.pos;
                    printf("EARLY NACK!!\n");
                }
                else  /* if (event & I2C_EVENT_ERROR) */
                {
                    /* other error */
                    i2c_slave_abort_asynch(&obj);
                    printf("ERROR!!\n");
                }
                break;
            }
            else
            {
                /* no event */
                /* wait forever */
            }
        }   
    }
    
}

/*--------------------------------------------------*/
/**
  * @brief     Checking abort of slave
  * @param     -
  * @retval    -
  */
/*--------------------------------------------------*/
static void ts_i2c_slave_abort_check(void)
{
    timeout = TEST_TIMEOUT;
    while (i2c_active(&obj))
    {
        /* checking bus release */
        if ((timeout--) == 0)
        {
            printf("TimeOut Error\r\n");
            printf("slave reset!!\r\n");
            printf("\r\n");
            i2c_reset(&obj);
            i2c_slave_address(&obj, self_address);
            i2c_slave_mode(&obj, 1);
            break;
        }           
    }
}

/**
 *  @}
 */ /* End of group I2C_MASTER_SLAVE_Private_functions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup I2C_MASTER_SLAVE_Exported_functions I2C_MASTER_SLAVE Exported Functions
 *  @{
 */
/*--------------------------------------------------*/
/**
  * @brief      Implementation of INTI2Cx_IRQHandler
  * @param      -
  * @retval     -
  */
/*--------------------------------------------------*/
void ts_i2c_irq_handler(void)
{
    uint32_t event = 0;

    if (mode == MODE_MASTER)
    {
        event = i2c_irq_handler_asynch(&obj);
    }
    else if (mode == MODE_SLAVE)
    {
        event = i2c_slave_irq_handler_asynch(&obj);
    }
    else
    {
    }
    if (event != 0)
    {
        transfer_event = event;
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
int main(void)
{
    int i;
    int base ;
    int len = 1;

    /*----------------------*/
    /* Initialize           */
    /*----------------------*/
    /*----------------------*/
    /* BSP initialize       */
    /*----------------------*/
    bsp_initialize();
    /* Application initialization */
    application_initialize();  
    configuration();
    clear_slave_rx_data();
    clear_slave_tx_data();
    clear_master_rx_data();
    clear_master_tx_data();

    printf("\r\n");
    printf("I2C TEST - %s\r\n", I2Cx_TEXT);

    proc = PROC_MASTER_START;
    ts_i2c_initialize();

    /*----------------------*/
    /* Main Processing      */
    /*----------------------*/
    while(1)
    {
        switch(proc)
        {
        case PROC_MASTER_START:
            printf("\r\n");
            printf( "%s\r\n", " ----------------- ");
            printf( "%s\r\n", "| I2C master mode |");
            printf( "%s\r\n", " ----------------- ");
            proc = PROC_COMMAND_MSG;
            break;

        case PROC_COMMAND_MSG:
            {
                /* Clear input data.     */
                clear_input_data();
                /* Output first message. */
                if (printf("command > ") != -1)
                {
                    proc = PROC_INPUT;
                }
                else
                {
                    proc = PROC_ERROR;
                }
            }
            break;

        case PROC_INPUT:
            {
                int data;

                data = getchar();
                if ((data != -1) && (input.wp < UART_IO_RECEIVE_MAX))
                {
                    input.data[input.wp] = (char)data;
                    input.wp++;
                    switch ((char)data)
                    {
                    /*--- CR: carriage return ---*/
                    case '\r':
                        /* no process */
                        break;
                    /*--- LF: line feed ---*/
                    case '\n':
                        len = input.wp;
                        proc = PROC_COMMAND_PARSE;
                        break;
                    /*--- default ---*/
                    default:
                        /* no process */
                        break;
                    }
                }
                else
                {
                    proc = PROC_INPUT_ERR;
                }
            }
            break;

        case PROC_COMMAND_PARSE:
            if ((strncmp((const char *)&input.data[0], "write", 5)) == 0)
            {
                /* Write Command */
                clear_master_rx_data();
                len -= 1;
                if ((len==5)||(len==6))
                {
                    if((len==6)&&(input.data[5]!=0x20))
                    {
                        proc = PROC_INPUT_ERR ;
                        break ;
                    }
                    else
                    {
                        slave_address = SLAVE_ADDR ;
                        master_tx_data[0] = CMD_MASTER_WRITE ; /* cmd : 0=write / 1=read */
                        master_tx_data[1] = 0x04 ; /* size */
                        master_tx_data[2] = 0x11 ;
                        master_tx_data[3] = 0x22 ;
                        master_tx_data[4] = 0x33 ;
                        master_tx_data[5] = 0x44 ;
                        data_form1 = 6 ;
                        data_form2 = 0 ;
                    }
                }
                if (((len>=7)&&(len%2==1))||(len>8+PARAM_A_SIZE_MAX*2))  /* 7,9,11,13,15-- */
                {
                    proc = PROC_INPUT_ERR ;
                    break ;
                }
                if((check_param_hex_array(&input.data[6],len-6))==TXZ_ERROR)
                {
                    proc = PROC_INPUT_ERR ;
                    break ;
                }
                if (len >= 8)
                {
                    slave_address = get_param_hex_byte(&input.data[6]);
                    if (slave_address >= 128)
                    {
                        proc = PROC_INPUT_ERR ;
                        break ;
                     }
                }
                if (len == 8)
                {
                    master_tx_data[0] = CMD_MASTER_WRITE ; /* cmd : 0=write / 1=read */
                    master_tx_data[1] = 0x04 ; /* size */
                    master_tx_data[2] = 0x11 ;
                    master_tx_data[3] = 0x22 ;
                    master_tx_data[4] = 0x33 ;
                    master_tx_data[5] = 0x44 ;
                    data_form1 = 6 ;
                    data_form2 = 0 ;
                }
                if ((len>=10)&&(len%2==0)&&(len<=8+PARAM_A_SIZE_MAX*2))          /* 10,12,14,16 */
                {
                    master_tx_data[0] = CMD_MASTER_WRITE ;      /* cmd : 0=write / 1=read */
                    master_tx_data[1] = (len-8)/2 ; /* size */
                    get_param_hex_array(&master_tx_data[2], &input.data[8], (len - 8));
                    data_form1 = (len-8)/2 ;
                    data_form2 = 0 ;
                }
                proc = PROC_MASTER_WRITE;
            }
            else if ((strncmp((const char *)&input.data[0], "read", 4)) == 0)
            {
                /* Read Command */
                clear_master_rx_data();
                len -= 1;
                if ((len==4)||(len==5))
                {
                    if((len==5)&&(input.data[4]!=0x20))
                    {
                        proc = PROC_INPUT_ERR ;
                        break ;
                     }
                     else
                     {
                        slave_address = SLAVE_ADDR ;
                        master_tx_data[0] = CMD_MASTER_READ ; /* cmd : 0=write / 1=read */
                        master_tx_data[1] = INIT_RNUM ; /* size */
                        data_form1 = 2 ;
                        data_form2 = INIT_RNUM ;
                     }
                }
                if (((len>=6)&&(len%2==0))||(len>7+PARAM_B_SIZE_MAX*2))  /* 6,8,10,12-- */
                {
                    proc = PROC_INPUT_ERR ;
                    break ;
                }
                if((check_param_hex_array(&input.data[5],len-5))==TXZ_ERROR)
                {
                    proc = PROC_INPUT_ERR ;
                    break ;
                }
                if (len >= 7)
                {
                    slave_address = get_param_hex_byte(&input.data[5]);
                    if (slave_address >= 128)
                    {
                        proc = PROC_INPUT_ERR ;
                        break ;
                    }
                }
                if (len == 7)
                {
                    master_tx_data[0] = CMD_MASTER_READ ; /* cmd : 0=write / 1=read */
                    master_tx_data[1] = INIT_RNUM ; /* size */
                    data_form1 = 2 ;
                    data_form2 = INIT_RNUM ;
                }
                if ((len>=9)&&(len%2==1)&&(len<=7+PARAM_B_SIZE_MAX*2))          /* 9,11, */
                {
                    for(i=0;i<DATA_LENGTH;i++)
                    {
                        master_tx_data[i] = 0 ;
                    }
                    master_tx_data[0] = CMD_MASTER_READ ;      /* cmd : 0=write / 1=read */
                    get_param_hex_array(&master_tx_data[1], &input.data[7], (len-7) ); /* size */
                    data_form1 = 2 ;

                    data_form2 = 0 ;
                    base = 1 ;
                    for(i=(len-7)/2-1;i>=0;i--)  /* tx[2] + tx[1]*16 */
                    {
                        data_form2 += master_tx_data[i+1] * base ;
                        base *= 16 ;
                    }
                    master_tx_data[1] = data_form2 ;      /* size */
                    master_tx_data[2] = 0x00 ;            /* NULL */

                    if((data_form2==0)||(data_form2>PARAM_B_DATA_MAX)) {
                        proc = PROC_INPUT_ERR ;
                        break ;
                    }
                }

                proc = PROC_MASTER_WRITE_READ;
            }
            else if ((strncmp((const char *)&input.data[0], "slave", 5)) == 0)
            {
                /* Slave Mode Command */
                len -= 2;
                if (len >= 8)
                {
                    self_address = get_param_hex_byte(&input.data[6]);
                }
                printf("\r\n");
                proc = PROC_SLAVE_START;
            }
            else if ((strncmp((const char *)&input.data[0], "form", 4)) == 0)
            {
                /* Data Form Command */
                len -= 2;
                if (len >= 10)
                {
                    uint32_t length1 = get_param_hex_byte(&input.data[5]);
                    uint32_t length2 = get_param_hex_byte(&input.data[8]);
                    
                    if ((length1 <= DATA_LENGTH) && (length2 <= DATA_LENGTH) && ((length1 + length2) <= DATA_LENGTH))
                    {
                        data_form1 = length1;
                        data_form2 = length2;
                    }
                    else
                    {
                        printf("not support\r\n");
                    }
                }
                printf("data form > %02X %02X\r\n", data_form1, data_form2);
                proc = PROC_COMMAND_END;
            }
            else if ((strncmp((const char *)&input.data[0], "hz", 2)) == 0)
            {
               /* Frequency Command */
                if (len >= (3+2))
                {
                    /* "hz d" */
                    frequency = strtol(&input.data[3], 0, 10);

                    i2c_reset(&obj);

                    if (i2c_frequency(&obj, frequency) != TXZ_SUCCESS)
                    {
                        printf("i2c frequency(%d) error!!\r\n", frequency);
                        printf("please reset!!\r\n");
                        proc = PROC_COMMAND_END;
                        break;
                    }
                }
                printf("i2c frequency(%d)\r\n", frequency);
                proc = PROC_COMMAND_END;
            }
            else if ((strncmp((const char *)&input.data[0], "api", 3)) == 0)
            {
                /* API Select Command */
                len -= 2;
                if (len >= 4)
                {
                    /* "api m" */
                    if ((strncmp((const char *)&input.data[4], "m", 1)) == 0)
                    {
                        master_blocking = (master_blocking)? 0:1;
                    }
                    /* "api s" */
                    if ((strncmp((const char *)&input.data[4], "s", 1)) == 0)
                    {
                        slave_blocking = (slave_blocking)? 0:1;
                    }
                }
                if (master_blocking)
                {
                    printf("master blocking\r\n");
                }
                else
                {
                    printf("master non-blocking\r\n");
                }
                if (slave_blocking)
                {
                    printf("slave blocking\r\n");
                }
                else
                {
                    printf("slave non-blocking\r\n");
                }
                proc = PROC_COMMAND_END;
            }
            else if ((strncmp((const char *)&input.data[0], "set m", 5)) == 0)
            {
                int i;
               /* Master Tx Data Command */
               len -= 2;
               if (len >= 8)
               {
                    get_param_hex_array(master_tx_data, &input.data[6], (len - 6));
               }
                printf("master_tx_data\r\n");
                for (i = 0; i < DATA_LENGTH; i++)
                {
                    printf("tx[%d] %02X\r\n", i, master_tx_data[i]);
                }          
                proc = PROC_COMMAND_END;
            }
            else if ((strncmp((const char *)&input.data[0], "set s", 5)) == 0)
            {
                int i;
               /* Slave Tx Data Command */
               len -= 2;
               if (len >= 8)
               {
                    get_param_hex_array(slave_tx_data, &input.data[6], (len - 6));
               }
                printf("slave_tx_data\r\n");
                for (i = 0; i < DATA_LENGTH; i++)
                {
                    printf("tx[%d] %02X\r\n", i, slave_tx_data[i]);
                }          
                proc = PROC_COMMAND_END;
            }
            else if ((strncmp((const char *)&input.data[0], "w", 1)) == 0)
            {
                /* Abbreviation of Write Command */
                if (len == 3)
                {
                    proc = PROC_MASTER_WRITE;
                }
                else
                {
                    proc = PROC_INPUT_ERR;
                }
            }
            else if ((strncmp((const char *)&input.data[0], "r", 1)) == 0)
            {
                /* Abbreviation of Read Command */
                if (len == 3)
                {
                    proc = PROC_MASTER_WRITE_READ;
                }
                else
                {
                    proc = PROC_INPUT_ERR;
                }
            }
            else
            {
                proc = PROC_INPUT_ERR;
            }
            break;

        case PROC_COMMAND_END:
            printf("\r\n");
            proc = PROC_COMMAND_MSG;
            break;

        case PROC_MASTER_WRITE:
            if (master_blocking)
            {
                ts_i2c_master_blocking_write();
            }
            else
            {
                ts_i2c_master_non_blocking_write();
            }
            proc = PROC_MASTER_END;
            break;

        case PROC_MASTER_WRITE_READ:
            if (master_blocking)
            {
                ts_i2c_master_blocking_write_read();
            }
            else
            {
                ts_i2c_master_non_blocking_write_read();
            }
            proc = PROC_MASTER_END;
            break;

        case PROC_MASTER_END:
            {
                int i;
                printf("master\r\n");
                printf("sa    %02X\r\n", slave_address);
                for (i = 0; i < master_tx_length; i++)
                {
                    printf("tx[%d] %02X\r\n", i, master_tx_data[i+2]);
                }
                for (i = 0; i < master_rx_length; i++)
                {
                    printf("rx[%d] %02X\r\n", i, master_rx_data[i]);
                }
                proc = PROC_COMMAND_END;
            }
            break;

        case PROC_INPUT_ERR:
            printf("?\r\n");
            proc = PROC_COMMAND_MSG;
            break;
        
        case PROC_SLAVE_START:
            printf("\r\n");
            printf( "%s\r\n", " ---------------- ");
            printf( "%s\r\n", "| I2C slave mode |");
            printf( "%s\r\n", " ---------------- ");
            ts_i2c_set_slave_mode();
            proc = PROC_SLAVE_RECEIVE;
            break;

        case PROC_SLAVE_RECEIVE:
            ts_i2c_slave_abort_check();
            printf("slave\r\n");
            printf("sa    %02X\r\n", self_address);
            clear_slave_rx_data();
            if (slave_blocking)
            {
                ts_i2c_slave_blocking_receive();
            }
            else
            {
                ts_i2c_slave_non_blocking_receive();
            }
            proc = PROC_SLAVE_END;
            break;

        case PROC_SLAVE_END:
            {
                int i;
                for (i = 0; i < slave_rx_length; i++)
                {
                    printf("rx[%d] %02X\r\n", i, slave_rx_data[i]);
                }
                for (i = 0; i < slave_tx_length; i++)
                {
                    printf("tx[%d] %02X\r\n", i, slave_tx_data[i]);
                }
            }
            printf("\r\n");
            
            if (slave_continue != 0) slave_continue--;
            if (slave_continue == 0)
            {
                slave_continue = SLAVE_CONTINUE;
                ts_i2c_set_master_mode();
                proc = PROC_MASTER_START;
            }
            else
            {
                proc = PROC_SLAVE_RECEIVE;
            }
            break;

        case PROC_ERROR:
            /* System Error. Please Debug. */
#ifdef __DEBUG__
            assert_failed(MAIN_NULL, 0);
#endif
            break;

        default:
            proc = PROC_ERROR;
            break;
        }
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART reception IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_rx(BSPUsbUart uart)
{
    switch (uart)
    {
    /*--- usb uart 0 ---*/
    case BSP_USB_UART_0:
        uart_io_rx_irq_handler();
        break;
    /*--- default    ---*/
    default:
        /* no processing */
        break;
    }

}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART transmit IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_tx(BSPUsbUart uart)
{
    switch (uart)
    {
    /*--- usb uart 0 ---*/
    case BSP_USB_UART_0:
        uart_io_tx_irq_handler();
        break;
    /*--- default    ---*/
    default:
        /* no processing */
        break;
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART error IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_err(BSPUsbUart uart)
{
    switch (uart)
    {
    /*--- usb uart 0 ---*/
    case BSP_USB_UART_0:
        uart_io_err_irq_handler();
        break;
    /*--- default    ---*/
    default:
        /* no processing */
        break;
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  I2C IRQ.
  * @param  -
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_i2c(void)
{
    ts_i2c_irq_handler();
}

#ifdef __DEBUG__
/*--------------------------------------------------*/
/** 
  * @brief  Failure Assert.
  * @note   for debug
  */
/*--------------------------------------------------*/
void assert_failed(char *file, int32_t line)
{
    while (1) {
      /*  __NOP(); */
    }
}
#endif

/**
 *  @}
 */ /* End of group I2C_MASTER_SLAVE_Exported_functions */

/**
 *  @} 
 */ /* End of group I2C_MASTER_SLAVE */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
