/** 
 *******************************************************************************
 * @file    main.c
 * @brief   SMIF FLASH Sample Application.
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2020
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif
/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "txz_sample_def.h"
#include "bsp.h"
#include "bsp_uart_io.h"
#include "bsp_timer.h"
#include "txz_hal.h"
#include "txz_smif.h"
/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup SMIF_FLASH SMIF_FLASH Sample Appli
 *  @{
 */

/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup SMIF_FLASH_Private_macro SMIF_FLASH Private Macro
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group SMIF_FLASH_Private_macro */


/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup SMIF_FLASH_Private_define SMIF_FLASH Private Define
 *  @{
 */
/* no define */

/** 
 *  @}
 */ /* End of group SMIF_FLASH_Private_define */

/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup SMIF_FLASH_Private_define SMIF_FLASH Private Define
 *  @{
 */

#define MAIN_NULL       ((void *)0)         /*!< Null Pointer.            */
#define STRING_NUM      (16)                /*!< String Size. */
#define EXT_FLASH_BASE  (0xA0000000)        /*!< Access location address. */
/*----------------------*/
/*  UART Setting        */
/*----------------------*/
/** 
 *  @defgroup UART_Setting UART Setting
 *  @{
 */
#define USER_ID_UART    ((uint32_t)1)   /*!< User ID :USB UART (Any value) */
/** 
 *  @}
 */ /* End of group UART_Setting */

/** 
 *  @}
 */ /* End of group SMIF_FLASH_Private_define */

/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup SMIF_FLASH_Private_define SMIF_FLASH Private Define
 *  @{
 */

/*----------------------------------*/
/** 
 * @enum   Processing
 * @brief  Processing mode.
*/
/*----------------------------------*/
typedef enum
{
    PROC_IDLE,              /*!< Idle.                        */
    PROC_OUTPUT_MSG,        /*!< Output first message.        */
    PROC_INPUT,             /*!< Wait to input.               */
    PROC_COMMAND_PARSE,     /*!< Output Echo back data.       */
    PROC_OUTPUT_ERR,        /*!< Output input error message.  */
    PROC_ERROR,             /*!< Error.                       */
    PROC_STOP               /*!< STOP.                        */
} Processing;

/** 
 *  @}
 */ /* End of group SMIF_FLASH_Private_define */

/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup SMIF_FLASH_Private_typedef SMIF_FLASH Private Typedef
 *  @{
 */
/*----------------------------------*/
/** 
 * @struct input_t
 * @brief  Input data information.
*/
/*----------------------------------*/
typedef struct
{
    char    data[(UART_IO_RECEIVE_MAX+1)];  /*!< Input data(+ for NULL 1byte). */
    uint8_t wp;                             /*!< Write pointer.                */
} input_t;
/*----------------------------------*/
/** 
 * @brief Instance Information.
*/
/*----------------------------------*/
typedef struct
{
    SMIF_t      smif;           /*!< Driver      :SMIF.     */
    t32a_t      t32a;           /*!< Driver      :T32A.     */
    timer_t     timer;          /*!< Application :Timer (1ms timer). */
} instance_t;


/** 
 *  @}
 */ /* End of group SMIF_FLASH_Private_typedef */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup SMIF_FLASH_Private_variables SMIF_FLASH Private Variables
 *  @{
 */


static instance_t instance;     /*!< Instance.               */
static Processing proc;         /*!< Processing mode.        */
static input_t input;           /*!< Input data information. */
static uint8_t ch = 0;          /*!< SMIF channel            */

/**
 *  @}
 */ /* End of group SMIF_FLASH_Private_variables */


/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup SMIF_FLASH_Private_functions SMIF_FLASH Private Fuctions
 *  @{
 */
static void clear_data(void);
static void variable_initialize(void);
static TXZ_Result driver_initialize(void);
static TXZ_Result driver_finalize(void);
static TXZ_Result application_initialize(void);
static TXZ_Result application_finalize(void);
static void timer_interval_handler(uint32_t id);

/*--------------------------------------------------*/
/** 
  * @brief  Clear input data.
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void clear_data(void)
{
    uint8_t i;

    for (i = 0; i < (UART_IO_RECEIVE_MAX+1); i++)
    {
        input.data[i] = 0;
    }
    input.wp = 0;
    
}
/*--------------------------------------------------*/
/** 
  * @brief  Variable Initialize
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void variable_initialize(void)
{
    /* Instance. */
    instance.t32a.p_instance    = MAIN_NULL;
    instance.timer.init.p_t32a  = MAIN_NULL;
}
/*--------------------------------------------------*/
/** 
  * @brief  Driver Initialize
  * @param  -
  * @retval TXZ_SUCCESS :Success
  * @retval TXZ_ERROR   :Failure
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result driver_initialize(void)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*----------------------*/
    /* T32A                 */
    /*----------------------*/
    {
        t32a_t *p_t32a = &instance.t32a;
        
        /*--- Construct  ---*/
        /* Register Allocation */
        switch (bsp_get_timer_ch(BSP_TIMER_1MS))
        {
        case 0:
            p_t32a->p_instance = TSB_T32A0;
            break;
        case 1:
            p_t32a->p_instance = TSB_T32A1;
            break;
        case 2:
            p_t32a->p_instance = TSB_T32A2;
            break;
        case 3:
            p_t32a->p_instance = TSB_T32A3;
            break;
        case 4:
            p_t32a->p_instance = TSB_T32A4;
            break;
        case 5:
            p_t32a->p_instance = TSB_T32A5;
            break;
        default:
            p_t32a->p_instance = MAIN_NULL;
            break;
        }
        /*--- Initialize ---*/
        p_t32a->init_mode.mode.halt = T32A_DBG_HALT_STOP;
        p_t32a->init_mode.mode.mode = T32A_MODE_16;
        if (p_t32a->p_instance != MAIN_NULL)
        {
            if (t32a_mode_init(p_t32a) != TXZ_SUCCESS)
            {
                result = TXZ_ERROR;
            }
        }
        else
        {
            result = TXZ_ERROR;
        }
    }
    /*----------------------*/
    /* SMIF                 */
    /*----------------------*/
    {
        SMIF_t *p_smif = &instance.smif;
        
        p_smif->p_instance = TSB_SMI0;
        p_smif->init.map[0].base     = 0x00000000;
        p_smif->init.map[0].capacity = SMIF_FDEN_16MB;
        p_smif->init.map[0].wenable   = SMIF_MAP_DISABLE;
        p_smif->init.map[0].renable   = SMIF_MAP_ENABLE;
        p_smif->init.map[1].base     = 0x00000000;
        p_smif->init.map[1].capacity = SMIF_FDEN_16MB;
        p_smif->init.map[1].wenable   = SMIF_MAP_DISABLE;
        p_smif->init.map[1].renable   = SMIF_MAP_DISABLE;
        p_smif->init.clock.trans_clock  = 0x07;     /*less than 25MHz*/
        p_smif->init.clock.deasert_time = 0x02;
        p_smif->init.clock.poll_wip     = SMIF_WIP_POLLING_ENABLE;
        p_smif->init.direct[0].CmdOP = SMIF_CMD_QUAD_I_O;
        p_smif->init.direct[1].CmdOP = SMIF_CMD_QUAD_I_O;
        if(smif_init(p_smif) != TXZ_SUCCESS)
        {
            result = TXZ_ERROR;
        }
    }
    return (result);
}
/*--------------------------------------------------*/
/** 
  * @brief  Driver Finalize
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success
  * @retval TXZ_ERROR   :Failure
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result driver_finalize(void)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*----------------------*/
    /* T32A                 */
    /*----------------------*/
    {
        t32a_t *p_t32a = &instance.t32a;

        /*--- Finalize ---*/
        /* no processing */
        /*--- Destruct ---*/
        /* Register Release */
        p_t32a->p_instance = MAIN_NULL;
    }
    /*----------------------*/
    /* SMIF                 */
    /*----------------------*/
    {
        if(smif_finalize() != TXZ_SUCCESS)
        {
            result = TXZ_ERROR;
        }
    }
    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Application Initialize.
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result application_initialize(void)
{
   TXZ_Result result = TXZ_SUCCESS;

    /*----------------------*/
    /* Timer                */
    /*----------------------*/
    /* 1ms timer */
    {
        timer_t *p_timer = &instance.timer;

        p_timer->init.id       = (uint32_t)&instance.timer;
        p_timer->init.p_t32a   = &instance.t32a;
        p_timer->init.type     = bsp_get_timer_type(BSP_TIMER_1MS);
        p_timer->init.irq      = bsp_get_timer_nvic(BSP_TIMER_1MS);
        p_timer->init.interval = 1000;
        p_timer->init.handler  = timer_interval_handler;
        timer_initialize(p_timer);
    }
    /*----------------------*/
    /* Usb Uart             */
    /*----------------------*/
    /* Tx & Rx use same UART channel. */
    {
        uart_io_initial_setting_t param;

        param.id         = USER_ID_UART;
        switch (bsp_get_usb_uart_tx_ch(BSP_USB_UART_0))
        {
        case 0:
            param.p_reg  = TSB_UART0;
            break;
        case 1:
            param.p_reg  = TSB_UART1;
            break;
        case 2:
            param.p_reg  = TSB_UART2;
            break;
        default:
            param.p_reg  = MAIN_NULL;
            break;
        }
        param.boudrate   = bsp_get_usb_uart_tx_baudrate(BSP_USB_UART_0);
        param.irq.tx     = bsp_get_usb_uart_tx_nvic(BSP_USB_UART_0);
        param.irq.rx     = bsp_get_usb_uart_rx_nvic(BSP_USB_UART_0);
        param.irq.err    = bsp_get_usb_uart_err_nvic(BSP_USB_UART_0);
        if (param.p_reg != MAIN_NULL)
        {
            result  = uart_io_initialize(&param);
        }
        else
        {
            result = TXZ_ERROR;
        }
    }
    return (result);
}
/*--------------------------------------------------*/
/** 
  * @brief  Application Finalize.
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result application_finalize(void)
{
    TXZ_Result result = TXZ_SUCCESS;
    
    /*----------------------*/
    /* Usb Uart             */
    /*----------------------*/
    {
        uart_io_finalize();
    }
    /*----------------------*/
    /* Timer                */
    /*----------------------*/
    if (instance.timer.init.p_t32a != MAIN_NULL)
    {
        timer_finalize(&instance.timer);
    }
    
    return (result);
}
/*--------------------------------------------------*/
/** 
  * @brief  Interval Time Handler.
  * @param  id        :User ID
  * @return -
  * @retval -
  * @note   for 1ms Timer.
  * @note   Call by Timer.
  */
/*--------------------------------------------------*/
static void timer_interval_handler(uint32_t id)
{
    if (id == (uint32_t)&instance.timer)
    {
        hal_inc_tick();
    }
}

/**
 *  @}
 */ /* End of group SMIF_FLASH_Private_functions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup SMIF_FLASH_Exported_functions SMIF_FLASH Exported Functions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
int main(void)
{
    uint32_t num;
    uint32_t len = 0;
    uint32_t cnum = 0;
    /*----------------------*/
    /* Set Processing Mode  */
    /*----------------------*/
    proc = PROC_OUTPUT_MSG;
    /*----------------------*/
    /* Initialize           */
    /*----------------------*/
    variable_initialize();
    /*----------------------*/
    /* BSP initialize       */
    /*----------------------*/
    bsp_initialize();
    if(driver_initialize() != TXZ_SUCCESS)
    {
        proc = PROC_ERROR;
    }
    /* Application initialization */
    if(application_initialize() != TXZ_SUCCESS)
    {
        proc = PROC_ERROR;
    }
    
    /*----------------------*/
    /* Main Processing      */
    /*----------------------*/
    while(proc != PROC_STOP)
    {
        switch(proc)
        {
        /*--- Output "First Message" ---*/
        case PROC_OUTPUT_MSG:
            {
                /* Clear input data.     */
                clear_data();
                /* Output first message. */
                if (printf("command > ") != -1)
                {
                    proc = PROC_INPUT;
                }
                else
                {
                    proc = PROC_ERROR;
                }
            }
            break;
        /*--- Wait to Input ---*/
        case PROC_INPUT:
            {   
                int data;
                
                data = getchar();
                if ((data != -1) && (input.wp < UART_IO_RECEIVE_MAX))
                {
                    input.data[input.wp] = (char)data;
                    input.wp++;
                    switch ((char)data)
                    {
                    /*--- C.dataR: carriage return ---*/
                    case '\r':
                        /* no process */
                        break;
                    /*--- LF: line feed ---*/
                    case '\n':
                        len = input.wp;
                        proc = PROC_COMMAND_PARSE;
                        break;
                    /*--- default ---*/
                    default:
                        /* no process */
                        break;
                    }
                }
                else
                {
                    proc = PROC_OUTPUT_ERR;
                }
            }
            break;
        /*--- Output "Echo Back Data" ---*/
        case PROC_COMMAND_PARSE:
            {
                uint32_t i;
                int32_t res = 0;

                if ((strncmp((const char *)&input.data[0], "write", 5)) == 0)
                {
                    /* WRITE Command */
                    if (len >= 9)
                    {
                        int32_t address = 0;
                        uint8_t *flash_ptr = (uint8_t *)EXT_FLASH_BASE;
                        (void)printf("write data > ");
                        num = (len - (6 + 2));
                        if(num > STRING_NUM){
                            num = STRING_NUM;
                        }
                        input.data[6+num] = 0x00;
                        if(smif_erase_sector((uint8_t *)address, ch) != TXZ_SUCCESS)
                        {
                            /* no process */
                        }
                        if(smif_write_data((uint8_t *)address, (uint8_t *)&input.data[6], num+1, ch) != TXZ_SUCCESS)
                        {
                            /* no process */
                        }
                        for (i = 0; i < num; i++)
                        {
                            if(*flash_ptr++ != (uint8_t)(input.data[6 + i]))
                            {
                                res = -1;
                                break;
                            }
                        }
                        cnum = i;
                        
                        if (res >= 0)
                        {
                            flash_ptr = (uint8_t *)EXT_FLASH_BASE;
                            (void)printf("%s\n", (const char *)flash_ptr);
                            proc = PROC_OUTPUT_MSG;
                        }
                        else
                        {
                            (void)printf("\n write error !! \n");
                            proc = PROC_ERROR;
                        }
                        (void)printf("\n");
                    }
                    else
                    {
                        proc = PROC_OUTPUT_ERR;
                    }
                    break;
                }
                else if ((strncmp((const char *)&input.data[0], "read", 4)) == 0)
                {
                    /* READ Command */
                    (void)printf("read data > ");
                    if (cnum > 0)
                    {
                        uint8_t *flash_ptr = (uint8_t *)EXT_FLASH_BASE;
                        if(*(flash_ptr + num) == 0)
                        {
                            (void)printf("%s\n", (const char *)flash_ptr);
                        }
                        else
                        {
                            (void)printf("\n read error(no write data) !! \n");
                        }
                        proc = PROC_OUTPUT_MSG;
                    }
                    else
                    {
                        (void)printf("\n read error(no write data) !! \n");
                        proc = PROC_OUTPUT_MSG;
                    }
                    (void)printf("\n");
                }
                else if ((strncmp((const char *)&input.data[0], "dump", 4)) == 0)
                {
                    /* DAMP Command */
                    uint8_t *flash_ptr = (uint8_t *)EXT_FLASH_BASE;
                    char *data_ptr;
                    char str[9] = {0};
                    char dump_len_str[5] = {0};
                    int dump_length = 0x20;
                    uint32_t dump_count = 0;
                    uint32_t j;
                    int32_t address = 0;
                    
                    if(len > 9)
                    {
                        data_ptr = strchr((const char *)&input.data[6], (int)' ');
                        if(data_ptr == MAIN_NULL)
                        {
                            for(i= 5; ((i< len) && (i< (5+6))); ++i)
                            {
                                str[i-5]= input.data[i];
                            }
                            address = strtol(str, 0, 16);
                            if((address < 0) || (address > 0xFFFFFF))
                            {
                                (void)printf("\n address error !! \n");
                                proc = PROC_OUTPUT_MSG;
                                break;
                            }
                        }
                        else{
                            for(i= 5; ((i< len) && (i< (5+6))); ++i)
                            {
                                if(data_ptr == &input.data[i])
                                {
                                    break;
                                }
                                str[i-5]= input.data[i];
                            }
                            address = strtol(str, 0, 16);
                            if((address < 0) || (address > 0xFFFFFF))
                            {
                                (void)printf("\n address error !! \n");
                                proc = PROC_OUTPUT_MSG;
                                break;
                            }
                            ++i;
                            for(j=i; ((j< len) && (j< (i+4))); ++j)
                            {
                                dump_len_str[j-i]= input.data[j];
                            }
                            dump_length = strtol(dump_len_str, 0, 16);
                            if((dump_length < 0) || (dump_length > 0xFFFF))
                            {
                                (void)printf("\n length error !! \n");
                                proc = PROC_OUTPUT_MSG;
                                break;
                            }
                            if(len > (j+2))
                            {
                                (void)printf("\n length error !! \n");
                                proc = PROC_OUTPUT_MSG;
                                break;
                            }
                        }
                    }
                    else
                    {
                        for(i= 5; ((i< len) && (i< (5+6))); ++i)
                        {
                            str[i-5]= input.data[i];
                        }
                        address = strtol(str, 0, 16);
                        if((address < 0) || (address > 0xFFFFFF))
                        {
                            (void)printf("\n address error !! \n");
                            proc = PROC_OUTPUT_MSG;
                            break;
                        }
                    }
                    flash_ptr += address;
                    (void)printf("dump data >");
                    do
                    {
                        if((dump_count++ % 0x10) == 0x00)
                        {
                            (void)printf("\n");
                            (void)printf("0x%08X: ", (uint32_t)flash_ptr - EXT_FLASH_BASE);
                        }
                        (void)printf("%02X ", *flash_ptr++);
                    }
                    while(--dump_length > 0);
                    (void)printf("\n");
                    proc = PROC_OUTPUT_MSG;
                }
                else if ((strncmp((const char *)&input.data[0], "erase", 5)) == 0)
                {
                    /* ERASE Command */
                    char str[9] = {0};
                    int32_t address;
                    
                    if((len == 7) || (len == 8))
                    {
                        address = 0;
                    }
                    else if((len >= 9) && (len <= 14))
                    {
                        for(i=0;i<(len-4);++i)
                        {
                            str[i] = input.data[i+6];
                        }
                        address = strtol(str, 0, 16);
                    }
                    else
                    {
                        proc = PROC_OUTPUT_ERR;
                        break;
                    }
                     (void)printf("erase data > ");
                    if((address < 0) || (address > 0xFFFFFF))
                    {
                        (void)printf("\n address error !! \n");
                        proc = PROC_OUTPUT_MSG;
                        break;
                    }
                    if(smif_erase_sector((uint8_t *)address, ch) != TXZ_SUCCESS)
                    {
                        /* no process */
                    }
                    (void)printf("erase:0x%x\n", address);
                    
                    (void)printf("\n");
                    proc = PROC_OUTPUT_MSG;
                }
                else if ((strncmp((const char *)&input.data[0], "bw", 2)) == 0)
                {
                    /* BYTE WRITE Command */
                    if (len > 7)
                    {
                        char *data_ptr;
                        char str[7] = {0};
                        int32_t address;
                        char pr_data[2] = {0};
                        
                        data_ptr = strchr((const char *)&input.data[4], (int)' ');
                        if(data_ptr == MAIN_NULL)
                        {
                            proc = PROC_OUTPUT_ERR;
                            break;
                        }
                        for(i= 3; ((i< len) && (i< (3+6))); ++i)
                        {
                            if(data_ptr == &input.data[i])
                            {
                                break;
                            }
                            str[i-3]= input.data[i];
                        }
                        address = strtol(str, 0, 16);
                        if((address < 0) || (address > 0xFFFFFF))
                        {
                            (void)printf("\n address error !! \n");
                            proc = PROC_OUTPUT_MSG;
                            break;
                        }
                        ++data_ptr;
                        if(smif_write_data((uint8_t *)(address), (uint8_t *)data_ptr, 1, ch) != TXZ_SUCCESS)
                        {
                            /* no process */
                        }
                        pr_data[0] = *(char *)((uint32_t)address+EXT_FLASH_BASE);
                        pr_data[1] = 0;
                        
                        if (pr_data[0] == (uint8_t)*data_ptr)
                        {
                            (void)printf("0x%x = %s(0x%02X)\n", address, (const char *)&pr_data[0], pr_data[0]);
                            proc = PROC_OUTPUT_MSG;
                        }
                        else
                        {
                            (void)printf("\n write error !! \n");
                            (void)printf("0x%x = %s(0x%02X)\n", address, (const char *)&pr_data[0], pr_data[0]);
                            proc = PROC_ERROR;
                        }
                        (void)printf("\n");
                    }
                    else
                    {
                        proc = PROC_OUTPUT_ERR;
                    }
                }
                else if ((strncmp((const char *)&input.data[0], "br", 2)) == 0)
                {
                    /* BYTE READ Command */
                    char str[9] = {0};
                    int32_t address;
                    char pr_data[2] = {0};

                    if((len == 4) || (len == 5))
                    {
                        address = 0;
                    }
                    else if((len >= 6) && (len <= 11))
                    {
                        for(i=0;i<(len-4);++i)
                        {
                            str[i] = input.data[i+3];
                        }
                        address = strtol(str, 0, 16);
                    }
                    else
                    {
                        proc = PROC_OUTPUT_ERR;
                        break;
                    }
                     (void)printf("byte read data > ");
                    if((address < 0) || (address > 0xFFFFFF))
                    {
                        (void)printf("\n address error !! \n");
                        proc = PROC_OUTPUT_MSG;
                        break;
                    }
                    pr_data[0] = *(char *)((uint32_t)address + EXT_FLASH_BASE);
                    pr_data[1] = 0;
                    if (*(uint8_t *)((uint32_t)address + EXT_FLASH_BASE) == (uint8_t)(pr_data[0]))
                    {
                        (void)printf("0x%x = %s(0x%02X)\n", address, (const char*)&pr_data[0], pr_data[0]);
                        proc = PROC_OUTPUT_MSG;         
                    }
                    else
                    {
                        (void)printf("\n read error !! \n");
                        proc = PROC_ERROR;
                    }
                    (void)printf("\n");
                }
                else if ((strncmp((const char *)&input.data[0], "bank", 4)) == 0)
                {
                    /* Bank Write Command */
                    if(len == 6)
                    {
                        uint8_t read_bank;
                        
                        if(smif_bank_read(&read_bank, ch) != TXZ_SUCCESS)
                        {
                            /* no process */
                        }
                        (void)printf("bank > %X ", read_bank);
                        (void)printf("\n");
                        proc = PROC_OUTPUT_MSG;
                    }
                    else if(len == 8)
                    {
                        uint8_t bank = 0;
                        
                        if((input.data[5] == '0') || (input.data[5] == '1') || (input.data[5] == '2') || (input.data[5] == '3'))
                        {
                            bank = (uint8_t)(input.data[5] - '0');
                        }
                        if(smif_bank_write(bank, ch) == TXZ_SUCCESS)
                        {
                            uint8_t read_bank;
                            
                            if(smif_bank_read(&read_bank, ch) != TXZ_SUCCESS)
                            {
                                /* no process */
                            }
                            (void)printf("bank > %X ", read_bank);
                            (void)printf("\n");
                        }
                        proc = PROC_OUTPUT_MSG;
                    }
                    else
                    {
                        proc = PROC_OUTPUT_ERR;
                    }
                }
                else
                {
                    proc = PROC_OUTPUT_ERR;
                }
            }
            break;
                
        /*--- Output "Input Error Message" ---*/
        case PROC_OUTPUT_ERR:
            (void)printf("? \n");
            proc = PROC_OUTPUT_MSG;
            break;
        /*--- Error ---*/
        case PROC_ERROR:
            /* System Error. Please Debug. */
#ifdef __DEBUG__
            assert_failed(MAIN_NULL, 0);
#endif
            proc = PROC_OUTPUT_MSG;
            break;
        /*--- Default ---*/
        default:
            proc = PROC_STOP;
            break;
        }
    }
    if(application_finalize() != TXZ_SUCCESS)
    {
        /* no process */
    }
    if(driver_finalize() != TXZ_SUCCESS)
    {
        /* no process */
    }
    bsp_finalize();
    while(1)
    {
    }
}
/*--------------------------------------------------*/
/** 
  * @brief  Systick IRQ.
  * @param  -
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_systick(void)
{
    /* Sample Soft don't use this function. */
}
/*--------------------------------------------------*/
/** 
  * @brief  Push SW State Change.
  * @param  psw :Push SW.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_psw(BSPPsw psw)
{
    /* Sample Soft don't use this function. */
}


/*--------------------------------------------------*/
/** 
  * @brief  SPI flash reception IRQ.
  * @param  sflash :SPI Flash.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_sflash_rx(BSPSFlash sflash)
{
    /* Sample Soft don't use this function. */
}

/*--------------------------------------------------*/
/** 
  * @brief  SPI flash transmit IRQ.
  * @param  sflash :SPI Flash.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_sflash_tx(BSPSFlash sflash)
{
    /* Sample Soft don't use this function. */
}

/*--------------------------------------------------*/
/** 
  * @brief  SPI flash error IRQ.
  * @param  sflash :SPI Flash.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_sflash_err(BSPSFlash sflash)
{
    /* Sample Soft don't use this function. */
}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART reception IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_rx(BSPUsbUart uart)
{
    switch (uart)
    {
    /*--- usb uart 0 ---*/
    case BSP_USB_UART_0:
        uart_io_rx_irq_handler();
        break;
    /*--- usb uart 1 ---*/
    case BSP_USB_UART_1:
        uart_io_rx_irq_handler();
        break;
    /*--- default    ---*/
    default:
        /* no processing */
        break;
    }

}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART transmit IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_tx(BSPUsbUart uart)
{
    switch (uart)
    {
    /*--- usb uart 0 ---*/
    case BSP_USB_UART_0:
        uart_io_tx_irq_handler();
        break;
    /*--- usb uart 1 ---*/
    case BSP_USB_UART_1:
        uart_io_tx_irq_handler();
        break;
    /*--- default    ---*/
    default:
        /* no processing */
        break;
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART error IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_err(BSPUsbUart uart)
{
    switch (uart)
    {
    /*--- usb uart 0 ---*/
    case BSP_USB_UART_0:
        uart_io_err_irq_handler();
        break;
    /*--- usb uart 1 ---*/
    case BSP_USB_UART_1:
        uart_io_err_irq_handler();
        break;
    /*--- default    ---*/
    default:
        /* no processing */
        break;
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  1ms Timer.
  * @param  timer :1ms Timer.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_timer(BSPTimer timer)
{
    /* Sample Soft don't use this function. */
}

/*--------------------------------------------------*/
/** 
  * @brief  DMA transfer error IRQ.
  * @param  -
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_dmac_err(void)
{
    /* Sample Soft don't use this function. */
}

/*--------------------------------------------------*/
/** 
  * @brief  RTC IRQ.
  * @param  -
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_rtc(void)
{
    /* Sample Soft don't use this function. */
}
/*--------------------------------------------------*/
/** 
  * @brief  I2C IRQ.
  * @param  -
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_i2c(void)
{
    /* Sample Soft don't use this function. */
}
/*--------------------------------------------------*/
/** 
  * @brief  ADC SGL IRQ.
  * @param  -
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_adc_sgl(void)
{
    /* Sample Soft don't use this function. */
}

/*--------------------------------------------------*/
/** 
  * @brief  ADC CNT IRQ.
  * @param  -
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_adc_cnt(void)
{
    /* Sample Soft don't use this function. */
}
/*--------------------------------------------------*/
/** 
  * @brief  EXINT IRQ.
  * @param  exi
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_exint(BSPExInt exi)
{
    /* Sample Soft don't use this function. */
}
#ifdef __DEBUG__
/*--------------------------------------------------*/
/** 
  * @brief  Failure Assert.
  * @note   for debug
  */
/*--------------------------------------------------*/
void assert_failed(char *file, int32_t line)
{
    while (1) {
     /*   __NOP(); */
    }
}
#endif

/**
 *  @}
 */ /* End of group SMIF_FLASH_Exported_functions */


/**
 *  @} 
 */ /* End of group SMIF_FLASH */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
