/**
 *******************************************************************************
 * @file    IOSCaddjustment.c
 * @brief   This file provides API functions for IOSC adjustment.
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2020
 *******************************************************************************
 */

#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include <stdlib.h>
#include "IOSCadjustment.h"
#include "trm_t32a.h"
#include "trm.h"

/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup TRMOSC TRMOSC Sample Appli
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TRMOSC_Private_macro TRMOSC Private Macro
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group TRMOSC_Private_macro */


/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TRMOSC_Private_define TRMOSC Private Define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group TRMOSC_Private_define */


/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TRMOSC_Private_define TRMOSC Private Define
 *  @{
 */
#ifdef TRMOSC_LOSC
#define CAPTUREVAL10MHZ        (9765)          /*!< Caputer value @10MHz */
#else
#define CAPTUREVAL10MHZ        (10416)         /*!< Caputer value @10MHz */
#endif
#define FIXEDPOINT             (100)          /*!< fixed point 2 */
#define SETRIMSTEP             (7)            /*!< trimming step 0.07% * 100 * FIXEDPOINT        */
#define MARGINADJUST           (20)           /*!< Adjustment margin 0.2% * 100 * FIEXDPOINT   */
#define TRIMSETLIMIT           (0x0000007FUL)  /*!< Setting limit */
#define TRIMSETLIMIT_M         (0x00000100UL)  /*!< Setting limit */


#define CFG_IHOSC_WARM_UP_TIME  ((uint32_t)(5000))      /*!< Warm up time(us).  Minimam warm-up time is 100(us). */
#define CFG_IHOSC_CLOCK         ((uint32_t)(10000000))  /*!< Clock(hz).        */
#define WU_TIME_INT_MIN         ((uint32_t)0x000003C0)  /* minimum warm-up time for INT */
#define WU_TIME_INT_MAX         ((uint32_t)0x0000FFFF)  /* maximum warm-up time for INT */
/* IHOSC1EN */
#define CGOSCCR_IHOSC1EN_MASK           ((uint32_t)0x00000001)    /*!< IHOSC1EN  :Mask                  */
#define CGOSCCR_IHOSC1EN_RW_DISABLE     ((uint32_t)0x00000000)    /*!< IHOSC1EN  :[R/W] :Disable        */
#define CGOSCCR_IHOSC1EN_RW_ENABLE      ((uint32_t)0x00000001)    /*!< IHOSC1EN  :[R/W] :Enable         */
/* WUPT  */
#define CGWUPHCR_WUPT_HIGH_MASK         ((uint32_t)0xFFF00000)    /*!< WUPT    :High Bit Mask.  */
/* WUCLK */
#define CGWUPHCR_WUCLK_MASK             ((uint32_t)0x00000100)    /*!< WUCLK   :Mask.           */
#define CGWUPHCR_WUCLK_RW_IHOSC         ((uint32_t)0x00000000)    /*!< WUCLK   :[R/W]  :IHOSC   */
#define CGWUPHCR_WUCLK_RW_EHOSC         ((uint32_t)0x00000100)    /*!< WUCLK   :[R/W]  :EHOSC   */
/* WUEF  */
#define CGWUPHCR_WUEF_MASK              ((uint32_t)0x00000002)    /*!< WUEF    :Mask.           */
#define CGWUPHCR_WUEF_R_DONE            ((uint32_t)0x00000000)    /*!< WUEF    :[R]    :Done    */
#define CGWUPHCR_WUEF_R_RUNNING         ((uint32_t)0x00000002)    /*!< WUEF    :[R]    :Running */
/** 
 *  @}
 */ /* End of group TRMOSC_Private_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TRMOSC_Private_define TRMOSC Private Define
 *  @{
 */
/* no define */

/** 
 *  @}
 */ /* End of group TRMOSC_Private_define */



/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TRMOSC_Private_typedef TRMOSC Private Typedef
 *  @{
 */

/* no define */

/**
 *  @}
 */ /* End of group TRMOSC_Private_typedef */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TRMOSC_Private_variables TRMOSC Private Variables
 *  @{
 */
static IOSCADJ_TypeDef IoscAdjust;
static TRM_ControlTypeDef AdjustData;
static int32_t  difference;

/**
 *  @}
 */ /* End of group TRMOSC_Private_variables */


/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TRMOSC_Private_fuctions TRMOSC Private Fuctions
 *  @{
 */
static void MeasureWave(IOSCADJ_TypeDef *Measure);
static int32_t CalcDiff(IOSCADJ_TypeDef *Diff);
static int32_t CalcAdjVal(IOSCADJ_TypeDef *Adjust);
static int32_t GetAdjustmentValue(void);

/*--------------------------------------------------*/
/** 
  * @brief  Measurement Waveform
  * @param  Measure  :TRMOSC adjustment Structure
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void MeasureWave(IOSCADJ_TypeDef *Measure)
{
    StartCapture();
    Measure->MeasureVal = GetCaptureData();
    StopCapture();
}

/*--------------------------------------------------*/
/** 
  * @brief  Calculation of difference
  * @param  Diff  :TRMOSC adjustment Structure
  * @retval TRMOSC_DONE, TRMOSC_ERROR
  * @note   -
  */
/*--------------------------------------------------*/
static int32_t CalcDiff(IOSCADJ_TypeDef *Diff)
{
    uint32_t absdiff;
    int32_t retval;

    Diff->MeasureDiff = (int32_t)((100*FIXEDPOINT)-((Diff->MeasureVal * (100*FIXEDPOINT)) / CAPTUREVAL10MHZ));
/**/
    absdiff = (uint32_t)abs(Diff->MeasureDiff);
    if (absdiff <= MARGINADJUST) 
    {
        retval = TRMOSC_DONE;
    }
    else 
    {
        retval = TRMOSC_ERROR;
    }

    return (retval);
}

/*--------------------------------------------------*/
/** 
  * @brief  Calculation of adjustment value
  * @param  Adjust  :TRMOSC adjustment Structure
  * @retval TRMOSC_RUN, TRMOSC_ERROR
  * @note   -
  */
/*--------------------------------------------------*/
static int32_t CalcAdjVal(IOSCADJ_TypeDef *Adjust)
{
    int32_t retval;
    uint32_t trim;
    uint32_t adjust_step;

    adjust_step = (uint32_t)(abs(Adjust->MeasureDiff)/SETRIMSTEP);
    /* adjust step (at least 1)check */
    if(adjust_step == 0)
    {
       adjust_step += 1; 
    }

    /* plus diretion */
    if(Adjust->MeasureDiff > 0)
    {
        difference = (adjust_step*SETRIMSTEP)+GetAdjustmentValue();
    }
    /* minus direction */
    else
    {
        difference = -(adjust_step*SETRIMSTEP)+GetAdjustmentValue();
    }
     
    /* adujust value */    
    trim = (uint32_t)(abs(difference)/SETRIMSTEP);
    
    if (difference < 0)
    {
        trim = TRIMSETLIMIT_M-trim;
        if(trim > TRIMSETLIMIT)
        {
            Adjust->Trim = trim;
            retval = TRMOSC_RUN;
        }
        else 
        {
            retval = TRMOSC_ERROR;
        }
    }
    else 
    {
        if(trim <= TRIMSETLIMIT)
        {
            Adjust->Trim = trim;
            retval = TRMOSC_RUN;
        }
        else 
        {
            retval = TRMOSC_ERROR;
        }
    }

    return (retval);
}

/*--------------------------------------------------*/
/** 
  * @brief  Get Adjustment value
  * @param  -
  * @retval Adjustment value
  * @note   -
  */
/*--------------------------------------------------*/
static int32_t GetAdjustmentValue(void)
{
    uint32_t trimvalue;
    int32_t adjvalue;

    if (GetTrimmingControlValue() == TRMOSC_CONTROL_DISABLE) 
    {
        trimvalue = GetInitTrimValue();
    }
    else 
    {
        trimvalue = GetTrimValueSet();
    }
/**/
    if (trimvalue > TRIMSETLIMIT) 
    {
        trimvalue = TRIMSETLIMIT_M - trimvalue;
        adjvalue = -(SETRIMSTEP*trimvalue);
    }
    else 
    {
        adjvalue = SETRIMSTEP*trimvalue;
    }

    return (adjvalue);
}

/**
 *  @}
 */ /* End of group TRMOSC_Private_fuctions */


/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TRMOSC_Exported_functions TRMOSC Exported Functions
 *  @{
 */

/*--------------------------------------------------*/
/** 
  * @brief  Adjustment of internal OSC
  * @param  -
  * @retval TRMOSC_DONE, TRMOSC_ERROR, TRMOSC_RUN
  * @note   -
  */
/*--------------------------------------------------*/
int32_t IOSCAdjustment(void)
{
    int32_t retval;
    uint32_t work;

    MeasureWave(&IoscAdjust);                                           /* Measurement Waveform */
    retval = CalcDiff(&IoscAdjust);                                     /* Calculation of difference */
    if (retval == TRMOSC_ERROR) 
    {
        retval = CalcAdjVal(&IoscAdjust);                               /* Calculation of adjustment value */
        if(retval == TRMOSC_RUN) 
        {
            AdjustData.TrimmingControl     = TRMOSC_CONTROL_ENABLE;
            AdjustData.TrimmingValue = IoscAdjust.Trim;
            SetTrmOsc(&AdjustData);                                     /* Set TRMOSC register */
            /*----------------------*/
            /* Internal HOSC Select */
            /*----------------------*/
            /* [CGWUPHCR]<WUCLK> : IHOSC */
            work = (uint32_t)(TSB_CG->WUPHCR & ~CGWUPHCR_WUCLK_MASK);
            TSB_CG->WUPHCR = (uint32_t)(work | CGWUPHCR_WUCLK_RW_IHOSC);
            /*----------------------*/
            /* Internal HOSC Warm-up Time Set*/
            /*----------------------*/
            /*-----------------------------------------------*/
            /* "1"counter  (s) = 1 / IHOSC                   */
            /* "1"counter (us) = (10^6) / IHOSC              */
            /* "x"counter (us) = time                        */
            /*-----------------------------------------------*/
            /* x : time = 1 : (10^6) / IHOSC                 */
            /*-----------------------------------------------*/
            {
                uint64_t x = (uint64_t)(CFG_IHOSC_WARM_UP_TIME * 10);
                x = ((uint32_t)(((uint64_t)(x - 633) * (CFG_IHOSC_CLOCK / 10) / (uint64_t)(1000000)) - 41));
                if (x > (uint64_t)(WU_TIME_INT_MAX))
                {
                     x = (uint64_t)(WU_TIME_INT_MAX);
                }
                else if(x < (uint64_t)WU_TIME_INT_MIN)
                {
                     x = (uint64_t)(WU_TIME_INT_MIN);
                }
                else{
                    /* do nothing */
                }
                work = (uint32_t)x;
            }
            work  &= (uint32_t)(0xFFFFFFF0);
            work <<= 16;
            work  |= (uint32_t)(TSB_CG->WUPHCR & ~CGWUPHCR_WUPT_HIGH_MASK);
            TSB_CG->WUPHCR = work;
            /* [CGOSCCR]<IHOSC1EN> : HOSC Enable    */
            work = (uint32_t)(TSB_CG->OSCCR & ~CGOSCCR_IHOSC1EN_MASK);
            TSB_CG->OSCCR = (uint32_t)(work | CGOSCCR_IHOSC1EN_RW_ENABLE);
            TSB_CG_WUPHCR_WUON = 1U;
           /* [CGWUPHCR]<WUEF>  : Read(wait for done) */
           while ((TSB_CG->WUPHCR & CGWUPHCR_WUEF_MASK) != CGWUPHCR_WUEF_R_DONE)
           {
                /* no processing */
           }
        }
        else 
        {                                                           /* TRMOSC_ERROR */
            /* do nothing */
        }
    }
    else 
    {                                                               /* TRMOSC_DONE  */
        /* do nothing */
    }
    return (retval);
}

/*--------------------------------------------------*/
/** 
  * @brief  Initialize TRM Function
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void InitTRM(void)
{
    TRM_ControlTypeDef InitData;
    uint32_t work;

    InitData.TrimmingControl     = TRMOSC_CONTROL_DISABLE;
    InitData.TrimmingValue = 0;
    SetTrmOsc(&InitData);

    /*----------------------*/
    /* Internal HOSC Select */
    /*----------------------*/
    /* [CGWUPHCR]<WUCLK> : IHOSC */
    work = (uint32_t)(TSB_CG->WUPHCR & ~CGWUPHCR_WUCLK_MASK);
    TSB_CG->WUPHCR = (uint32_t)(work | CGWUPHCR_WUCLK_RW_IHOSC);
    /*----------------------*/
    /* Internal HOSC Warm-up Time Set*/
    /*----------------------*/
    /*-----------------------------------------------*/
    /* "1"counter  (s) = 1 / IHOSC                   */
    /* "1"counter (us) = (10^6) / IHOSC              */
    /* "x"counter (us) = time                        */
    /*-----------------------------------------------*/
    /* x : time = 1 : (10^6) / IHOSC                 */
    /*-----------------------------------------------*/
    {
        uint64_t x = (uint64_t)(CFG_IHOSC_WARM_UP_TIME * 10);
        x = ((uint32_t)(((uint64_t)(x - 633) * (CFG_IHOSC_CLOCK / 10) / (uint64_t)(1000000)) - 41));
        if (x > (uint64_t)(WU_TIME_INT_MAX))
        {
            x = (uint64_t)(WU_TIME_INT_MAX);
        }
        else if(x < (uint64_t)WU_TIME_INT_MIN)
        {
            x = (uint64_t)(WU_TIME_INT_MIN);
        }
        else{
            /* do nothing */
        }
        work = (uint32_t)x;
    }
    work  &= (uint32_t)(0xFFFFFFF0);
    work <<= 16;
    work  |= (uint32_t)(TSB_CG->WUPHCR & ~CGWUPHCR_WUPT_HIGH_MASK);
    TSB_CG->WUPHCR = work;
    /* [CGOSCCR]<IHOSC1EN> : HOSC Enable    */
    work = (uint32_t)(TSB_CG->OSCCR & ~CGOSCCR_IHOSC1EN_MASK);
    TSB_CG->OSCCR = (uint32_t)(work | CGOSCCR_IHOSC1EN_RW_ENABLE);
    TSB_CG_WUPHCR_WUON = 1U;
    /* [CGWUPHCR]<WUEF>  : Read(wait for done) */
    while ((TSB_CG->WUPHCR & CGWUPHCR_WUEF_MASK) != CGWUPHCR_WUEF_R_DONE)
    {
        /* no processing */
    }
}

/**
 *  @}
 */ /* End of group TRMOSC_Exported_functions */

/**
 *  @}
 */ /* End of group TRMOSC */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
