/** 
 *******************************************************************************
 * @file    main.c
 * @brief   TSSI_ChToCh Sample Application.
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2020
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif
/**
 *  @addtogroup Sample_Appli
 *  @{
 */

/** 
 *  @addtogroup TSSI_ChToCh
 *  @{
 */

/** 
 *  @defgroup MAIN_PRIVATE  MAIN Private
 *  @brief    MAIN Private Group.
 *  @{
 */

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "txz_sample_def.h"
#include "txz_tssi.h"
#include "bsp.h"
#include "bsp_uart_io.h"
/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/* no define */

/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @}
 */ /* End of group Timer_EVENT_COUNT_Private_define */
#define MAIN_NULL          ((void *)0)     /*!< NULL. */
#define USER_ID_UART       ((uint32_t)1)   /*!< User ID :USB UART (Any value) */
#define USER_ID_TSSI_TX    ((uint32_t)2)   /*!< User ID :TSSI (Any value) */
#define USER_ID_TSSI_RX    ((uint32_t)3)   /*!< User ID :TSSI (Any value) */
/** 
 *  @defgroup TSSI_IO_SendMax  TSSI IO Send Max.
 *  @brief    Size of send max. (byte)
 *  @{
 */
#define TSSI_IO_SEND_MAX       ((uint32_t)8)
/**
 *  @}
 */ /* End of group TSSI_IO_SendMax */

/** 
 *  @defgroup TSSI_IO_ReceiveMax  TSSI IO Receive Max.
 *  @brief    Size of receive max. (byte)
 *  @{
 */
#define TSSI_IO_RECEIVE_MAX    ((uint32_t)8)
/**
 *  @}
 */ /* End of group TSSI_IO_ReceiveMax */
 
 /** 
 *  @defgroup TSSI_IO_FRAME_SIZE  TSSI IO Frame size.
 *  @brief    Size of TSSI IO Frame. regster: +1 (bit)
 *  @{
 */
#define TSSI_IO_FRAME_SIZE    ((uint32_t)31)
/**
 *  @}
 */ /* End of group TSSI_IO_FRAME_SIZE */
 
 
 /** 
 *  @defgroup TSSIxCPR_DIV  TSSI clock division
 *  @brief    TSSI clock division.(SCLK=fsyxm/(DIV+1))
 *  @{
 */
#define TSSIxCPR_DIV_VALUE    ((uint32_t)95)
/**
 *  @}
 */ /* End of group TSSIxCPR_DIV */
 
/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/*----------------------------------*/
/** 
 * @enum   Processing
 * @brief  Processing mode.
*/
/*----------------------------------*/
typedef enum
{
    PROC_IDLE,              /*!< Idle.                        */
    PROC_OUTPUT_MSG,        /*!< Output first message.        */
    PROC_INPUT,             /*!< Wait to input.               */
    PROC_COMMAND_PARSE,       /*!< Output Echo back data.       */
    PROC_WRITE_EXEC,       /*!< Output Echo back data.       */
    PROC_READ_EXEC,       /*!< Output Echo back data.       */
    PROC_OUTPUT_ERR,        /*!< Output input error message.  */
    PROC_ERROR              /*!< Error.                       */
} Processing;

/*----------------------------------*/
/** 
 * @enum   SendState
 * @brief  Send state.
*/
/*----------------------------------*/
typedef enum {
    SEND_IDLE = 0U,     /*!< Idle                   */
    SEND_RUN,           /*!< Transmiting.           */
    SEND_SUCCESS,       /*!< Transmission complete. */
    SEND_FAILURE        /*!< Transmission error.    */
} SendState;
/*----------------------------------*/
/** 
 * @enum   ReceiveState
 * @brief  Receive state.
*/
/*----------------------------------*/
typedef enum {
    RECV_IDLE = 0U,     /*!< Idle                   */
    RECV_RUN,           /*!< Receiving              */
    RECV_SUCCESS,       /*!< Transmission complete. */
    RECV_FAILURE        /*!< Received error.        */
} ReceiveState;

/** 
 * @struct send_t
 * @brief  Send information.
*/
/*----------------------------------*/
typedef struct
{
    volatile SendState  state;                          /*!< Status.                     */
    uint32_t            wp;                             /*!< Write pointer.              */
    uint32_t            rp;                             /*!< Read pointer(Now, no use).  */
    uint8_t             buff[TSSI_IO_SEND_MAX];         /*!< Send data.                  */
} send_t;

/*----------------------------------*/
/** 
 * @struct receive_t
 * @brief  Receive information.
*/
/*----------------------------------*/
typedef struct
{
    volatile ReceiveState state;                        /*!< Status.                     */
    volatile uint32_t     wp;                           /*!< Write pointer.              */
    volatile uint32_t     rp;                           /*!< Read pointer(Now, no use).  */
    volatile uint32_t     length;                       /*!< Num of receive data.        */
    volatile uint8_t      buff[TSSI_IO_RECEIVE_MAX];    /*!< Receive data.               */
} receive_t;

/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/*----------------------------------*/
/** 
 * @struct input_t
 * @brief  Input data information.
*/
/*----------------------------------*/
typedef struct
{
    char    data[(32+1)];  /*!< Input data(+ for NULL 1byte). */
    uint8_t wp;                             /*!< Write pointer.                */
} input_t;
/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/*----------------------------------*/
/** 
 * @brief Instance Information.
*/
/*----------------------------------*/
typedef struct
{
    tssi_t      tssi_tx_obj;               /*!< Application :TSSI. */
    tssi_t      tssi_rx_obj;               /*!< Application :TSSI. */
    gpio_t      gpio_obj;                  /*!< Application :GPIO. */
} instance_t;

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
static instance_t instance; /*!< Instance.                    */
static Processing proc;     /*!< Processing mode.        */
static input_t    input;        /*!< Input data information. */
static uint8_t write_data[10] = "TOSHIBA"; /*!< Write data buffer. */
static uint8_t read_data[UART_IO_RECEIVE_MAX];  /*!< read data buffer. */
static receive_t    receive;  /*!< Receive information.  */
static tssi_transmit_t t_info;
static uint8_t rx_buffer[TSSI_IO_RECEIVE_MAX];
send_t       send;                               /*!< Send information.     */
/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
static void variable_initialize(void);
static void clear_data(void);
static void transmit_handler(uint32_t id, TXZ_Result result);
static void receive_handler(uint32_t id, TXZ_Result result, tssi_receive_t *p_info);
static void tssi_io_initialize(void);
static TXZ_Result application_initialize(void);
/*--------------------------------------------------*/
/** 
  * @brief  Variable Initialize
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void variable_initialize(void)
{
    /* Instance. */
    instance.tssi_tx_obj.p_instance    = MAIN_NULL;
    instance.tssi_rx_obj.p_instance    = MAIN_NULL; 
    instance.gpio_obj.p_pd_instance    = MAIN_NULL;
    instance.gpio_obj.p_pu_instance    = MAIN_NULL;
}

/*--------------------------------------------------*/
/** 
  * @brief  Clear input data.
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void clear_data(void)
{
    uint8_t i;

    for (i=0; i<(UART_IO_RECEIVE_MAX+1); i++)
    {
        input.data[i] = 0;
    }
    input.wp = 0;

    for (i=0; i<UART_IO_RECEIVE_MAX; i++)
    {
        read_data[i] = 0;
    }
}

/*--------------------------------------------------*/
/** 
  * @brief      Transmit handler.
  * @param[in]  p_obj   :Object address.
  * @param[in]  result  :Result.
  * @retval     -
  * @note       Called by tssi driver
  */
/*--------------------------------------------------*/
static void transmit_handler(uint32_t id, TXZ_Result result)
{
    /*------------------------------*/
    /*  Result Check                */
    /*------------------------------*/
    if (result == TXZ_SUCCESS)
    {
        send.state = SEND_SUCCESS;
    }
    else
    {
        send.state = SEND_FAILURE;
    }
}
/*--------------------------------------------------*/
/** 
  * @brief      Receive handler.
  * @param[in]  p_obj   :Object address.
  * @param[in]  result  :Result.
  * @retval     -
  * @note       Called by tssi driver
  */
/*--------------------------------------------------*/
static void receive_handler(uint32_t id, TXZ_Result result, tssi_receive_t *p_info)
{
    /*------------------------------*/
    /*  Result Check                */
    /*------------------------------*/
    if (result == TXZ_SUCCESS)
    {
        uint32_t i;
        uint8_t *src_ptr = p_info->rdr0.p_data;
        for(i=0; i<p_info->rdr0.num; ++i)
        {
            if(receive.wp > TSSI_IO_RECEIVE_MAX)
            {
                break;
            }
            receive.buff[receive.wp++] = *src_ptr++;
        }
        receive.state = RECV_SUCCESS;
    }
    else
    {
        receive.state = RECV_FAILURE;
    }
}
/**
 *  @}
 */ /* End of group TSSI_IO_PRIVATE */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/*--------------------------------------------------*/
/** 
  * @brief  Initialize
  * @param  -
  * @retval -
  * @pre    -
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result tssi_port_initialize(void)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*------------------------------*/
    /*  Port initialize             */
    /*  TSSITx:  TSSI0_TCK  PD2     */
    /*           TSSI0_TFS  PD3     */
    /*           TSSI0_TXD  PD4     */
    /*  TSSIRx:  TSSI1_RXD  PU5     */
    /*           TSSI1_RFS  PU6     */
    /*           TSSI1_RCK  PU7     */
    /*------------------------------*/
    /* Register Allocation */
    instance.gpio_obj.p_pd_instance = TSB_PD;
    /*--- Initialize ---*/
    if (gpio_init(&instance.gpio_obj, GPIO_PORT_D) != TXZ_SUCCESS)
    {
      result = TXZ_ERROR;
    }
    /*  PD2  */
    if(gpio_func(&instance.gpio_obj,GPIO_PORT_D,GPIO_PORT_2,(uint32_t)GPIO_PD2_TSSI0TCK,GPIO_PIN_INOUT)!= TXZ_SUCCESS)
    {
      result = TXZ_ERROR;
    }
    /*   PD3  */
    if(gpio_func(&instance.gpio_obj,GPIO_PORT_D,GPIO_PORT_3,(uint32_t)GPIO_PD3_TSSI0TFS,GPIO_PIN_INOUT)!= TXZ_SUCCESS)
    {
      result = TXZ_ERROR;
    }
    /*  PD4 */
    if(gpio_func(&instance.gpio_obj,GPIO_PORT_D,GPIO_PORT_4,(uint32_t)GPIO_PD4_TSSI0TXD,GPIO_PIN_OUTPUT)!= TXZ_SUCCESS)
    {
      result = TXZ_ERROR;
    }

    /* Register Allocation */
    instance.gpio_obj.p_pu_instance = TSB_PU;
    /*--- Initialize ---*/
    if (gpio_init(&instance.gpio_obj, GPIO_PORT_U) != TXZ_SUCCESS)
    {
      result = TXZ_ERROR;
    }
    /*  PU5  */
    if(gpio_func(&instance.gpio_obj,GPIO_PORT_U,GPIO_PORT_5,(uint32_t)GPIO_PU5_TSSI1RXD,GPIO_PIN_INPUT)!= TXZ_SUCCESS)
    {
      result = TXZ_ERROR;
    }
    /*  PU6  */
    if(gpio_func(&instance.gpio_obj,GPIO_PORT_U,GPIO_PORT_6,(uint32_t)GPIO_PU6_TSSI1RFS,GPIO_PIN_INOUT)!= TXZ_SUCCESS)
    {
      result = TXZ_ERROR;
    }
    /*  PU7 */
    if(gpio_func(&instance.gpio_obj,GPIO_PORT_U,GPIO_PORT_7,(uint32_t)GPIO_PU7_TSSI1RCK,GPIO_PIN_INOUT)!= TXZ_SUCCESS)
    {
      result = TXZ_ERROR;
    }

    return result;
}

/*--------------------------------------------------*/
/** 
  * @brief  Initialize
  * @param  -
  * @retval -
  * @pre    -
  * @note   -
  */
/*--------------------------------------------------*/
void tssi_io_initialize(void)
{
    /*------------------------------*/
    /*  Clear RAM                   */
    /*------------------------------*/
    send.state      = SEND_IDLE;
    send.wp         = 0;
    send.rp         = 0;
    receive.state   = RECV_RUN;     /* Auto run. */
    receive.wp      = 0;
    receive.rp      = 0;
    receive.length  = 0;
}

/*--------------------------------------------------*/
/** 
  * @brief  Application Initialize.
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result application_initialize(void)
{
   TXZ_Result result = TXZ_SUCCESS;

    /*----------------------*/
    /* Usb Uart             */
    /*----------------------*/
    /* Tx & Rx use same UART channel. */
    {
        uart_io_initial_setting_t param;

        param.id         = USER_ID_UART;
        switch (bsp_get_usb_uart_tx_ch(BSP_USB_UART_0))
        {
            case 0:
                 param.p_reg  = TSB_UART0;
                 break;
            case 1:
                 param.p_reg  = TSB_UART1;
                 break;
            case 2:
                 param.p_reg  = TSB_UART2;
                 break;
            default:
                 param.p_reg  = MAIN_NULL;
                 break;
        }
        param.boudrate   = bsp_get_usb_uart_tx_baudrate(BSP_USB_UART_0);
        param.irq.tx     = bsp_get_usb_uart_tx_nvic(BSP_USB_UART_0);
        param.irq.rx     = bsp_get_usb_uart_rx_nvic(BSP_USB_UART_0);
        param.irq.err    = bsp_get_usb_uart_err_nvic(BSP_USB_UART_0);
        if (param.p_reg != MAIN_NULL)
        {
            result  = uart_io_initialize(&param);
        }
        else
        {
            result = TXZ_ERROR;
        }
    }
    /*----------------------*/
    /* TSSI             */
    /*----------------------*/
    /* Tx TSSI channel(master transmit). */
    {
        instance.tssi_tx_obj.init.id  = USER_ID_TSSI_TX;
        /* transmit use ch0 */
        instance.tssi_tx_obj.p_instance  = TSB_TSSI0;
        /* Set Initial parameter of TSSIxCPR */
        instance.tssi_tx_obj.init.cpr.div = TSSIxCPR_DIV_VALUE;
        /* Set Initial parameter of TSSIxTCMR */
        instance.tssi_tx_obj.transmit.info.tcmr.tstart = TSSITCMR_TSTART_FIFO;
        instance.tssi_tx_obj.transmit.info.tcmr.tckout = TSSITCMR_TCKOUT_TRANS;
        instance.tssi_tx_obj.transmit.info.tcmr.tcksel = TSSITCMR_TCKSEL_SCLK;
        /* Set Initial parameter of TSSIxTDMACR */
        instance.tssi_tx_obj.transmit.info.tdmacr.tdmae = TSSITDMACR_TDMAE_DISABLE;
        /* Set Initial parameter of TSSIxTIER */
        instance.tssi_tx_obj.transmit.info.tier.tfurie = TSSITIER_TFURIE_DISABLE;
        instance.tssi_tx_obj.transmit.info.tier.tforie = TSSITIER_TFORIE_DISABLE;
        instance.tssi_tx_obj.transmit.info.tier.tftfie = TSSITIER_TFTFIE_ENABLE;
        instance.tssi_tx_obj.transmit.info.tier.tfnfie = TSSITIER_TFNFIE_DISABLE;
        /* Set Initial parameter of TSSIxTFMR */
        instance.tssi_tx_obj.transmit.info.tfmr.tdfs   = TSSI_IO_FRAME_SIZE;

        /* Set Initial parameter of TSSIxTFTLR */
        instance.tssi_tx_obj.transmit.info.tftlr.tfthd = TSSITFTLR_TFTHD_MIN;

        /* Set Transmit handler              */
        instance.tssi_tx_obj.transmit.handler = transmit_handler;
        if ( instance.tssi_tx_obj.p_instance != MAIN_NULL)
        {
            if (tssi_init(&instance.tssi_tx_obj) != TXZ_SUCCESS)
            {
                result = TXZ_ERROR;
            }
        }
        else
        {
            result = TXZ_ERROR;
        }
    }
    /* Rx TSSI channel(slave receive). */
    {
        instance.tssi_rx_obj.init.id  = USER_ID_TSSI_RX;
        /* receive use ch1  */
        instance.tssi_rx_obj.p_instance  = TSB_TSSI1;
        /* Set Initial parameter of TSSIxRCMR */
        instance.tssi_rx_obj.receive.info.rcmr.rstart = TSSIRCMR_RSTART_RFS;
        instance.tssi_rx_obj.receive.info.rcmr.rckout = TSSIRCMR_RCKOUT_INPUT;
        instance.tssi_rx_obj.receive.info.rcmr.rcksel = TSSIRCMR_RCKSEL_RCK;
        /* Set Initial parameter of TSSIxRDMACR */
        instance.tssi_rx_obj.receive.info.rdmacr.rdmae = TSSIRDMACR_RDMAE_DISABLE;
        /* Set Initial parameter of TSSIxRIER */
        instance.tssi_rx_obj.receive.info.rier.rfteie = TSSIRIER_RFTEIE_DISABLE;
        instance.tssi_rx_obj.receive.info.rier.rfurie = TSSIRIER_RFURIE_DISABLE;
        instance.tssi_rx_obj.receive.info.rier.rforie = TSSIRIER_RFORIE_DISABLE;
        instance.tssi_rx_obj.receive.info.rier.rcmie  = TSSIRIER_RCMIE_DISABLE;
        instance.tssi_rx_obj.receive.info.rier.rftfie = TSSIRIER_RFTFIE_ENABLE;
        instance.tssi_rx_obj.receive.info.rier.rfneie = TSSIRIER_RFNEIE_DISABLE;
        /* Set Initial parameter of TSSIxRFMR */
        instance.tssi_rx_obj.receive.info.rfmr.rdfs   = TSSI_IO_FRAME_SIZE;
        instance.tssi_rx_obj.receive.info.rftlr.rfthd = 1;
        
        instance.tssi_rx_obj.receive.info.rdr0.p_data = (uint8_t*)rx_buffer;
        instance.tssi_rx_obj.receive.handler = receive_handler;
        if ( instance.tssi_rx_obj.p_instance != MAIN_NULL)
        {
            if (tssi_init(&instance.tssi_rx_obj) != TXZ_SUCCESS)
            {
                result = TXZ_ERROR;
            }
        }
        else
        {
            result = TXZ_ERROR;
        }
    }
    return (result);
}
/**
 *  @}
 */ /* End of group MAIN_PRIVATE */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
int main(void)
{
    int ret;
    /*----------------------*/
    /* Set Processing Mode  */
    /*----------------------*/
    proc = PROC_OUTPUT_MSG;
    /*----------------------*/
    /* Initialize           */
    /*----------------------*/
    /* BSP initialize       */
    bsp_initialize();
    /* Valiable initialization */
    variable_initialize();
    /* Tssi port initialization */
    if(tssi_port_initialize() != TXZ_SUCCESS)
    {
        proc = PROC_ERROR;
    }

    /* Application initialization */
    if(application_initialize() != TXZ_SUCCESS)
    {
        proc = PROC_ERROR;
    }
    tssi_io_initialize();
    
    /*----------------------*/
    /* Main Processing      */
    /*----------------------*/
    while(1)
    {
        switch(proc)
        {
            /*--- Output "First Message" ---*/
            case PROC_OUTPUT_MSG:
                  {
                     /* Clear input data.     */
                     clear_data();
                     /* Output first message. */
                     if (printf("command > ") != -1 )
                     {
                           proc = PROC_INPUT;
                     }
                     else
                     {
                           proc = PROC_ERROR;
                     }
                  }
                  break;
            /*--- Wait to Input ---*/
            case PROC_INPUT:
                  {
                    int data;
                    
                    data = getchar();
                    if ((data != -1) && (input.wp < UART_IO_RECEIVE_MAX))
                    {
                          input.data[input.wp] = (char)data;
                          input.wp++;
                          switch ((char)data)
                          {
                          /*--- C.dataR: carriage return ---*/
                          case '\r':
                                /* no process */
                                break;
                          /*--- LF: line feed ---*/
                          case '\n':
                                proc = PROC_COMMAND_PARSE;
                                break;
                          /*--- default ---*/
                          default:
                                /* no process */
                                break;
                          }
                    }
                    else
                    {
                          proc = PROC_OUTPUT_ERR;
                    }
                  }
                  break;
            /*--- Output "Echo Back Data" ---*/
            case PROC_COMMAND_PARSE:
                  {
                    if ((strncmp((const char *)&input.data[0], "write", 5)) == 0)
                    {
                          proc = PROC_WRITE_EXEC;
                          break;
                    }
                    else if ((strncmp((const char *)&input.data[0], "read", 4)) == 0)
                    {
                          if(receive.state == RECV_SUCCESS)
                          {
                                uint32_t i;
                                volatile uint8_t *src_ptr = receive.buff;
                                uint8_t *dst_ptr = read_data;
                                
                                for(i=0;i<TSSI_IO_RECEIVE_MAX;++i)
                                {
                                      *dst_ptr++ = *src_ptr++;
                                }
                          }
                          (void)printf("read data > ");
                          if(ret != TXZ_ERROR)   {
                                (void)printf("%s", (const char*)&read_data[0]);
                          }
                          (void)printf("\n");
                          proc = PROC_OUTPUT_MSG;
                    }   
                    else
                    {
                          proc = PROC_OUTPUT_ERR;
                    }
                  }
                  break;
                  
            case PROC_WRITE_EXEC:
                  {
                    if(send.state == SEND_IDLE){
                          receive.wp = 0;
                          receive.state = RECV_IDLE;

                          /* always receive */
                          ret = tssi_slave_receiver(&instance.tssi_rx_obj);

                          /* Send Data Set */
                          t_info.tdr0.p_data = write_data;
                          t_info.tdr0.num = TSSI_IO_SEND_MAX;
                          /* Transfer Start */
                          ret = tssi_master_transfer(&instance.tssi_tx_obj, &t_info);
                          send.state = SEND_RUN;
                          if(ret == TXZ_SUCCESS)
                          {
                              /* NVIC Enable (TX CH0) */
                              NVIC_EnableIRQ(INTTSSI0TX_IRQn);
                              
                              /* NVIC Enable (RX CH1) */
                              NVIC_EnableIRQ(INTTSSI1RX_IRQn);
                          }
                          proc = PROC_WRITE_EXEC;                    
                    }
                    else if(send.state == SEND_FAILURE){
                          send.state = SEND_IDLE;
                          proc = PROC_ERROR;
                    }
                    else if(send.state == SEND_SUCCESS) {
                          /* NVIC Disable (TXCH0) */
                          NVIC_DisableIRQ(INTTSSI0TX_IRQn);
                          if(ret == TXZ_ERROR)
                          {
                               proc = PROC_ERROR;
                          }
                          /* wait for disable IRQ */
                          wait(100);
                          send.state = SEND_IDLE;
                          proc = PROC_OUTPUT_MSG;
                    }
                    break;
                }
           /*--- Output "Input Error Message" ---*/
            case PROC_OUTPUT_ERR:
                  {
                    if (printf("\n Command Error !! \n") != -1 )
                    {
                          proc = PROC_OUTPUT_MSG;
                    }
                    else
                    {
                          proc = PROC_ERROR;
                    }
                  }
                  break;
            /*--- Error ---*/
            case PROC_ERROR:
                  /* System Error. Please Debug. */
      #ifdef __DEBUG__
                  assert_failed(MAIN_NULL, 0);
      #endif
                  break;
            /*--- Default ---*/
            default:
                  proc = PROC_ERROR;
                  break;
        }
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  TSSI CH1 receive reception IRQ.
  * @param  -
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_tssi1_rx()
{
   tssi_irq_handler_receive(&instance.tssi_rx_obj);
}

/*--------------------------------------------------*/
/** 
  * @brief  SPI flash transmit IRQ.
  * @param  -
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_tssi0_tx()
{
  tssi_irq_handler_transmit(&instance.tssi_tx_obj);
}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART reception IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_rx(BSPUsbUart uart)
{
    switch (uart)
    {
      /*--- usb uart 0 ---*/
      case BSP_USB_UART_0:
          uart_io_rx_irq_handler();
          break;
      /*--- default    ---*/
      default:
          /* no processing */
          break;
    }

}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART transmit IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_tx(BSPUsbUart uart)
{
    switch (uart)
    {
      /*--- usb uart 0 ---*/
      case BSP_USB_UART_0:
          uart_io_tx_irq_handler();
          break;
      /*--- default    ---*/
      default:
          /* no processing */
          break;
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART error IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_err(BSPUsbUart uart)
{
    switch (uart)
    {
      /*--- usb uart 0 ---*/
      case BSP_USB_UART_0:
          uart_io_err_irq_handler();
          break;
      /*--- default    ---*/
      default:
          /* no processing */
          break;
    }
}

#ifdef __DEBUG__
/*--------------------------------------------------*/
/** 
  * @brief  Failure Assert.
  * @note   for debug
  */
/*--------------------------------------------------*/
void assert_failed(char *file, int32_t line)
{
    while (1) {
        __NOP();
    }
}
#endif

/**
 *  @} 
 */ /* End of group TSSI_ChToCh */

/**
 *  @} 
 */ /* End of group Sample_Appli */

#ifdef __cplusplus
}
#endif /* __cplusplus */
